@@ -40,4 +40,7 @@ void screem_ctags_model_set_pathname( ScreemCtagsModel *model,
 					const gchar *pathname );
 gchar *screem_ctags_model_get_pathname( const ScreemCtagsModel *model );
 
+GSList *screem_ctags_model_autocomplete( const ScreemCtagsModel *model,
+					const gchar *mime_type,	const gchar *prefix );
+
 #endif
