@@ -25,7 +25,8 @@ typedef struct  {
 typedef struct {
         GObjectClass parent_class;
 
-	void (*changed)( ScreemMDI *mdi, ScreemPage *current );
+	void (*changed)( ScreemMDI *mdi, ScreemPage *current,
+			ScreemPage *previous );
 	
 } ScreemMDIClass;
 
