@@ -79,6 +79,9 @@ void screem_page_set_lang( ScreemPage *page );
 void screem_page_set_pathname( ScreemPage *page, const gchar *path );
 const gchar *screem_page_get_pathname( ScreemPage *page );
 
+gchar *screem_page_get_short_name( ScreemPage *page );
+gchar *screem_page_get_title_name( ScreemPage *page );
+
 void screem_page_set_mime_type( ScreemPage *page, const gchar *mime_type );
 const gchar *screem_page_get_mime_type( ScreemPage *page );
 
@@ -93,6 +96,8 @@ gboolean screem_page_get_changed( ScreemPage *page );
 gboolean screem_page_load( ScreemPage *page, GError **error );
 gboolean screem_page_revert( ScreemPage *page, GError **error  );
 gboolean screem_page_save( ScreemPage *page, GError **error );
+void screem_page_unload( ScreemPage *page );
+glong screem_page_get_seconds_since_last_save_or_load( ScreemPage *page );
 
 gboolean screem_page_is_file_page( const gchar *path );
 gboolean screem_page_is_mime_type_page( const gchar *mime_type );
@@ -111,8 +116,9 @@ gboolean screem_page_is_dynamic( ScreemPage *page );
 gboolean screem_page_is_markup( ScreemPage *page );
 
 gchar *screem_page_template_path( ScreemPage *page );
-gboolean screem_page_is_template( ScreemPage *page, gchar **tag,
-				gchar **path, GSList **blocks );
+gboolean screem_page_is_template( ScreemPage *page, 
+		const gchar *base,
+		gchar **tag, gchar **path, GSList **blocks );
 
 GtkTreeModel *screem_page_get_model( ScreemPage *page );
 void screem_page_build_model( ScreemPage *page );
