@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1999-2000, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2004, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,8 +26,6 @@
 #include "sites.h"
 #include "frontend.h" /* for fe_*_abort */
 
-#include "ne_md5.h" /* for ne_md5_* */
-
 /* Internal sites code handling */
 
 void file_uploaded(struct site_file *file, struct site *site);
@@ -55,14 +53,9 @@ NE_DEBUG(ch, "%s: local=%s stored=%s, %s%s\n",		\
 #define DEBUG_DUMP_FILE_PROPS(ch, file, site)			\
 DEBUG_DUMP_FILE(ch, file);					\
 if (site->state_method == state_timesize) {			\
-    NE_DEBUG(ch, "Time: %ld/%ld Size: %ld/%ld ",			\
+    NE_DEBUG(ch, "Time: %ld/%ld Size: %" NE_FMT_OFF_T "/%" NE_FMT_OFF_T, \
        file->local.time, file->stored.time,			\
        file->local.size, file->stored.size);			\
-} else {							\
-    char l[33], s[33];						\
-    ne_md5_to_ascii(file->local.checksum, l);			\
-    ne_md5_to_ascii(file->stored.checksum, s);			\
-    NE_DEBUG(ch, "Checksum: %32s/%32s\n", l, s);			\
 }								\
 NE_DEBUG(ch, "ASCII:%c/%c Perms:%03o/%03o\n",			\
        file->local.ascii?'y':'n', file->stored.ascii?'y':'n',	\
@@ -99,13 +92,6 @@ file_find(struct site *site, const char *fname, enum file_type type);
 #define site_leave(site) do {\
     if (!--site->critical) fe_enable_abort(site); } while (0)
 
-/* Compares two files states for a file of given type, belonging to
- * the given site. */
-enum file_diff inline static
-file_compare(const enum file_type type, 
-	     const struct file_state *, const struct file_state *, 
-	     const struct site *site);
-
 struct site_file *
 file_set_stored(enum file_type type, struct file_state *state,
 		struct site *site);
@@ -202,6 +188,10 @@ file_compare(const enum file_type type,
     /* They're both there... compare them properly */
     switch (type) {
     case file_dir:
+        if (site->dirperms) {
+            return (first->mode != second->mode ?
+                    file_changed : file_unchanged);
+        }
 	break;
 
     case file_link:
