@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -36,22 +36,18 @@
 #include <gtk/gtkprogressbar.h>
 
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-authentication-manager.h>
 
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 
+#include <libgnomeui/gnome-thumbnail.h>
+
 #include <time.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-directory.h>
-#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include <libgnomevfs/gnome-vfs-module-callback.h>
 #include <libgnomevfs/gnome-vfs-standard-callbacks.h>
@@ -62,6 +58,9 @@
 
 #include "screem-errors.h"
 
+#include "screem-file.h"
+#include "screem-file-progress-dialog.h"
+
 /* not nice including these, we need access to the ScreemSession
  * so we can keep track of the file selector window size */
 #include "screem-application.h"
@@ -71,80 +70,47 @@
  * provide the ScreemWindow in glade dialogs to get access to the app */
 extern ScreemApplication *app;
 
+static gchar *screem_default_dir = NULL;
+
 gboolean copy_file( const gchar *source, const gchar *dest,
+		GnomeVFSXferOverwriteMode ovr_mode,
 		ScreemFileOp cb, gpointer data )
 {
 	GnomeVFSURI *suri;
 	GnomeVFSURI *duri;
-
-	GnomeVFSHandle *shandle;
-	GnomeVFSHandle *dhandle;
-
-	GnomeVFSOpenMode mode;
 	GnomeVFSResult result;
 
 	gboolean ret = FALSE;
-	gboolean wrote = FALSE;
 
-	gchar buffer[ BUFSIZ + 1 ];
-
-	if( ! overwrite( dest ) ) {
-		return FALSE;
+	if( ovr_mode == GNOME_VFS_XFER_OVERWRITE_MODE_QUERY ) {
+		if( ! overwrite( dest ) ) {
+			return FALSE;
+		}
 	}
+	ovr_mode = GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE;
 
 	suri = gnome_vfs_uri_new( source );
+	if( ! suri ) {
+		return FALSE;
+	}
 	duri = gnome_vfs_uri_new( dest );
-
-	mode = GNOME_VFS_OPEN_READ;
-	result = gnome_vfs_open_uri( &shandle, suri, mode );
-	if( result == GNOME_VFS_OK ) {
-		GnomeVFSFileSize wsize = 0;
-		mode = GNOME_VFS_OPEN_WRITE;
-
-		result = gnome_vfs_open_uri( &dhandle, duri, mode );
-		if( result == GNOME_VFS_ERROR_NOT_FOUND ) {
-			GnomeVFSFileInfo *info;
-			GnomeVFSFileInfoOptions options;
-
-			/* get permissions for source */
-			options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-			info = gnome_vfs_file_info_new();
-			result = gnome_vfs_get_file_info_uri( suri, info, options );
-			if( result == GNOME_VFS_OK ) {
-				result = gnome_vfs_create_uri( &dhandle, duri,
-						       mode, TRUE, 
-						       info->permissions );
-			}
-			gnome_vfs_file_info_unref( info );
+	if( ! duri ) {
+		gnome_vfs_uri_unref( suri );
+		return FALSE;
+	}
+	
+	result = gnome_vfs_xfer_uri( suri, duri,
+			GNOME_VFS_XFER_DEFAULT,
+			GNOME_VFS_XFER_ERROR_MODE_ABORT,
+			ovr_mode,
+			NULL,
+			NULL );
+	if( result == GNOME_VFS_OK || result == GNOME_VFS_ERROR_EOF ) {
+		ret = TRUE;
+		if( cb ) {
+			cb( GNOME_VFS_MONITOR_EVENT_CREATED, 
+					dest, data );
 		}
-		while( result == GNOME_VFS_OK ) {
-			GnomeVFSFileSize rsize;
-
-			wrote = TRUE;
-			
-			result = gnome_vfs_read( shandle, 
-						 buffer, BUFSIZ, &rsize );
-			buffer[ rsize ] = '\0';
-			
-			if( result == GNOME_VFS_OK ) {
-				result = gnome_vfs_write( dhandle, buffer, rsize,
-							  &rsize );
-				wsize += rsize;
-			} 
-		}
-		if( result == GNOME_VFS_ERROR_EOF ) {
-			ret = TRUE;
-		}
-
-		if( wrote ) {
-			gnome_vfs_truncate_handle( dhandle, wsize );
-			gnome_vfs_close( dhandle );
-			if( cb ) {
-				cb( GNOME_VFS_MONITOR_EVENT_CREATED,
-						dest, data );
-			}
-		}
-		gnome_vfs_close( shandle );
 	}
 
 	gnome_vfs_uri_unref( duri );
@@ -153,8 +119,8 @@ gboolean copy_file( const gchar *source, const gchar *dest,
 	return ret;
 }
 
-gboolean copy_dir( const gchar *source, const gchar *dest, gboolean move,
-		ScreemFileOp cb, gpointer data )
+gboolean copy_dir( const gchar *source, const gchar *dest, 
+		gboolean move, ScreemFileOp cb, gpointer data )
 {
 	gchar *orig;
 	gchar *new_path;
@@ -221,7 +187,9 @@ gboolean copy_dir( const gchar *source, const gchar *dest, gboolean move,
 				if( move ) {
 					move_file( orig, new_path, cb, data );
 				} else {
-					copy_file( orig, new_path, cb, data );
+					copy_file( orig, new_path, 
+						GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
+						cb, data );
 				}
 			} else {
 				copy_dir( orig, new_path, move, cb, data );
@@ -261,14 +229,12 @@ gboolean move_file( const gchar *source, const gchar *dest,
 
 gboolean delete_file( const gchar *file, ScreemFileOp cb, gpointer data )
 {
-	GnomeVFSURI *uri;
 	gboolean ret;
+	GnomeVFSResult result;
 
-	uri = gnome_vfs_uri_new( file );
-
-	ret = (gnome_vfs_unlink_from_uri( uri ) == GNOME_VFS_OK);
-
-	gnome_vfs_uri_unref( uri );
+	ret = FALSE;
+	result = gnome_vfs_unlink( file );
+	ret = ( result == GNOME_VFS_OK );
 
 	if( ret && cb ) {
 		cb( GNOME_VFS_MONITOR_EVENT_DELETED, file, data );
@@ -456,14 +422,17 @@ gchar *relative_path( const gchar *text, const gchar *root )
 	}
 	
 	/* up and bp should now be at the point where they differ */
-	temp = g_string_new( NULL );
 	if( bp ) {
 		gint parents;
-		for( parents = g_slist_length( bp ); parents > 0; -- parents ) {
+
+		parents = g_slist_length( bp );
+		temp = g_string_sized_new( parents * strlen( "../" ) );
+		
+		for( ; parents > 0; -- parents ) {
 			g_string_append( temp, "../" );
 		}
 	} else {
-		g_string_append( temp, "./" );
+		temp = g_string_new( "./" );
 	}
 
 	for( ; up; up = up->next ) {
@@ -579,7 +548,6 @@ gboolean uri_accessible( const gchar *filename,
 
 gboolean overwrite( const gchar *filename )
 {
-	GtkWidget *prompt;
 	GnomeVFSResult result;
 
 	gboolean ret;
@@ -589,53 +557,50 @@ gboolean overwrite( const gchar *filename )
 	if( ! ret ) {
 		ret = TRUE;
 	} else if( result == GNOME_VFS_OK ) {
-		gchar *glade_path;
-		GladeXML *xml;
-		GString *str;
-		const gchar *ltxt;
-		GtkWidget *label;
-		const gchar *temp;
+		GtkWidget *widget;
+		GtkWidget *area;
+		GtkWidget *button;
+		GList *children;
+		GtkWidget *img;
+		gchar *temp;
 		gchar *tmp;
-		guint pos;
+		gchar *basename;
 
-		glade_path = screem_get_glade_path();
-		xml = glade_xml_new( glade_path, "overwrite", NULL );
-		g_free( glade_path );
-	
-		label = glade_xml_get_widget( xml, "overwrite_label" );
-		ltxt = gtk_label_get_text( GTK_LABEL( label ) );
-		temp = strstr( ltxt, "%s" );
-		g_assert( temp );
-		pos = (guint)(temp - ltxt);
-		str = g_string_new( "<span weight=\"bold\" size=\"larger\">" );
-		pos += str->len;
-		g_string_append( str, ltxt );
-	
-		g_string_erase( str, pos, 2 );
 		temp = gnome_vfs_unescape_string_for_display( filename );
-		tmp = g_markup_escape_text( temp, -1 );
-		g_string_insert( str, pos, tmp );
-		g_free( (gchar*)temp );
+		basename = g_path_get_basename( temp );
+		tmp = g_markup_escape_text( basename, -1 );
+		g_free( basename );
+		g_free( temp );
+				
+		widget = gtk_message_dialog_new( NULL, 
+			GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
+			GTK_BUTTONS_NONE,
+			_( "A file named \"%s\" already exists." ),
+			tmp );
 		g_free( tmp );
-	
-		temp = strstr( str->str, "\n\n" );
-		g_assert( temp );
-		pos = (guint)(temp - str->str);
-		g_string_insert( str, pos, "</span>" );
-
-		gtk_label_set_markup( GTK_LABEL( label ), str->str );
-		g_string_free( str, TRUE );
-	
-		prompt = glade_xml_get_widget( xml, "overwrite" );
-		gtk_window_set_wmclass( GTK_WINDOW( prompt ),
+		
+		gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( widget ), _( "Do you want to replace it with the one you are saving" ) );
+		gtk_dialog_add_buttons( GTK_DIALOG( widget ),
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				_( "_Replace existing file" ),
+				GTK_RESPONSE_OK,
+				NULL );
+		area = GTK_DIALOG( widget )->action_area;
+		children = gtk_container_get_children( GTK_CONTAINER( area ) );
+		button = GTK_WIDGET( children->data );
+		img = gtk_image_new_from_stock( GTK_STOCK_REFRESH,
+				GTK_ICON_SIZE_BUTTON );
+		gtk_button_set_image( GTK_BUTTON( button ),
+				img );
+		
+		gtk_window_set_wmclass( GTK_WINDOW( widget ),
 					"Screem",
 					"overwrite_prompt" );
-		gtk_window_set_type_hint( GTK_WINDOW( prompt ), 
+		gtk_window_set_type_hint( GTK_WINDOW( widget ), 
 					  GDK_WINDOW_TYPE_HINT_DIALOG );
-		ret = ( gtk_dialog_run( GTK_DIALOG( prompt ) ) == 
+		ret = ( gtk_dialog_run( GTK_DIALOG( widget ) ) == 
 				GTK_RESPONSE_OK );
-		gtk_widget_destroy( prompt );
-		g_object_unref( G_OBJECT( xml ) );
+		gtk_widget_destroy( widget );
 	} else if( result != GNOME_VFS_ERROR_NOT_FOUND ) {
 		/* FIXME: report error reason */
 		ret = FALSE;
@@ -662,240 +627,105 @@ gchar* paths_match( const gchar *base_uri,
 	return abs_path1;
 }
 
-void load_file_internal_cancel( GtkWidget *widget )
-{
-	g_object_set_data( G_OBJECT( widget ), "cancel",
-			GUINT_TO_POINTER( 1 ) );
-}
-
-static GString *load_file_internal( GnomeVFSHandle *handle, 
-				   const gchar *path,
-				   GnomeVFSFileSize size,
-				   gboolean interactive,
-				   GnomeVFSResult *res )
-{
-	GString *ret;
-	GnomeVFSResult result;
-	GnomeVFSFileSize bytes_read;
-	GnomeVFSFileSize total_read;
-	
-	GladeXML *xml;
-	gchar *glade_path;
-	const gchar *ltxt;
-	const gchar *temp;
-	gchar *tmp;
-	guint pos;
-	GString *str;
-	GtkWidget *widget;
-
-#define READ_CHUNK_SIZE 8192
-	gchar buffer[ READ_CHUNK_SIZE ];
-	gdouble fraction;
-	gboolean pulse;
-	
-	/* shut compiler up, they won't be used uninitialised */
-	xml = NULL;
-	widget = NULL;
-	
-	ret = g_string_new( NULL );
-
-	pulse = ( size == 0 );
-	
-	if( interactive ) {
-		glade_path = screem_get_glade_path();
-		xml = glade_xml_new( glade_path, "loading", NULL );
-		g_free( glade_path );
-
-		widget = glade_xml_get_widget( xml, "text" );
-		ltxt = gtk_label_get_text( GTK_LABEL( widget ) );
-		temp = strstr( ltxt, "%s" );
-		g_assert( temp );
-		pos = (guint)(temp - ltxt);
-		str = g_string_new( "<span weight=\"bold\" size=\"larger\">" );
-		pos += str->len;
-		g_string_append( str, ltxt );
-	
-		g_string_erase( str, pos, 2 );
-		temp = gnome_vfs_unescape_string_for_display( path );
-		tmp = g_markup_escape_text( temp, -1 );
-		g_string_insert( str, pos, tmp );
-		g_free( (gchar*)temp );
-		g_free( tmp );
-	
-		temp = strstr( str->str, "\n\n" );
-		g_assert( temp );
-		pos = (guint)(temp - str->str);
-		g_string_insert( str, pos, "</span>" );
-	
-		gtk_label_set_markup( GTK_LABEL( widget ), str->str );
-		g_string_free( str, TRUE );
-	
-		widget = glade_xml_get_widget( xml, "loading" );
-		gtk_window_set_wmclass( GTK_WINDOW( widget ),
-					"Screem",
-					"loading_dialog" );
-		gtk_window_set_type_hint( GTK_WINDOW( widget ), 
-					  GDK_WINDOW_TYPE_HINT_DIALOG );
-
-		gtk_widget_show( widget );
-		glade_xml_signal_autoconnect( xml );
-
-		widget = glade_xml_get_widget( xml, "progress" );
-	}
-	
-	total_read = bytes_read = 0;
-	fraction = 0.0;
-	do {
-		result = gnome_vfs_read( handle, buffer,
-				READ_CHUNK_SIZE, &bytes_read );
-		
-		if( result == GNOME_VFS_OK || 
-		    result == GNOME_VFS_ERROR_EOF ) {
-			/* gzip method never gives EOF, assume
-			 * 0 bytes is EOF,
-			 *
-			 * see http://bugzilla.gnome.org/show_bug.cgi?id=41173
-			 *
-			 * */
-			if( bytes_read == 0 ) {
-				result = GNOME_VFS_ERROR_EOF;
-			}
-			if( interactive ) {
-				total_read += bytes_read;
-
-				if( ! pulse ) {
-					fraction = total_read / ( size * 1.0 );
-					gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( widget ), fraction );
-				} else {
-					gtk_progress_bar_pulse( GTK_PROGRESS_BAR( widget ) );
-				}
-				gdk_threads_leave();
-				while( g_main_iteration( FALSE ) ) {}
-				
-				gdk_threads_enter();
-
-				if( g_object_get_data( G_OBJECT( widget ),
-							"cancel" ) ) {
-					result = GNOME_VFS_ERROR_CANCELLED;
-				}
-			}
-			ret = g_string_append_len( ret, buffer, 
-						   bytes_read );
-		}
-	} while( result == GNOME_VFS_OK );
-
-	if( result != GNOME_VFS_ERROR_EOF ) {
-		g_string_free( ret, TRUE );
-		ret = NULL;
-	} else {
-		result = GNOME_VFS_OK;
-	}
-	
-	if( interactive ) {
-		widget = glade_xml_get_widget( xml, "loading" );
-		gtk_widget_destroy( widget );
-		g_object_unref( G_OBJECT( xml ) );
-	}
-	
-	if( res ) {
-		*res = result;
-	}
-	
-	return ret;
-}
-
-GString *load_file( const gchar *path, gboolean *compress,
+gchar *load_file( const gchar *path, gboolean *compress,
 		gchar **charset, GError **error )
 {
-	GnomeVFSResult result;
-	GnomeVFSFileInfo *info;
-	GnomeVFSHandle *handle;
-	gboolean interactive;
-
-	gchar *gzpath;
-	GnomeVFSFileSize size;
-	gboolean local;
-	gboolean compressed;
+	gchar *ret;
+	ScreemFile *file;
+	ScreemFileProgressDialog *dialog;
 	gchar *temp;
+	guint64 size;
+	const gchar *mime;
 	
-#define FILE_SIZE_FOR_DIALOG ( 8 * 1024000 )
-	
-	GString *ret;
+	file = screem_file_new_with_uri( path );
+	dialog = screem_file_progress_dialog_new( file );
 
-	if( error ) {
-		*error = NULL;
-	}
-	if( ! compress ) {
-		compress = &compressed;
-	}
-	
-	ret = NULL;
-	
-	info = gnome_vfs_file_info_new();
-	
-	gzpath = g_strconcat( path, "#ugzip:", NULL );
-	
-	handle = NULL;
-	result = gnome_vfs_open( &handle, gzpath, GNOME_VFS_OPEN_READ );
-	*compress = TRUE;
-	if( result != GNOME_VFS_OK ) {
-		result = gnome_vfs_open( &handle, path, GNOME_VFS_OPEN_READ );
-		*compress = FALSE;
-	}
-	if( result == GNOME_VFS_OK ) {
-		result = gnome_vfs_get_file_info_from_handle( handle, 
-				info,
-				GNOME_VFS_FILE_INFO_DEFAULT );
-		size = 0;
-		local = FALSE;
-		if( result == GNOME_VFS_OK ) {
-			local = GNOME_VFS_FILE_INFO_LOCAL( info );
-			if( info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE )  {
-				size = info->size;
-			}
-		}
-		gnome_vfs_file_info_unref( info );
-	
-		interactive = ( ( size > FILE_SIZE_FOR_DIALOG ) ||
-				! local );
-		ret = load_file_internal( handle, path, size,
-					interactive, &result );
+	screem_file_load( file );
+	screem_file_progress_dialog_run( dialog );
+
+	if( compress ) {
+		*compress = (screem_file_get_compression_type( file ) != SCREEM_FILE_UNCOMPRESSED );
 	}
 
-	if( handle ) {
-		gnome_vfs_close( handle );
-	}
+	ret = (gchar*)screem_file_get_data( file );
 	
-	/* if we have a result convert it to UTF-8 if possible */
 	if( ret ) {
-		temp = screem_support_charset_convert( ret->str, 
-				ret->len, charset );
-		g_string_assign( ret, temp );
-	} else if( error ) {
-		*error = g_error_new_literal( g_quark_from_string( SCREEM_LOAD_FILE_ERR ),
-				(gint)result,
-				gnome_vfs_result_to_string( result ) );
+		size = screem_file_get_size( file );
+
+		/* FIXME: this will spot utf-16, use the default
+		 * screem charset, or the default system charset,
+		 * it would be nice to look for a <meta> which
+		 * gives the charset info, but that only works for
+		 * html files */
+		mime = screem_file_get_mime_type( file );	
+		
+		if( mime && ! strcmp( mime, "text/html" ) ) {
+			/* look for meta */
+
+		}
+		
+		temp = screem_support_charset_convert( ret,
+				size, charset );
+
+		ret = temp;
+	} else {
+		ret = NULL;
+		if( error ) {
+			*error = screem_file_get_error( file );
+		}
 	}
 	
-	g_free( gzpath );
-	
+	g_object_unref( dialog );
+	g_object_unref( file );
+
 	return ret;
 }
 
 gboolean save_file( const gchar *path, const gchar *data,
 		    GnomeVFSFilePermissions perms,
-		    gboolean compress,
+		    gboolean compress, gboolean create_backup,
 		    GError **error )
 {
+	GConfClient *client;
+	
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
+	GnomeVFSHandle *rhandle;
 	GnomeVFSFileSize wsize;
 	gint len;
-	gchar *gzpath;
+
+	GnomeVFSFileInfo *orig;
+	GnomeVFSSetFileInfoMask orig_mask;
+
+	gchar *dir;
+	gchar *base;
+	GnomeVFSFileInfo *info;
+	gboolean can_write_dir;
+	gchar *backup_uri;
+	const gchar *ext;
+	gchar *tmp;
+	gboolean is_tmp_dir;
+	gboolean renamed;
+	gboolean restore;
+	GnomeVFSResult restore_res;
+	gchar buffer[ 8192 ];
+	GnomeVFSFileSize vfssize;
+	uid_t user;
+	GnomeVFSSetFileInfoMask mask;
 	
 	gboolean ret;
 
+	/* AND requested backup creation state with user preference */	
+	client = gconf_client_get_default();
+	create_backup &= gconf_client_get_bool( client,
+			"/apps/screem/general/create_backups",
+			NULL );
+	g_object_unref( client );
+
+	
+	ret = FALSE;
+	
 	if( ! data ) {
 		data = "";
 	}
@@ -903,35 +733,154 @@ gboolean save_file( const gchar *path, const gchar *data,
 	if( error ) {
 		*error = NULL;
 	}
-	
-	if( ! compress ) {
-		uri = gnome_vfs_uri_new( path );
-	} else {
-		gzpath = g_strconcat( path, "#gzip:", NULL );
-		uri = gnome_vfs_uri_new( gzpath );
-		g_free( gzpath );
-	}
 
-	/* FIXME: make a backup of the original first
-	 * to avoid any data loss if the save fails? */
-	
-	if( uri ) {
-		len = strlen( data );
-		wsize = 0;
-		
-		result = gnome_vfs_create_uri( &handle, uri,
-					       GNOME_VFS_OPEN_WRITE,
-					       FALSE, perms );
-	}
+	info = orig = NULL;
+	dir = base = backup_uri = NULL;
+	is_tmp_dir = FALSE;
 	
+	uri = gnome_vfs_uri_new( path );
+	/* can't proceed with an invalid uri */
 	if( ! uri ) {
 		ret = FALSE;
 		result = GNOME_VFS_ERROR_INVALID_URI;
-	} else if( result != GNOME_VFS_OK ) {
+		goto out;
+	}
+	
+	/* no need for backups if the file doesn't already exist,
+	 * not using uri_exists() here as we want the info as well */
+	orig = gnome_vfs_file_info_new();
+	result = gnome_vfs_get_file_info( path, orig,
+			       GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
+				GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS );
+	if( result == GNOME_VFS_OK ) {
+		/* file already exists */
+		orig_mask = GNOME_VFS_SET_FILE_INFO_OWNER;
+		if( orig->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS ) {
+			orig_mask |= GNOME_VFS_SET_FILE_INFO_PERMISSIONS;
+		}
+		/* follow symlink, is this correct behaviour ? */	
+		if( GNOME_VFS_FILE_INFO_SYMLINK( orig ) ) {
+			path = orig->symlink_name;
+			gnome_vfs_uri_unref( uri );
+			uri = gnome_vfs_uri_new( path );
+			if( ! uri ) {
+				ret = FALSE;
+				result = GNOME_VFS_ERROR_INVALID_URI;
+				goto out;
+			}
+		}
+		
+	} else {
+		gnome_vfs_file_info_unref( orig );
+		orig = NULL;
+		create_backup = FALSE;
+	}
+
+	/* can we write to the directory? */
+	dir = gnome_vfs_uri_to_string( uri, GNOME_VFS_URI_HIDE_NONE );
+	tmp = strrchr( dir, '/' );
+	if( tmp ) {
+		*tmp = '\0';
+	
+		info = gnome_vfs_file_info_new();
+		result = gnome_vfs_get_file_info( dir, info,
+				GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS );
+	}
+	if( ( ! tmp ) || result != GNOME_VFS_OK ) {
+		ret = FALSE;
+		goto out;
+	}
+	can_write_dir =( ( info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS ) && ( info->permissions & GNOME_VFS_PERM_ACCESS_WRITABLE ) );
+	/* can we set the necessary permissions? */
+	if( can_write_dir && orig ) {
+		result = gnome_vfs_set_file_info( path, orig, orig_mask );
+	}
+	
+	ext = "~";
+	base = gnome_vfs_uri_extract_short_name( uri );
+	renamed = FALSE;
+	if( orig &&
+	    ( ( ! create_backup ) || ( ! can_write_dir ) ) ) {
+		/* can't write to the directory, or we don't
+		 * want a backup to be kept, make a backup copy
+		 * in TMPDIR, this gets erased if we
+		 * successfully save.*/
+
+		backup_uri = screem_create_tmp_file( base, ext, NULL );
+
+		tmp = g_strconcat( base, "-XXXXXX", NULL );
+		backup_uri = screem_create_tmp_file( tmp, NULL, NULL );
+		g_free( tmp );
+		
+		is_tmp_dir = TRUE;
+		result = GNOME_VFS_OK;	
+		if( ! copy_file( path, backup_uri, 
+				GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
+				NULL, NULL ) ) {
+			/* failed to create backup */
+			result = GNOME_VFS_ERROR_IO;
+		}
+	} else if( orig && create_backup ) {
+		/* create backup by renaming path */
+		tmp = g_strconcat( base, ext, NULL );
+		backup_uri = g_build_filename( dir, tmp, NULL );
+
+		/* erase if it already exists */
+		result = gnome_vfs_unlink( backup_uri );
+		if( result == GNOME_VFS_OK || 
+		    result == GNOME_VFS_ERROR_NOT_FOUND ) {
+			g_free( orig->name );
+			orig->name = tmp;
+
+			result = gnome_vfs_set_file_info( path, orig,
+					GNOME_VFS_SET_FILE_INFO_NAME );
+			/* restore orig base name */	
+			g_free( tmp );
+			orig->name = g_strdup( base );
+
+			renamed = TRUE;
+			perms = orig->permissions;
+		}
+	}
+
+	if( result != GNOME_VFS_OK ) {
+		/* failed to create backup */
+		ret = FALSE;
+		g_set_error( error,
+			g_quark_from_string( SCREEM_SAVE_BACKUP_ERR ),
+			(gint)result,
+			gnome_vfs_result_to_string( result ) );
+		goto out;
+	}
+	
+	/* add gzip mode if necessary - NOTE gzip method doesn't support
+	 * file creation currently so this will cause create to fail */
+	if( compress ) {
+		gnome_vfs_uri_append_string( uri, "#gzip" );
+	}
+
+	len = strlen( data );
+	wsize = 0;
+	
+	/* if the file didn't exist, or we renamed it to create the
+	 * backup we need to create/recreate the file */
+	if( renamed || ! orig ) {
+		result = gnome_vfs_create_uri( &handle, uri,
+					       GNOME_VFS_OPEN_WRITE,
+					       FALSE, perms );
+	} else {
+		/* not creating backup, or backup was a copy,
+		 * open and truncate */
+		result = gnome_vfs_open_uri( &handle, uri,
+				GNOME_VFS_OPEN_WRITE );
+	}
+	restore = FALSE;
+	if( result != GNOME_VFS_OK ) {
 		ret = FALSE;
 	} else {
+		/* truncate to 0 */
 		while( len > 0 && result == GNOME_VFS_OK ) {
-			GnomeVFSFileSize vfssize;
+			restore = TRUE;
 			
 			result = gnome_vfs_write( handle, data, len,
 						  &vfssize );
@@ -941,16 +890,136 @@ gboolean save_file( const gchar *path, const gchar *data,
 		}
 		gnome_vfs_truncate_handle( handle, wsize );
 		gnome_vfs_close( handle );
+		ret = ( result == GNOME_VFS_OK );
+	}
+	
+	/* set original file permissions on new file */
+	if( ret ) {
+		mask = GNOME_VFS_SET_FILE_INFO_PERMISSIONS |
+			GNOME_VFS_SET_FILE_INFO_OWNER;
+		if( renamed ) {
+			/* set original perms */
+			gnome_vfs_set_file_info( path, orig, mask );
+		} else if( orig ) {
+			user = orig->uid;
 
-		gnome_vfs_uri_unref( uri );
+			/* set backup file perms correctly,
+			 *
+			 * owned by current user, group as original */
+			orig->uid = getuid();
+			if( gnome_vfs_set_file_info( backup_uri, orig,
+						mask ) != GNOME_VFS_OK ) {
+				/* failed to set perms, set group perms
+				 * to same as other perms */
+				perms = orig->permissions;
+				orig->permissions = ( perms & ( GNOME_VFS_PERM_USER_ALL | GNOME_VFS_PERM_OTHER_ALL ) ) | ( ( perms & GNOME_VFS_PERM_OTHER_ALL ) << 3 );
+				mask = GNOME_VFS_SET_FILE_INFO_PERMISSIONS;	
+				gnome_vfs_set_file_info( backup_uri, 
+						orig, mask );
+				orig->permissions = perms;
+			}
+			
+			orig->uid = user;
+		}
+		goto out;
+	}
+	if( ! orig || ! restore ) {
+		goto out;
+	}
+	
+	/* restore from backup */
 
-		ret = TRUE;
+	restore_res = GNOME_VFS_OK;
+	if( renamed ) {
+		/* erase if it already exists */
+		restore_res = gnome_vfs_unlink( path );
+		if( restore_res == GNOME_VFS_OK ) {
+			restore_res = gnome_vfs_set_file_info( backup_uri, 
+					orig,
+					GNOME_VFS_SET_FILE_INFO_NAME );
+		}
+	} else {
+		/* truncate path, open backup_uri, write */
+
+		rhandle = NULL;
+		handle = NULL;
+		restore_res = gnome_vfs_open( &handle, path,
+				GNOME_VFS_OPEN_WRITE );
+		if( restore_res == GNOME_VFS_OK ) {
+			restore_res = gnome_vfs_truncate_handle( handle,
+					0 );
+		}
+		if( restore_res == GNOME_VFS_OK ) {
+			restore_res = gnome_vfs_open( &rhandle,
+					backup_uri,
+					GNOME_VFS_OPEN_READ );
+		}
+		if( restore_res == GNOME_VFS_OK ) {
+			while( restore_res == GNOME_VFS_OK ) {
+				vfssize = 0;
+				restore_res = gnome_vfs_read( rhandle,
+						buffer, 8192,
+						&vfssize );
+				if( restore_res == GNOME_VFS_OK || 
+					( restore_res == GNOME_VFS_ERROR_EOF && vfssize > 0 ) ) {
+					restore_res = gnome_vfs_write( handle, buffer, vfssize, &vfssize );
+				}
+			}	
+			if( restore_res == GNOME_VFS_ERROR_EOF ) {
+				restore_res = GNOME_VFS_OK;
+			}
+		}
+		if( handle ) {
+			gnome_vfs_close( handle );
+		}
+		if( rhandle ) {
+			gnome_vfs_close( rhandle );
+		}
+	}
+	if( restore_res != GNOME_VFS_OK && error && ! *error ) {
+		/* failed to restore backup */
+		g_set_error( error,
+			g_quark_from_string( SCREEM_RESTORE_BACKUP_ERR ),
+			(gint)restore_res,
+			"%s\nFailed to restore from backup file: %s\n\nBackup is at %s",
+			gnome_vfs_result_to_string( result ),
+			gnome_vfs_result_to_string( restore_res ),
+			backup_uri);
+
+		/* prevent erasing of backup file if it is in TMPDIR */
+		is_tmp_dir = FALSE;
+	}
+	
+out:
+	if( ( ! ret ) && error && ! *error ) {
+		g_set_error( error,
+			g_quark_from_string( SCREEM_SAVE_FILE_ERR ),
+			(gint)result,
+			gnome_vfs_result_to_string( result ) );
 	}
 
-	if( error && ! ret ) {
-		*error = g_error_new_literal( g_quark_from_string( SCREEM_SAVE_FILE_ERR ),
-				(gint)result,
-				gnome_vfs_result_to_string( result ) );
+	/* erase backups to TMPDIR after we are done */
+	if( is_tmp_dir ) {
+		gnome_vfs_unlink( backup_uri );
+	}
+	
+	if( dir ) {
+		g_free( dir );
+	}
+	if( base ) {
+		g_free( base );
+	}
+	if( backup_uri ) {
+		g_free( backup_uri );
+	}
+	if( orig ) {
+		gnome_vfs_file_info_unref( orig );
+	}
+	if( info ) {
+		gnome_vfs_file_info_unref( info );
+	}
+	if( uri ) {
+		gnome_vfs_uri_unref( uri );
 	}
 	
 	return ret;
@@ -977,12 +1046,13 @@ gboolean screem_uri_is_dir( const gchar *uri )
 }
 
 /* pinched from Epiphany ephy-file-helpers.c + modified */
-gchar *screem_create_tmp_file( const gchar *base, const gchar *ext, gboolean tmpdir )
+gchar *screem_create_tmp_file( const gchar *base, const gchar *ext,
+		const gchar *tmpdir )
 {
 	gint fd;
 	gchar *name;
 
-	if( tmpdir ) {
+	if( ! tmpdir ) {
 		const gchar *tmp;
 
 		tmp = g_getenv( "TMPDIR" );
@@ -995,7 +1065,7 @@ gchar *screem_create_tmp_file( const gchar *base, const gchar *ext, gboolean tmp
 		}
 		name = g_build_filename( tmp, base, NULL );
 	} else {
-		name = g_strdup( base );
+		name = g_build_filename( tmpdir, base, NULL );
 	}
 
 	fd = g_mkstemp( name );
@@ -1122,132 +1192,62 @@ GSList *screem_vfs_scandir( const gchar *dir,
 	return ret;
 }
 
-
-/* vfs callbacks */
-static void screem_vfs_full_auth_cb( gconstpointer in,
-				gsize in_size,
-				gpointer out,
-				gsize out_size,
-				gpointer data )
+static void screem_fileselect_update_preview( GtkFileChooser *file_chooser,
+	gpointer data )
 {
-	const GnomeVFSModuleCallbackFullAuthenticationIn *full_in;
-	GnomeVFSModuleCallbackFullAuthenticationOut *full_out;
+	GnomeThumbnailFactory *factory;
+	GtkWidget *preview;
+	const gchar *href;
+	GnomeVFSFileInfo *info;
+	GdkPixbuf *pixbuf;
+	gchar *uri;
 
-	full_in = (GnomeVFSModuleCallbackFullAuthenticationIn*) in;
-	full_out = (GnomeVFSModuleCallbackFullAuthenticationOut*) out;
+	factory = g_object_get_data( G_OBJECT( file_chooser ), "thumb_factory" );
+	preview = GTK_WIDGET( data );
+	href = gtk_file_chooser_get_preview_uri( file_chooser );
+	info = gnome_vfs_file_info_new();
 	
-}
-
-static void screem_vfs_fill_auth_cb( gconstpointer in,
-				gsize in_size,
-				gpointer out,
-				gsize out_size,
-				gpointer data )
-{
-	const GnomeVFSModuleCallbackFillAuthenticationIn *fill_in;
-	GnomeVFSModuleCallbackFillAuthenticationOut *fill_out;
-
-	fill_in = (GnomeVFSModuleCallbackFillAuthenticationIn*) in;
-	fill_out = (GnomeVFSModuleCallbackFillAuthenticationOut*) out;
-
-}
-
-static void screem_vfs_save_auth_cb( gconstpointer in,
-				gsize in_size,
-				gpointer out,
-				gsize out_size,
-				gpointer data )
-{
-	const GnomeVFSModuleCallbackSaveAuthenticationIn *save_in;
-	GnomeVFSModuleCallbackSaveAuthenticationOut *save_out;
-
-	save_in = (GnomeVFSModuleCallbackSaveAuthenticationIn*) in;
-	save_out = (GnomeVFSModuleCallbackSaveAuthenticationOut*) out;
-
-}
-#if 0
-static void screem_vfs_auth_cb( gconstpointer in,
-				gsize in_size,
-				gpointer out,
-				gsize out_size,
-				gpointer data )
-{
-
-}
-
-static void screem_vfs_proxy_auth_cb( gconstpointer in,
-				gsize in_size,
-				gpointer out,
-				gsize out_size,
-				gpointer data )
-{
-
-}
-#endif
-void screem_init_vfs_callbacks( void )
-{
-	static const struct {
-		const gchar *type;
-		GnomeVFSModuleCallback cb;
-	} callbacks[] = {
-		{
-			GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION,
-			screem_vfs_full_auth_cb
-		},
-		{
-			GNOME_VFS_MODULE_CALLBACK_FILL_AUTHENTICATION,
-			screem_vfs_fill_auth_cb
-		},
-		{
-			GNOME_VFS_MODULE_CALLBACK_SAVE_AUTHENTICATION,
-			screem_vfs_save_auth_cb
-		},
-/*		{
-			GNOME_VFS_MODULE_CALLBACK_AUTHENTICATION,
-			screem_vfs_auth_cb
-		},
-		{
-			GNOME_VFS_MODULE_CALLBACK_HTTP_PROXY_AUTHENTICATION,
-			screem_vfs_proxy_auth_cb
-		}*/
-	};
-	static const guint n_cbs = G_N_ELEMENTS( callbacks );
-	guint i;
-	
-	
-	for( i = 0; FALSE && i < n_cbs; ++ i ) {
-		gnome_vfs_module_callback_set_default( callbacks[ i ].type,
-				callbacks[ i ].cb,
-				NULL, /* data */
-				NULL );
+	pixbuf = NULL;
+	if( gnome_vfs_get_file_info( href, info, 
+				GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+				GNOME_VFS_FILE_INFO_FOLLOW_LINKS ) ==
+			GNOME_VFS_OK ) {
+			
+		uri = gnome_thumbnail_factory_lookup( factory,
+				href, info->mtime );
+		if( uri ) {
+			pixbuf = gdk_pixbuf_new_from_file( uri, NULL );	
+			g_free( uri );
+		} else {
+			pixbuf = gnome_thumbnail_factory_generate_thumbnail( factory, href, info->mime_type );
+			if( pixbuf ) {
+				gnome_thumbnail_factory_save_thumbnail( factory, pixbuf, href, info->mtime );
+			}
+		}
+		if( pixbuf ) {
+			gtk_image_set_from_pixbuf( GTK_IMAGE( preview ), pixbuf );
+			g_object_unref( pixbuf );
+		}
 	}
-
-	gnome_authentication_manager_init();
+	gnome_vfs_file_info_unref( info );
+	
+	gtk_file_chooser_set_preview_widget_active( file_chooser, pixbuf != NULL );
 }
 
-gchar *screem_fileselect( const gchar *title, GtkWindow *parent,
-			GtkFileChooserAction action,
-			const gchar *dir,
-			const GSList *filters )
+GtkWidget *screem_file_dialog_create( const gchar *title,
+		GtkWindow *parent, GtkFileChooserAction action,
+		gboolean multi,
+		const gchar *dir,
+		const GSList *filters, gboolean preview )
 {
-	ScreemSession *session;
-	
 	GtkWidget *dialog;
-	gchar *ret;
+	GtkWidget *image;
 	const gchar *button;
-	gchar *temp;
 	GtkFileFilter *all;
-
-	gint x;
-	gint y;
-	gint w;
-	gint h;
+	GnomeThumbnailFactory *factory;
 
 	const gchar *role = "screem_file_selector";
 	
-	session = screem_application_get_session( app );
-	ret = NULL;
-
 	button = GTK_STOCK_OPEN;
 	if( action == GTK_FILE_CHOOSER_ACTION_SAVE ||
 	    action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER ) {
@@ -1262,6 +1262,8 @@ gchar *screem_fileselect( const gchar *title, GtkWindow *parent,
 	gtk_window_set_role( GTK_WINDOW( dialog ), role );
 	gtk_file_chooser_set_local_only( GTK_FILE_CHOOSER( dialog ), 
 					FALSE );
+	gtk_file_chooser_set_select_multiple( GTK_FILE_CHOOSER( dialog),
+						multi );
 
 	while( filters ) {
 		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER( dialog ),
@@ -1273,7 +1275,53 @@ gchar *screem_fileselect( const gchar *title, GtkWindow *parent,
 	if( dir ) {
 		gtk_file_chooser_set_current_folder_uri( GTK_FILE_CHOOSER( dialog ), dir );
 	}
+
+	if( preview ) {
+		image = gtk_image_new();
+
+		gtk_file_chooser_set_preview_widget( GTK_FILE_CHOOSER( dialog ),
+			image );
+
+ 		g_signal_connect( GTK_FILE_CHOOSER( dialog ), "update-preview",
+ 			G_CALLBACK( screem_fileselect_update_preview ), image );
+
+		gtk_file_chooser_set_use_preview_label( GTK_FILE_CHOOSER( dialog ),
+			TRUE );
+			
+		factory = gnome_thumbnail_factory_new( GNOME_THUMBNAIL_SIZE_LARGE );
+		g_object_set_data_full( G_OBJECT( dialog ), "thumb_factory",
+			factory, (GDestroyNotify)g_object_unref );
+	}
 	
+	return dialog;
+}
+
+gchar *screem_fileselect( const gchar *title, GtkWindow *parent,
+			GtkFileChooserAction action,
+			const gchar *dir,
+			const GSList *filters, gboolean preview )
+{
+	ScreemSession *session;
+	
+	GtkWidget *dialog;
+	gchar *ret;
+	gchar *temp;
+
+	gint x;
+	gint y;
+	gint w;
+	gint h;
+
+	session = screem_application_get_session( app );
+	ret = NULL;
+
+	if( ! dir ) {
+		dir = screem_default_dir;
+	}
+
+	dialog = screem_file_dialog_create( title, parent,
+			action, FALSE, dir, filters, preview );
+		
 	screem_session_restore_dialog( session, dialog );
 
 	if( gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_ACCEPT ) {
@@ -1288,6 +1336,11 @@ gchar *screem_fileselect( const gchar *title, GtkWindow *parent,
 		}
 	}
 
+	if( dir == screem_default_dir ) {
+		g_free( screem_default_dir );
+		screem_default_dir = gtk_file_chooser_get_current_folder_uri( GTK_FILE_CHOOSER( dialog ) );
+	}
+
 	/* don't use screem_session_store_dialog(), we don't
 	 * want to store the width/height for save dialogs */
 	if( session ) {
@@ -1313,7 +1366,7 @@ gchar *screem_fileselect( const gchar *title, GtkWindow *parent,
 GSList *screem_fileselect_multi( const gchar *title, GtkWindow *parent,
 			GtkFileChooserAction action,
 			const gchar *dir,
-			const GSList *filters )
+			const GSList *filters, gboolean preview )
 {
 	ScreemSession *session;
 
@@ -1321,46 +1374,21 @@ GSList *screem_fileselect_multi( const gchar *title, GtkWindow *parent,
 	GSList *ret;
 	gchar *temp;
 	GSList *tmp;
-	const gchar *button;
-	GtkFileFilter *all;
-
+	
 	gint x;
 	gint y;
 	gint w;
 	gint h;
 	
-	const gchar *role = "screem_file_selector";
-	
 	session = screem_application_get_session( app );
 	ret = NULL;
 
-	button = GTK_STOCK_OPEN;
-	if( action == GTK_FILE_CHOOSER_ACTION_SAVE ||
-	    action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER ) {
-		button = GTK_STOCK_SAVE;
+	if( ! dir ) {
+		dir = screem_default_dir;
 	}
-		
-	dialog = gtk_file_chooser_dialog_new( title, parent,
-				action, 
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				button, GTK_RESPONSE_ACCEPT,
-				NULL );
-	gtk_window_set_role( GTK_WINDOW( dialog ), role );
-	gtk_file_chooser_set_local_only( GTK_FILE_CHOOSER( dialog ), 
-					FALSE );
-	gtk_file_chooser_set_select_multiple( GTK_FILE_CHOOSER( dialog),
-						TRUE );
-	while( filters ) {
-		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER( dialog ),
-						filters->data );
-		filters = filters->next;
-	}
-	all = screem_get_file_filter( _( "All Files" ) );
-	gtk_file_chooser_add_filter( GTK_FILE_CHOOSER( dialog ), all );
-	if( dir ) {
-		gtk_file_chooser_set_current_folder_uri( GTK_FILE_CHOOSER( dialog ), dir );
-	}
-	
+
+	dialog = screem_file_dialog_create( title, parent,
+			action, TRUE, dir, filters, preview );
 	screem_session_restore_dialog( session, dialog );
 
 	if( gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_ACCEPT ) {
@@ -1374,6 +1402,11 @@ GSList *screem_fileselect_multi( const gchar *title, GtkWindow *parent,
 			}
 		}	
 	}
+	
+	if( dir == screem_default_dir ) {
+		g_free( screem_default_dir );
+		screem_default_dir = gtk_file_chooser_get_current_folder_uri( GTK_FILE_CHOOSER( dialog ) );
+	}
 
 	/* don't use screem_session_store_dialog(), we don't
 	 * want to store the width/height for save dialogs */
@@ -1418,3 +1451,37 @@ gint screem_compare_file_age( const gchar *a, const gchar *b )
 	return ( atime - btime );
 }
 
+void screem_gtk_file_chooser_button_show( GtkWidget *widget )
+{
+	ScreemSession *session;
+	
+	session = screem_application_get_session( app );
+	screem_session_restore_dialog( session, widget );
+}
+
+void screem_gtk_file_chooser_button_response( GtkWidget *widget,
+		gint response )
+{
+	ScreemSession *session;
+	GtkFileChooserAction action;
+	gint w;
+	gint h;
+	gint x;
+	gint y;
+
+	action = gtk_file_chooser_get_action( GTK_FILE_CHOOSER( widget ) );
+	session = screem_application_get_session( app );
+	if( action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+	    action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER ) {
+		w = h = -1;
+	} else {
+		gtk_file_chooser_set_preview_widget_active( GTK_FILE_CHOOSER( widget ),
+			FALSE );
+		gtk_window_get_size( GTK_WINDOW( widget ), &w, &h );
+	}
+	gtk_window_get_position( GTK_WINDOW( widget ), &x, &y );
+	screem_session_set_dialog( session, 
+			gtk_window_get_role( GTK_WINDOW( widget ) ),
+			x, y, w, h );
+}
+
