@@ -14,13 +14,12 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #include "config.h"
 
 #include <glib/gi18n.h>
-#include <libgnome/gnome-macros.h>
 
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -31,17 +30,15 @@
 #include <string.h>
 
 #include "screem-ctags-model.h"
-#include "screem-file-browser.h"
 
 #include "readtags.h"
 
-static void screem_ctags_model_class_init( ScreemCtagsModelClass *klass );
-static void screem_ctags_model_instance_init( ScreemCtagsModel *ctags_model );
-static void screem_ctags_model_finalize( GObject *object );
-static void screem_ctags_model_set_prop( GObject *object, guint prop_id,
-				const GValue *value, GParamSpec *spec );
-static void screem_ctags_model_get_prop( GObject *object, guint prop_id,
-				GValue *value, GParamSpec *spec );
+#include "screem-application.h"
+#include "screem-icon-cache.h"
+
+#include "support.h"
+
+extern ScreemApplication *app;
 
 static void screem_ctags_model_pathname_set( ScreemCtagsModel *model );
 static void monitor_cb( GnomeVFSMonitorHandle *handle,
@@ -49,7 +46,7 @@ static void monitor_cb( GnomeVFSMonitorHandle *handle,
 			  const gchar *info_uri,
 			  GnomeVFSMonitorEventType type,
 			  gpointer data );
-static void screem_ctags_model_theme_change( ScreemFileBrowser *browser,
+static void screem_ctags_model_theme_change( ScreemIconCache *cache,
 					gpointer data );
 static gboolean screem_ctags_model_icon_change( GtkTreeModel *model,
 					GtkTreePath *path,
@@ -62,7 +59,7 @@ typedef struct {
 } Levels;
 
 struct ScreemCtagsModelPrivate {
-	ScreemFileBrowser *browser;
+	ScreemIconCache *icache;
 	gchar *pathname;
 	GnomeVFSMonitorHandle *monitor;
 
@@ -71,9 +68,16 @@ struct ScreemCtagsModelPrivate {
 
 	/* file->icon cache */
 	GHashTable *cache;
+	/* file->mime cache */
+	GHashTable *mime;
 
 	tagFile *file;
 	tagFileInfo *info;
+	
+	/* autocomplete */
+	const gchar *mime_type;
+	const gchar *prefix;
+	guint prefix_len;
 };
 
 enum {
@@ -87,9 +91,13 @@ enum {
 
 /*static guint screem_ctags_model_signals[ LAST_SIGNAL ] = { 0 };*/
 	
-GNOME_CLASS_BOILERPLATE( ScreemCtagsModel, screem_ctags_model,
-			  GtkTreeStore,
-			  GTK_TYPE_TREE_STORE )
+G_DEFINE_TYPE( ScreemCtagsModel, screem_ctags_model, GTK_TYPE_TREE_STORE )
+
+static void screem_ctags_model_finalize( GObject *object );
+static void screem_ctags_model_set_prop( GObject *object, guint prop_id,
+				const GValue *value, GParamSpec *spec );
+static void screem_ctags_model_get_prop( GObject *object, guint prop_id,
+				GValue *value, GParamSpec *spec );
 
 static void screem_ctags_model_class_init( ScreemCtagsModelClass *klass )
 {
@@ -113,7 +121,7 @@ static void screem_ctags_model_class_init( ScreemCtagsModelClass *klass )
 }
 
 
-static void screem_ctags_model_instance_init( ScreemCtagsModel *ctags_model )
+static void screem_ctags_model_init( ScreemCtagsModel *ctags_model )
 {
 	ScreemCtagsModelPrivate *priv;
 	GtkTreeStore *store;
@@ -146,8 +154,9 @@ static void screem_ctags_model_instance_init( ScreemCtagsModel *ctags_model )
 	gint i;
 	
 	priv = ctags_model->priv = g_new0( ScreemCtagsModelPrivate, 1 );
-	priv->browser = screem_file_browser_new();
-	g_signal_connect( G_OBJECT( priv->browser ), "theme_change",
+	priv->icache = screem_application_get_icon_cache( app );
+
+	g_signal_connect( G_OBJECT( priv->icache ), "changed",
 			G_CALLBACK( screem_ctags_model_theme_change ),
 			ctags_model );
 	
@@ -166,7 +175,10 @@ static void screem_ctags_model_instance_init( ScreemCtagsModel *ctags_model )
 	priv->cache = g_hash_table_new_full( g_str_hash, g_str_equal,
 			(GDestroyNotify)g_free,
 			(GDestroyNotify)g_object_unref );
-	
+	priv->mime = g_hash_table_new_full( g_str_hash, g_str_equal,
+			(GDestroyNotify)g_free,
+			(GDestroyNotify)g_free );
+		
 	priv->info = g_new0( tagFileInfo, 1 );
 }
 
@@ -175,7 +187,7 @@ static void screem_ctags_model_finalize( GObject *object )
 	ScreemCtagsModel *ctags_model;
 	ScreemCtagsModelPrivate *priv;
 	guint i;
-
+				
 	g_return_if_fail( object != NULL );
 	g_return_if_fail( SCREEM_IS_CTAGS_MODEL( object ) );
 
@@ -191,18 +203,24 @@ static void screem_ctags_model_finalize( GObject *object )
 		gtk_tree_path_free( priv->toplevels[ i ].path );
 	}
 	g_free( priv->toplevels );
+
+	if( priv->cache ) {
+		g_hash_table_destroy( priv->cache );
+	}
+	if( priv->mime ) {
+		g_hash_table_destroy( priv->mime );
+	}
+
+	g_free( priv->pathname );
 	
-	g_object_unref( priv->browser );
-	g_free( priv->pathname );	
 	if( priv->monitor ) {
 		gnome_vfs_monitor_cancel( priv->monitor );
 	}
 	g_free( priv->info );
-	g_hash_table_destroy( priv->cache );
 	
 	g_free( priv );
 	
-	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
+	G_OBJECT_CLASS( screem_ctags_model_parent_class )->finalize( object );
 }
 
 static void screem_ctags_model_set_prop( GObject *object, guint prop_id,
@@ -272,11 +290,12 @@ static gboolean screem_ctags_model_build( gpointer data )
 	const gchar *uname;
 	const gchar *cname;
 	guint i;
+	gchar *mimetype;
 	
 	model = SCREEM_CTAGS_MODEL( data );
 	priv = model->priv;
 	store = GTK_TREE_STORE( data );
-
+	
 	tres = tagsNext( priv->file, &entry ); 
 
 	if( tres != TagSuccess ) {
@@ -305,23 +324,31 @@ static gboolean screem_ctags_model_build( gpointer data )
 			break;
 		}
 	}
-	gtk_tree_model_get_iter( GTK_TREE_MODEL( model ), &pit, path );
+	g_assert( priv->pathname );
+	g_assert( entry.file );
 	ipath = g_build_path( G_DIR_SEPARATOR_S,
 			priv->pathname,
 			entry.file, NULL );
 	icon = g_hash_table_lookup( priv->cache, ipath );
 	if( ! icon ) {
-		icon = screem_file_browser_get_icon( priv->browser,
-			ipath, 16, 16, FALSE, NULL );
+		mimetype = screem_get_mime_type( ipath, FALSE );
+
+		icon = screem_icon_cache_get_pixbuf( priv->icache,
+				ipath, mimetype, 
+				SCREEM_ICON_CACHE_DEFAULT_SIZE );
 		g_hash_table_insert( priv->cache,
 				g_strdup( ipath ),
 				icon );
+		g_hash_table_insert( priv->mime, 
+				g_strdup( ipath ), 
+				mimetype );
 	}
 	/* place in toplevel class */
+	gtk_tree_model_get_iter( GTK_TREE_MODEL( model ), &pit, path );
 	gtk_tree_store_append( store, &it, &pit );
 	gtk_tree_store_set( store, &it,
 			CTAGS_NAME_COL, entry.name,
-			CTAGS_FILE_COL, entry.file,
+			CTAGS_FILE_COL, ipath,
 			CTAGS_ICON_COL, icon,
 			CTAGS_PATTERN_COL, entry.address.pattern,
 			CTAGS_LINE_COL, entry.address.lineNumber,
@@ -367,9 +394,10 @@ static void screem_ctags_model_pathname_set( ScreemCtagsModel *model )
 	store = GTK_TREE_STORE( model );
 	gtk_tree_store_clear( store );
 
-	icon = screem_file_browser_get_icon( priv->browser, "/",
-			16, 16, FALSE, NULL );
-	
+	icon = screem_icon_cache_get_pixbuf( priv->icache,
+				"/", "x-directory/normal",
+				SCREEM_ICON_CACHE_DEFAULT_SIZE );
+
 	for( i = 0; priv->toplevels[ i ].name; ++ i ) {
 		gtk_tree_store_append( store, &it, NULL );
 		gtk_tree_store_set( store, &it,
@@ -432,12 +460,24 @@ static void monitor_cb( GnomeVFSMonitorHandle *handle,
 	}
 }
 
-static void screem_ctags_model_theme_change( ScreemFileBrowser *browser,
+static void screem_ctags_model_theme_change( ScreemIconCache *cache,
 					gpointer data )
 {
+	ScreemCtagsModel *model;
+	ScreemCtagsModelPrivate *priv;
+
+	model = SCREEM_CTAGS_MODEL( data );
+	priv = model->priv;
+
+
+	g_hash_table_destroy( priv->cache );
+	priv->cache = g_hash_table_new_full( g_str_hash, g_str_equal,
+			(GDestroyNotify)g_free,
+			(GDestroyNotify)g_object_unref );
+
 	gtk_tree_model_foreach( GTK_TREE_MODEL( data ),
 			(GtkTreeModelForeachFunc)screem_ctags_model_icon_change,
-			browser );
+			cache );
 }
 
 static gboolean screem_ctags_model_icon_change( GtkTreeModel *model,
@@ -445,23 +485,34 @@ static gboolean screem_ctags_model_icon_change( GtkTreeModel *model,
 					GtkTreeIter *it,
 					gpointer data )
 {
-	ScreemFileBrowser *browser;
+	ScreemIconCache *cache;
 	gchar *file;
 	GdkPixbuf *pixbuf;
-
-	browser = SCREEM_FILE_BROWSER( data );
+	ScreemCtagsModel *cmodel;
+	ScreemCtagsModelPrivate *priv;
+	
+	cache = SCREEM_ICON_CACHE( data );
+	cmodel = SCREEM_CTAGS_MODEL( model );
+	priv = cmodel->priv;
 	
 	gtk_tree_model_get( GTK_TREE_MODEL( model ), it,
 			CTAGS_FILE_COL, &file,
 			-1 );
 	if( ! file ) {
-		pixbuf = screem_file_browser_get_icon( browser, "/",
-							16, 16,
-							FALSE, NULL );
+		pixbuf = screem_icon_cache_get_pixbuf( cache,
+				"/", "x-directory/normal",
+				SCREEM_ICON_CACHE_DEFAULT_SIZE);
 	} else {
-		pixbuf = screem_file_browser_get_icon( browser,
-							file, 16,16,
-							FALSE, NULL );
+		pixbuf = g_hash_table_lookup( priv->cache, file );
+		if( ! pixbuf ) {
+			pixbuf = screem_icon_cache_get_pixbuf( priv->icache,
+				file, NULL,
+				SCREEM_ICON_CACHE_DEFAULT_SIZE );
+			g_object_ref( pixbuf );
+			g_hash_table_insert( priv->cache,
+					g_strdup( file ), 
+					pixbuf );
+		}
 	}
 
 	gtk_tree_store_set( GTK_TREE_STORE( model ), it,
@@ -475,6 +526,49 @@ static gboolean screem_ctags_model_icon_change( GtkTreeModel *model,
 	return FALSE;
 }
 
+static gboolean screem_ctags_file_autocomplete_fill( GtkTreeModel *model,
+		GtkTreePath *path, GtkTreeIter *iter, gpointer data )
+{
+	ScreemCtagsModel *cmodel;
+	ScreemCtagsModelPrivate *priv;
+	GSList **ret = (GSList**)data;
+	const gchar *mime;
+	const gchar *mime_type;
+	const gchar *prefix;
+	guint len;
+	gchar *file;
+	gchar *name;
+
+	gboolean check;
+	
+	cmodel = SCREEM_CTAGS_MODEL( model );
+	priv = cmodel->priv;
+
+	prefix = priv->prefix;
+	len = priv->prefix_len;
+	mime_type = priv->mime_type;
+
+	gtk_tree_model_get( GTK_TREE_MODEL( model ), iter,
+			CTAGS_FILE_COL, &file,
+			CTAGS_NAME_COL, &name,
+			-1 );
+	check = FALSE;
+	if( name && file ) {
+		check = ( ! strncmp( name, prefix, len ) );
+	}
+		
+	if( check ) {
+		mime = g_hash_table_lookup( priv->mime, file );
+
+		if( mime && mime_type && ! strcmp( mime, mime_type ) ) {
+			*ret = g_slist_prepend( *ret, g_strdup( name ) );
+		}
+	}
+	g_free( file );
+	g_free( name );
+
+	return FALSE;
+}
 
 /* public stuff */
 
@@ -506,3 +600,25 @@ gchar *screem_ctags_model_get_pathname( const ScreemCtagsModel *model )
 	return ret;
 }
 
+GSList *screem_ctags_model_autocomplete( const ScreemCtagsModel *model,
+					const gchar *mime_type,	const gchar *prefix )
+{
+	ScreemCtagsModelPrivate *priv;
+	GSList *ret;
+	GtkTreeModelForeachFunc func;
+	
+	g_return_val_if_fail( SCREEM_IS_CTAGS_MODEL( model ), NULL );
+	g_return_val_if_fail( prefix != NULL, NULL );
+
+	priv = model->priv;
+	
+	ret = NULL;
+	priv->prefix = prefix;
+	priv->prefix_len = strlen( prefix );
+	priv->mime_type = mime_type;
+	func = screem_ctags_file_autocomplete_fill;
+	
+	gtk_tree_model_foreach( GTK_TREE_MODEL( model ), func, &ret );
+
+	return ret;			
+}
