@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -33,16 +33,6 @@
 #include "screem-helper.h"
 #include "screem-cvs.h"
 
-static void screem_cvs_class_init( ScreemCVSClass *klass );
-static void screem_cvs_init( ScreemCVS *cvs );
-static void screem_cvs_finalize( GObject *object );
-
-static void screem_cvs_set_property( GObject *object, guint prop_id,
-				     const GValue *value, GParamSpec *pspec );
-static void screem_cvs_get_property( GObject *object, guint prop_id,
-				     GValue *value, GParamSpec *pspec );
-
-
 static void screem_cvs_set_root( ScreemCVS *cvs, const gchar *root );
 static void screem_cvs_set_base_path( ScreemCVS *cvs, const gchar *base );
 
@@ -94,14 +84,14 @@ void screem_cvs_commit( ScreemCVS *cvs, const gchar *pathname,
 		gchar *path;
 
 		path = screem_cvs_usable_pathname( pathname );
-		command = g_strconcat( "cvs", root, " commit ", 
+		command = g_strconcat( "cvs -z3", root, " commit ", 
 				       extra_flags, " -m \"", 
 				       log_text, "\" \"", 
 				       path + cvs->private->len, "\"",
 				       NULL );
 		g_free( path );
 	} else {
-		command = g_strconcat( "cvs", root, " commit ", 
+		command = g_strconcat( "cvs -z3", root, " commit ", 
 				       extra_flags, " -m \"", 
 				       log_text, "\"",
 				       NULL );
@@ -124,13 +114,13 @@ void screem_cvs_update( ScreemCVS *cvs, const gchar *pathname,
 		gchar *path;
 
 		path = screem_cvs_usable_pathname( pathname );
-		command = g_strconcat( "cvs", root, " update ",
+		command = g_strconcat( "cvs -z3", root, " update ",
 				       extra_flags, " \"", 
 				       path + cvs->private->len, "\"",
 				       NULL );
 		g_free( path );
 	} else {
-		command = g_strconcat( "cvs", root, " update ",
+		command = g_strconcat( "cvs -z3", root, " update ",
 				       extra_flags, NULL );
 	}
 	screem_cvs_execute( cvs, command );
@@ -144,13 +134,15 @@ void screem_cvs_delete( ScreemCVS *cvs, const gchar *pathname,
 	gchar *root;
 	gchar *command;
 	gchar *path;
-
+	const gchar *delpath;
+	
 	root = screem_cvs_get_root( cvs );
 
 	path = screem_cvs_usable_pathname( pathname );
-	command = g_strconcat( "cvs", root, " delete ",
-			       extra_flags, " \"", 
-			       path + cvs->private->len, "\"",
+	delpath = path + cvs->private->len;
+		
+	command = g_strconcat( "cvs -z3", root, " delete ",
+			       extra_flags, " \"", delpath, "\"",
 			       NULL );
 	g_free( path );
 	screem_cvs_execute( cvs, command );
@@ -164,15 +156,15 @@ void screem_cvs_add( ScreemCVS *cvs, const gchar *pathname,
 	gchar *root;
 	gchar *command;
 	gchar *path;
+	const gchar *addpath;
 
 	root = screem_cvs_get_root( cvs );
 
 	path = screem_cvs_usable_pathname( pathname );
-	command = g_strconcat( "cvs", root, " add ",
-			       extra_flags, " -m \"", log_text,
-			       "\" \"", 
-			       path + cvs->private->len, "\"",
-			       NULL );
+	addpath = path + cvs->private->len;
+		
+	command = g_strconcat( "cvs -z3", root, " add ",
+			       extra_flags, "\"", addpath, "\"", NULL );
 	g_free( path );
 	screem_cvs_execute( cvs, command );
 	g_free( command );
@@ -187,7 +179,7 @@ void screem_cvs_checkout( ScreemCVS *cvs,
 
 	root = screem_cvs_get_root( cvs );
 
-	command = g_strconcat( "cvs", root, " checkout ",
+	command = g_strconcat( "cvs -z3", root, " checkout ",
 			       extra_flags, " \"", module, "\"",
 			       NULL );
 	screem_cvs_execute( cvs, command );
@@ -204,7 +196,7 @@ void screem_cvs_import( ScreemCVS *cvs,
 
 	root = screem_cvs_get_root( cvs );
 
-	command = g_strconcat( "cvs", root, " import ",
+	command = g_strconcat( "cvs -z3", root, " import ",
 			       extra_flags, " -m \"", log_text, 
 			       "\" \"", module, "\" screem start",
 			       NULL );
@@ -230,14 +222,22 @@ static void screem_cvs_set_root( ScreemCVS *cvs, const gchar *root )
 
 static void screem_cvs_set_base_path( ScreemCVS *cvs, const gchar *base )
 {
+	gchar *basedir;
+	
 	g_assert( base );
 
 	if( cvs->private->base ) {
 		g_free( cvs->private->base );
 		cvs->private->base = NULL;
 	}
-
-	cvs->private->base = screem_cvs_usable_pathname( base );
+	basedir = screem_cvs_usable_pathname( base );
+	if( basedir[ strlen( basedir ) -1 ] != G_DIR_SEPARATOR ) {
+		cvs->private->base = g_strconcat( basedir, G_DIR_SEPARATOR_S,
+			NULL );
+		g_free( basedir );
+	} else {
+		cvs->private->base = basedir;
+	}
 	cvs->private->len = strlen( cvs->private->base );
 }
 
@@ -305,15 +305,20 @@ static gchar *screem_cvs_usable_pathname( const gchar *pathname )
 
 /* G Object stuff */
 
-#define PARENT_TYPE SCREEM_TYPE_HELPER
+G_DEFINE_TYPE( ScreemCVS, screem_cvs, SCREEM_TYPE_HELPER )
+
+static void screem_cvs_finalize( GObject *object );
+static void screem_cvs_set_property( GObject *object, guint prop_id,
+				     const GValue *value, GParamSpec *pspec );
+static void screem_cvs_get_property( GObject *object, guint prop_id,
+				     GValue *value, GParamSpec *pspec );
+
 
-static gpointer parent_class;
 static void screem_cvs_class_init( ScreemCVSClass *klass )
 {
 	GObjectClass *object_class;
 
 	object_class = G_OBJECT_CLASS( klass );
-	parent_class = g_type_class_peek_parent( klass );
 
 	object_class->finalize = screem_cvs_finalize;
 
@@ -359,7 +364,7 @@ static void screem_cvs_finalize( GObject *object )
 
 	g_free( cvs->private );
 
-	G_OBJECT_CLASS( parent_class )->finalize( object );
+	G_OBJECT_CLASS( screem_cvs_parent_class )->finalize( object );
 }
 
 static void screem_cvs_set_property( GObject *object, guint prop_id,
@@ -416,27 +421,3 @@ static void screem_cvs_get_property( GObject *object, guint prop_id,
 	}
 }
 
-GType screem_cvs_get_type()
-{
-	static GType type = 0;
-	
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemCVSClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_cvs_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemCVS ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_cvs_init
-		};
-
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemCVS",
-					       &info, 0 );
-	}
-
-	return type;
-}
