@@ -14,13 +14,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #include "config.h"
 
-#include <libgnome/gnome-macros.h>
-
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include <gconf/gconf-client.h>
@@ -42,16 +40,8 @@
 #include "fileops.h"
 #include "support.h"
 
-static void screem_dtd_db_class_init( ScreemDTDDBClass *klass );
-static void screem_dtd_db_instance_init( ScreemDTDDB *dtd_db );
-static void screem_dtd_db_finalize( GObject *object );
-static void screem_dtd_db_set_prop( GObject *object, guint prop_id,
-				const GValue *value, GParamSpec *spec );
-static void screem_dtd_db_get_prop( GObject *object, guint prop_id,
-				GValue *value, GParamSpec *spec );
-
-static void screem_dtd_load_catalog( ScreemDTDDB *db, 
-				     const GString *pathname );
+static void screem_dtd_load_catalog( ScreemDTDDB *db,
+		const gchar *pathname );
 static void screem_dtd_load_catalogs( ScreemDTDDB *db );
 static gint screem_dtd_db_sort_compare( const gchar *a, const gchar *b);
 static void dtds_insert( gpointer key, gpointer value, GObject *widget);
@@ -73,6 +63,12 @@ static gboolean screem_dtd_db_load_internal( ScreemDTDDB *db,
 					const gchar *publicid,
 					const gchar *systemid );
 
+static void screem_dtd_db_set_hidden( ScreemDTDDB *db, 
+		const gchar *publicid, gboolean hide );
+static void screem_dtd_db_row_changed( GtkTreeModel *model,
+		GtkTreePath *path, GtkTreeIter *iter,
+		ScreemDTDDB *db );
+
 
 struct ScreemDTDDBPrivate {
 	GHashTable *loaded_dtds;
@@ -87,9 +83,13 @@ struct ScreemDTDDBPrivate {
 	guint notify;
 };
 
-GNOME_CLASS_BOILERPLATE( ScreemDTDDB, screem_dtd_db,
-			  GObject,
-			  G_TYPE_OBJECT )
+G_DEFINE_TYPE( ScreemDTDDB, screem_dtd_db, G_TYPE_OBJECT )
+
+static void screem_dtd_db_finalize( GObject *object );
+static void screem_dtd_db_set_prop( GObject *object, guint prop_id,
+				const GValue *value, GParamSpec *spec );
+static void screem_dtd_db_get_prop( GObject *object, guint prop_id,
+				GValue *value, GParamSpec *spec );
 
 static void screem_dtd_db_class_init( ScreemDTDDBClass *klass )
 {
@@ -102,12 +102,13 @@ static void screem_dtd_db_class_init( ScreemDTDDBClass *klass )
 }
 
 
-static void screem_dtd_db_instance_init( ScreemDTDDB *dtd_db )
+static void screem_dtd_db_init( ScreemDTDDB *dtd_db )
 {
 	ScreemDTDDBPrivate *priv;
 	GConfClient *client;
 	gchar *tmp;
 	xmlNodePtr node;
+	xmlDtdPtr dtd;
 	
 	priv = dtd_db->priv = g_new0( ScreemDTDDBPrivate, 1 );
 
@@ -125,32 +126,48 @@ static void screem_dtd_db_instance_init( ScreemDTDDB *dtd_db )
 	priv->loaded_dtds = g_hash_table_new( g_str_hash, g_str_equal );
 
 	tmp = screem_get_dot_dir();
-	priv->overridepath = g_build_filename( tmp, 
-				"dtd_override.xml", NULL );
+	priv->overridepath = g_build_filename( tmp, "doctypes.xml",
+			NULL );
 	g_free( tmp );
 
 	priv->overrides = NULL;
 	if( uri_exists( priv->overridepath, NULL ) ) {
 		priv->overrides = xmlParseFile( priv->overridepath );
 	}
+	if( ! priv->overrides && uri_exists( DATADIR"/screem/doctypes.xml", NULL ) ) {
+		/* no user override file, load system overrides */
+		priv->overrides = xmlParseFile( DATADIR"/screem/doctypes.xml" );
+		if( priv->overrides ) {
+			xmlSaveFile( priv->overridepath, 
+					priv->overrides );
+		}
+	}
 	if( ! priv->overrides ) {
 		/* no overrides file, create a new doc */
-		priv->overrides = xmlNewDoc( XML_DEFAULT_VERSION );
+
+		g_print( "No system or user doctypes file, all dtds will be hidden by default\n" );
+		
+		priv->overrides = xmlNewDoc( (const xmlChar*)XML_DEFAULT_VERSION );
+		dtd = xmlCreateIntSubset( priv->overrides,
+				(const xmlChar*)"doctypes",
+				(const xmlChar*)"-//Screem/DTD doctypes 1.0",
+				(const xmlChar*)"http://www.screem.org/dtd/screem-doctypes.dtd" );
 		node = xmlNewDocNode( priv->overrides,
-					NULL, /* ns */
-					"screemoverride",
-					NULL );
+				NULL, /* ns */ (const xmlChar*)"doctypes", NULL );
 		xmlDocSetRootElement( priv->overrides, node );
 		xmlSaveFile( priv->overridepath, priv->overrides );
 	}
 
 	screem_dtd_load_catalogs( dtd_db );
 
-	priv->store = gtk_list_store_new( 3, 
-				G_TYPE_STRING, 
-				G_TYPE_STRING,
-				G_TYPE_STRING );
-
+	priv->store = gtk_list_store_new( SCREEM_DTD_DB_MAX_COL, 
+			G_TYPE_STRING, /* public */
+			G_TYPE_STRING, /* system */
+			G_TYPE_STRING, /* root */
+			G_TYPE_BOOLEAN /* active */ );
+	g_signal_connect( G_OBJECT( priv->store ), "row_changed",
+			G_CALLBACK( screem_dtd_db_row_changed ),
+			dtd_db );
 	screem_dtd_db_fill_list( dtd_db, priv->store );
 }
 
@@ -179,7 +196,7 @@ static void screem_dtd_db_finalize( GObject *object )
 	ScreemDTDDB *dtd_db;
 	ScreemDTDDBPrivate *priv;
 	GConfClient *client;
-	
+			
 	g_return_if_fail( object != NULL );
 	g_return_if_fail( SCREEM_IS_DTD_DB( object ) );
 
@@ -208,7 +225,7 @@ static void screem_dtd_db_finalize( GObject *object )
 
 	g_free( priv );
 	
-	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
+	G_OBJECT_CLASS( screem_dtd_db_parent_class )->finalize( object );
 }
 
 static void screem_dtd_db_set_prop( GObject *object, guint prop_id,
@@ -243,81 +260,68 @@ static void screem_dtd_db_get_prop( GObject *object, guint prop_id,
 
 /* static stuff */
 
-static void screem_dtd_load_catalog( ScreemDTDDB *db, 
-				     const GString *pathname )
+static void screem_dtd_load_catalog( ScreemDTDDB *db,
+		const gchar *pathname )
 {
-	gchar *buffer;
+	gchar buffer[ BUFSIZ ];
 	FILE *file;
 
-	gchar *id;
-	gchar *path;
+	gchar id[ BUFSIZ ];
+	gchar path[ BUFSIZ ];
 
 	gchar *tid;
 	gchar *tpath;
 	gchar *temp;
 	gint ret;
-	GString *tpathname;
-
-	buffer = g_new0( gchar, BUFSIZ );
-	id = g_new0( gchar, BUFSIZ );
-	path = g_new0( gchar, BUFSIZ );
 
 	/* we have a catalog file */
-	file = fopen( pathname->str, "r" );
+	file = fopen( pathname, "r" );
 	if( ! file )
 		return;
 
 	while( fgets( buffer, BUFSIZ, file ) ) {
-		buffer = g_strchug( buffer );
+		g_strchug( buffer );
 		if( ! strncmp( "CATALOG ", buffer, strlen( "CATALOG " ) ) ) {
 			/* another catalog file to check */
 			buffer[ strlen( buffer) - 1 ] = '\0';
-			tpathname = g_string_new( buffer + strlen( "CATALOG " ) );
-			screem_dtd_load_catalog( db, tpathname );
-			g_string_free( tpathname, TRUE );
-			continue;
-		}
-		if( strncmp( "PUBLIC ", buffer, strlen( "PUBLIC " ) ) )
-			continue;
-		ret = sscanf( buffer, "PUBLIC \"%[^\"]\" \"%[^\"]\"", 
-			      id, path );
-		if( ! ret )
-			break;
-		else if( ret == 1 ) {
-			/* ok try a different match */
-			if( sscanf( buffer, "PUBLIC \"%[^\"]\" %s", 
-				    id, path ) != 2 )
+			screem_dtd_load_catalog( db, buffer + strlen( "CATALOG " ) );
+		} else if( ! strncmp( "PUBLIC ", buffer, strlen( "PUBLIC " ) ) ) {
+			ret = sscanf( buffer, "PUBLIC \"%[^\"]\" \"%[^\"]\"", 
+				      id, path );
+			if( ! ret ) {
 				break;
-		}
+			} else if( ret == 1 ) {
+				/* ok try a different match */
+				if( sscanf( buffer, "PUBLIC \"%[^\"]\" %s", 
+					    id, path ) != 2 )
+					break;
+			}
 		
-		/* is id already in the hash table? */
-		if( ! g_hash_table_lookup( db->priv->dtd_types, id ) ) {
-			/* nope */
-			gchar *tmp;
+			/* is id already in the hash table? */
+			if( ! g_hash_table_lookup( db->priv->dtd_types, id ) ) {
+				/* nope */
+				gchar *tmp;
 			
-			tid = g_strdup( id );
-			tpath = gnome_vfs_escape_host_and_path_string( path );
-			if( ! g_path_is_absolute( path ) ) {
-				
-				tmp = g_path_get_dirname( pathname->str );
-				temp = g_strconcat( "file://", tmp,
-						G_DIR_SEPARATOR_S,
-						tpath, NULL );
-				g_free( tmp );
-			} else {
-				temp = g_strconcat( "file://", tpath,
-						NULL );
+				tid = g_strdup( id );
+				tpath = gnome_vfs_escape_host_and_path_string( path );
+				if( ! g_path_is_absolute( path ) ) {
+					tmp = g_path_get_dirname( pathname );
+					temp = g_strconcat( "file://", 
+							tmp,
+							G_DIR_SEPARATOR_S,
+							tpath, NULL );
+					g_free( tmp );
+				} else {
+					temp = g_strconcat( "file://", 
+							tpath, NULL );
+				}
+				g_free( tpath );
+				tpath = temp;
+				g_hash_table_insert( db->priv->dtd_types, tid, tpath );
 			}
-			g_free( tpath );
-			tpath = temp;
-			g_hash_table_insert( db->priv->dtd_types, tid, tpath );
 		}
 	}
 	fclose( file );
-
-	g_free( buffer );
-	g_free( id );
-	g_free( path );
 }
 
 static void screem_dtd_load_catalogs( ScreemDTDDB *db )
@@ -337,7 +341,7 @@ static void screem_dtd_load_catalogs( ScreemDTDDB *db )
 	};
 
 	gint i;
-	GString *file;
+	gchar *file;
 
 	DIR *dir;
 	struct dirent *entry;
@@ -364,14 +368,12 @@ static void screem_dtd_load_catalogs( ScreemDTDDB *db )
 			    ( ! g_strncasecmp( "catalog.", 
 					       entry->d_name, 
 					       strlen( "catalog." ) ) ) ) {
-				file = g_string_new( paths[ i ] );
-				g_string_append_c( file, 
-						G_DIR_SEPARATOR );
-				g_string_append( file, entry->d_name );
-				
+				file = g_strconcat( paths[ i ],
+						G_DIR_SEPARATOR_S,
+						entry->d_name,
+						NULL );
 				screem_dtd_load_catalog( db, file );
-				
-				g_string_free( file, TRUE );
+				g_free( file );
 			}
 		}
 		closedir( dir );
@@ -457,14 +459,15 @@ static xmlNodePtr screem_dtd_db_lookup_override( ScreemDTDDB *db,
 	if( ret ) {
 		for( ret = ret->children; ret; ret = ret->next ) {
 			val = NULL;
-			if( ! strcmp( "dtd", ret->name ) ) {
-				val = xmlGetProp( ret, "public" ); 
+			if( ! strcmp( "doctype", (gchar*)ret->name ) ) {
+				val = xmlGetProp( ret, 
+						(const xmlChar*)"public" ); 
 			}
-			if( val && ! strcmp( val, publicid ) ) {
-				g_free( val );
+			if( val && ! strcmp( (gchar*)val, publicid ) ) {
+				xmlFree( val );
 				break;
 			}
-			g_free( val );
+			xmlFree( val );
 		}
 	}
 
@@ -483,7 +486,7 @@ static gchar *screem_dtd_db_override_get_root( ScreemDTDDB *db,
 	ret = NULL;
 	override = screem_dtd_db_lookup_override( db, publicid );
 	if( override ) {
-		ret = xmlGetProp( override, "root" );
+		ret = (gchar*)xmlGetProp( override, (const xmlChar*)"root" );
 	}
 
 	return ret;
@@ -501,12 +504,36 @@ static gchar *screem_dtd_db_override_get_system_id( ScreemDTDDB *db,
 	ret = NULL;
 	override = screem_dtd_db_lookup_override( db, publicid );
 	if( override ) {
-		ret = xmlGetProp( override, "systemid" );
+		ret = (gchar*)xmlGetProp( override, (const xmlChar*)"system" );
 	}
 
 	return ret;
 }
 
+static gboolean screem_dtd_db_get_hidden( ScreemDTDDB *db, 
+		const gchar *publicid )
+{
+	xmlChar *attr;
+	xmlNodePtr override;
+	gboolean ret;
+
+	g_return_val_if_fail( SCREEM_IS_DTD_DB( db ), TRUE );
+	g_return_val_if_fail( publicid != NULL, TRUE );
+
+	ret = TRUE;
+	override = screem_dtd_db_lookup_override( db, publicid );
+	if( override ) {
+		attr = xmlGetProp( override, (const xmlChar*)"active" );
+		if( attr ) {
+			ret = (*attr != '1');
+			xmlFree( attr );
+		}
+	}
+
+	return ret;
+
+}
+
 static ScreemDTD *screem_dtd_db_new_dtd( ScreemDTDDB *db,
 					 const gchar *publicid,
 					 const gchar *systemid )
@@ -532,20 +559,23 @@ static ScreemDTD *screem_dtd_db_new_dtd( ScreemDTDDB *db,
 	}
 
 	dtd = NULL;
-	data = screem_dtd_db_resolve_entity( db, publicid,
-					     systemid );
+	data = screem_dtd_db_resolve_entity( db, publicid, systemid );
 	if( data ) {
 		dtd = screem_dtd_new( publicid, systemid );
 
-		g_hash_table_insert( priv->loaded_dtds,
-				     g_strdup( systemid ),
-				     dtd );
 		/* parse data, adding to dtd */
 		parse.dtd = dtd;
 		parse.resolve_entity = screem_dtd_db_resolve_entity;
 		parse.userdata = db;
 		
-		screem_dtd_parse( dtd, &parse, data );
+		if( ! screem_dtd_parse( dtd, &parse, data ) ) {
+			g_object_unref( dtd );
+			dtd = NULL;
+		} else {
+			g_hash_table_insert( priv->loaded_dtds,
+				     g_strdup( systemid ),
+				     dtd );
+		}
 
 		g_free( data );
 	}
@@ -560,7 +590,6 @@ gchar *screem_dtd_db_resolve_entity( void *db,
 	ScreemDTDDB *dtddb;
 	ScreemDTDDBPrivate *priv;
 	const gchar *localuri;
-	GString *file;
 	gchar *ret;
 	GnomeVFSURI *uri;
 	gboolean local;
@@ -570,7 +599,6 @@ gchar *screem_dtd_db_resolve_entity( void *db,
 	dtddb = SCREEM_DTD_DB( db );
 
 	priv = dtddb->priv;
-	file = NULL;
 	localuri = NULL;	
 	if( publicid ) {
 		localuri = g_hash_table_lookup( priv->dtd_types, 
@@ -580,8 +608,8 @@ gchar *screem_dtd_db_resolve_entity( void *db,
 			/* no local copy of publicid, or we haven't
 			   loaded a catalog with it in */
 
-			if( screem_dtd_db_load_internal( dtddb, publicid,
-			   				systemid ) ) {
+			if( screem_dtd_db_load_internal( dtddb, 
+						publicid, systemid ) ) {
 				localuri = g_hash_table_lookup( priv->dtd_types,
 								publicid );
 			}
@@ -602,11 +630,7 @@ gchar *screem_dtd_db_resolve_entity( void *db,
 	ret = NULL;
 
 	if( localuri ) {
-		file = load_file( localuri, NULL, NULL, NULL );
-	}
-	if( file ) {
-		ret = file->str;
-		g_string_free( file, FALSE );
+		ret = load_file( localuri, NULL, NULL, NULL );
 	}
 
 	return ret;
@@ -618,8 +642,8 @@ static gboolean screem_dtd_db_load_internal( ScreemDTDDB *db,
 {
 	ScreemDTDDBPrivate *priv;
 	GnomeVFSURI *uri;
-	GString *file;
-	GString *catfile;
+	gchar *file;
+	gchar *catfile;
 	gboolean local;
 	gchar *filename;
 	gchar *tmp;
@@ -660,45 +684,92 @@ static gboolean screem_dtd_db_load_internal( ScreemDTDDB *db,
 		filename = gnome_vfs_escape_host_and_path_string( tmp );
 		g_free( tmp );
 		
-		if( save_file( filename, file->str,
+		if( save_file( filename, file,
 				GNOME_VFS_PERM_USER_READ |
 				GNOME_VFS_PERM_USER_WRITE,
-				FALSE,
+				FALSE, FALSE,
 				NULL ) ) {
 			catalog = g_build_filename( path, "catalog",
 						    NULL );
-			tmp = g_strdup_printf( "PUBLIC \"%s\" \"%s\"\n",
-						publicid, sysid );
-
 			catfile = load_file( catalog, NULL, NULL, NULL );
-			if( ! catfile ) {
-				catfile = g_string_new( NULL );
+			
+			if( catfile ) {
+				tmp = g_strdup_printf( "%s\nPUBLIC \"%s\" \"%s\"\n",
+						catfile, publicid, 
+						sysid );
+					
+			} else {
+				tmp = g_strdup_printf( "PUBLIC \"%s\" \"%s\"\n",
+						publicid, sysid );
 			}
-			g_string_append( catfile, tmp );
-			save_file( catalog, catfile->str,
+			save_file( catalog, tmp,
 				   GNOME_VFS_PERM_USER_READ |
 				   GNOME_VFS_PERM_USER_WRITE, 
-				   FALSE,
+				   FALSE, FALSE,
 				   NULL );
-			g_string_free( catfile, TRUE );
-
+			g_free( catfile );
 			g_free( tmp );
 			g_free( catalog );
 		}
 		g_free( path );
 		g_free( sysid );
 
-		g_string_free( file, TRUE );
-	
 		g_hash_table_insert( priv->dtd_types,
 				     g_strdup( publicid ),
 				     filename );
 		ret = TRUE;
 	}
+	g_free( file );
 
 	return ret;
 }
 
+static void screem_dtd_db_set_hidden( ScreemDTDDB *db, 
+		const gchar *publicid, gboolean hide )
+{
+	xmlNodePtr droot;
+	xmlNodePtr override;
+	
+	g_return_if_fail( SCREEM_IS_DTD_DB( db ) );
+	g_return_if_fail( publicid != NULL );
+
+	override = screem_dtd_db_lookup_override( db, publicid );
+	if( ! override ) {
+		override = xmlNewNode( NULL, (const xmlChar*)"doctype" );
+		xmlSetProp( override, (const xmlChar*)"public", 
+				(xmlChar*)publicid );
+		droot = xmlDocGetRootElement( db->priv->overrides );
+		xmlAddChild( droot, override );
+	}
+
+	if( ! hide ) {
+		xmlSetProp( override, (const xmlChar*)"active", 
+				(const xmlChar*)"1" );
+	} else {
+		xmlUnsetProp( override, (const xmlChar*)"active" );
+	}
+
+	xmlSaveFile( db->priv->overridepath, db->priv->overrides );
+}
+
+static void screem_dtd_db_row_changed( GtkTreeModel *model,
+		GtkTreePath *path, GtkTreeIter *iter,
+		ScreemDTDDB *db )
+{
+	gchar *public;
+	gboolean active;
+
+	gtk_tree_model_get( model, iter,
+			SCREEM_DTD_DB_PUBLIC_COL, &public,
+			SCREEM_DTD_DB_ACTIVE_COL, &active,
+			-1 );
+	if( public ) {
+		if( active == screem_dtd_db_get_hidden( db, public ) ) {
+			screem_dtd_db_set_hidden( db, public, ! active );
+		}
+		g_free( public );
+	}
+}
 
 /* public stuff */
 
@@ -718,18 +789,20 @@ GtkListStore *screem_dtd_db_get_store( ScreemDTDDB *db )
 	return db->priv->store;
 }
 
-ScreemDTD *screem_dtd_db_get_default_dtd( ScreemDTDDB *db )
+ScreemDTD *screem_dtd_db_get_default_dtd( ScreemDTDDB *db,
+		const gchar *mime, const gchar *root )
 {
 	ScreemDTD *dtd;
-	GConfClient *client;
 	gchar *ddtd;
 
-	client = gconf_client_get_default();
+	if( ! mime ) {
+		mime = "*";
+	}
 
-	ddtd = gconf_client_get_string( client, 
-				"/apps/screem/editor/default_dtd",
-				NULL );
-	g_object_unref( client );
+	ddtd = screem_dtd_db_lookup_mime_type( db, mime, root );
+	if( ! ddtd ) {
+		ddtd = screem_dtd_db_lookup_mime_type( db, "*", root );
+	}
 
 	if( ! ddtd ) {
 		ddtd = g_strdup( "-//W3C//DTD HTML 4.01 Transitional//EN" );
@@ -738,7 +811,7 @@ ScreemDTD *screem_dtd_db_get_default_dtd( ScreemDTDDB *db )
 	dtd = screem_dtd_db_get_dtd( db, ddtd, NULL ); 
 
 	g_free( ddtd );
-	
+		
 	return dtd;
 }
 
@@ -748,6 +821,8 @@ void screem_dtd_db_fill_list( ScreemDTDDB *db, GtkListStore *store )
 	GList *tmp;
 	GtkTreeIter it;
 	gchar *sysid;
+	gint i;
+	gboolean active;
 
 	gtk_list_store_clear( store );
 	g_object_set_data( G_OBJECT( store ), "list", NULL );
@@ -755,13 +830,14 @@ void screem_dtd_db_fill_list( ScreemDTDDB *db, GtkListStore *store )
 			      (GHFunc)dtds_insert, store );
 	list = g_object_get_data( G_OBJECT( store ), "list" );
 	list = g_list_sort( list, (GCompareFunc)screem_dtd_db_sort_compare );
-	for( tmp = list; tmp; tmp = tmp->next ) {
+	for( i = 0, tmp = list; tmp; tmp = tmp->next, ++ i ) {
 		sysid = screem_dtd_db_get_system_id( db,
 				tmp->data, NULL );
-		gtk_list_store_append( store, &it );
-		gtk_list_store_set( store, &it,
-				0, tmp->data,
-				1, sysid,
+		active = ! screem_dtd_db_get_hidden( db, tmp->data );
+		gtk_list_store_insert_with_values( store, &it, i,
+				SCREEM_DTD_DB_PUBLIC_COL, tmp->data,
+				SCREEM_DTD_DB_SYSTEM_COL, sysid,
+				SCREEM_DTD_DB_ACTIVE_COL, active,
 				-1 );
 
 		g_free( sysid );
@@ -781,19 +857,17 @@ void screem_dtd_db_override_root( ScreemDTDDB *db,
 
 	override = screem_dtd_db_lookup_override( db, publicid );
 	if( ! override ) {
-		override = xmlNewNode( NULL, "dtd" );
-		xmlSetProp( override, "public", publicid );
+		override = xmlNewNode( NULL, (const xmlChar*)"doctype" );
+		xmlSetProp( override, (const xmlChar*)"public", 
+				(xmlChar*)publicid );
 		droot = xmlDocGetRootElement( db->priv->overrides );
 		xmlAddChild( droot, override );
 	}
 	if( root ) {
-		xmlSetProp( override, "root", root );
+		xmlSetProp( override, (const xmlChar*)"root", 
+				(xmlChar*)root );
 	} else {
-		xmlUnsetProp( override, "root" );
-		if( ! xmlHasProp( override, "systemid" ) ) {
-			xmlUnlinkNode( override );
-			xmlFreeNode( override );
-		}
+		xmlUnsetProp( override, (const xmlChar*)"root" );
 	}
 
 	xmlSaveFile( db->priv->overridepath, db->priv->overrides );
@@ -811,19 +885,17 @@ void screem_dtd_db_override_systemid( ScreemDTDDB *db,
 
 	override = screem_dtd_db_lookup_override( db, publicid );
 	if( ! override ) {
-		override = xmlNewNode( NULL, "dtd" );
-		xmlSetProp( override, "public", publicid );
+		override = xmlNewNode( NULL, (const xmlChar*)"doctype" );
+		xmlSetProp( override, (const xmlChar*)"public", 
+				(xmlChar*)publicid );
 		droot = xmlDocGetRootElement( db->priv->overrides );
 		xmlAddChild( droot, override );
 	}
 	if( uri ) {
-		xmlSetProp( override, "systemid", uri );
+		xmlSetProp( override, (const xmlChar*)"system", 
+				(xmlChar*)uri );
 	} else {
-		xmlUnsetProp( override, "systemid" );
-		if( ! xmlHasProp( override, "root" ) ) {
-			xmlUnlinkNode( override );
-			xmlFreeNode( override );
-		}
+		xmlUnsetProp( override, (const xmlChar*)"system" );
 	}
 	
 	xmlSaveFile( db->priv->overridepath, db->priv->overrides );
@@ -838,7 +910,7 @@ gchar *screem_dtd_db_get_root( ScreemDTDDB *db, const gchar *publicid,
 	g_return_val_if_fail( SCREEM_IS_DTD_DB( db ), NULL );
 	g_return_val_if_fail( publicid != NULL, NULL );
 
-	dtd = screem_dtd_db_get_dtd( db, publicid, NULL ); 
+	dtd = screem_dtd_db_get_dtd( db, publicid, NULL );
 
 	ret = screem_dtd_db_override_get_root( db, publicid );
 	if( override ) {
@@ -879,35 +951,153 @@ gchar* screem_dtd_db_get_system_id( ScreemDTDDB *db,
 		tmp = g_path_get_basename( systemid );
 		base = gnome_vfs_unescape_string( tmp, "" );
 		g_free( tmp );
-		uri = gnome_vfs_uri_new( base );
+		tmp = gnome_vfs_unescape_string( base, "" );
+		uri = gnome_vfs_uri_new( tmp );
 		if( uri ) {
 			if( ! gnome_vfs_uri_is_local( uri ) ) {
 				g_free( systemid );
-				systemid = gnome_vfs_unescape_string( base, "" );
+				systemid = g_strdup( tmp );
+				/*gnome_vfs_unescape_string( base, "" );*/
 			}
 			gnome_vfs_uri_unref( uri );
 		}
+		g_free( tmp );
 		g_free( base );
 	}
 	
 	return systemid;
 }
 
-void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
-				 const gchar *doctype,
-				 gchar **publicid,
-				 gchar **systemid,
-				 gchar **root )
+gchar *screem_dtd_db_get_mime_type( ScreemDTDDB *db,
+		const gchar *publicid )
+{
+	xmlNodePtr override;
+	gchar *ret;
+	
+	g_return_val_if_fail( SCREEM_IS_DTD_DB( db ), NULL );
+	g_return_val_if_fail( publicid != NULL, NULL );
+
+	override = screem_dtd_db_lookup_override( db, publicid );
+	ret = NULL;
+	if( override ) {
+		ret = (gchar*)xmlGetProp( override, 
+				(const xmlChar*)"mime" );
+	} 	
+	return ret;
+}
+
+gboolean screem_dtd_db_get_match_root( ScreemDTDDB *db,
+		const gchar *publicid )
+{
+	xmlNodePtr override;
+	xmlChar *match;
+	gboolean ret;
+	
+	g_return_val_if_fail( SCREEM_IS_DTD_DB( db ), FALSE );
+	g_return_val_if_fail( publicid != NULL, FALSE );
+
+	override = screem_dtd_db_lookup_override( db, publicid );
+	ret = FALSE;
+	if( override ) {
+		match = xmlGetProp( override, 
+				(const xmlChar*)"rootmatch" );
+		if( match ) {
+			ret = ( *match == '1' );
+			xmlFree( match );
+		}
+	} 	
+
+	return ret;
+}
+
+void screem_dtd_db_set_mime_type( ScreemDTDDB *db,
+		const gchar *publicid, const gchar *mime )
+{
+	ScreemDTDDBPrivate *priv;
+	xmlNodePtr override;
+	xmlNodePtr droot;
+
+	g_return_if_fail( SCREEM_IS_DTD_DB( db ) );
+	g_return_if_fail( publicid != NULL );
+
+	priv = db->priv;
+
+	override = screem_dtd_db_lookup_override( db, publicid );
+	if( ! override ) {
+		override = xmlNewNode( NULL, (const xmlChar*)"doctype" );
+		xmlSetProp( override, (const xmlChar*)"public", 
+				(xmlChar*)publicid );
+		droot = xmlDocGetRootElement( db->priv->overrides );
+		xmlAddChild( droot, override );
+	}
+	if( ! mime ) {
+		xmlUnsetProp( override, (const xmlChar*)"mime" );
+		/* can't rootmatch without mime */
+		xmlUnsetProp( override, (const xmlChar*)"rootmatch" );
+	} else if( strcmp( "*", mime ) ) {
+		/* we want this doctype as the main default */
+		xmlSetProp( override, (const xmlChar*)"mime", 
+				(xmlChar*)mime );
+	} else {
+		/* default for specific type */
+		xmlSetProp( override, (const xmlChar*)"mime", 
+				(xmlChar*)mime );
+	}
+
+	/* could be possible conflicts for handling,
+	 * resolve them */
+
+	xmlSaveFile( priv->overridepath, priv->overrides );
+}
+
+void screem_dtd_db_set_match_root( ScreemDTDDB *db,
+		const gchar *publicid, gboolean match )
+{
+	ScreemDTDDBPrivate *priv;
+	xmlNodePtr override;
+	xmlNodePtr droot;
+	
+	g_return_if_fail( SCREEM_IS_DTD_DB( db ) );
+	g_return_if_fail( publicid != NULL );
+
+	priv = db->priv;
+
+	override = screem_dtd_db_lookup_override( db, publicid );
+	if( ! override ) {
+		override = xmlNewNode( NULL, (const xmlChar*)"doctype" );
+		xmlSetProp( override, (const xmlChar*)"public", 
+				(xmlChar*)publicid );
+		droot = xmlDocGetRootElement( db->priv->overrides );
+		xmlAddChild( droot, override );
+	}
+	if( ! match ) {
+		xmlUnsetProp( override, (const xmlChar*)"rootmatch" );
+	} else {
+		xmlSetProp( override, (const xmlChar*)"rootmatch", 
+				(const xmlChar*)"1" );
+	}
+
+	/* could be possible conflicts for handling,
+	 * resolve them */
+	
+	xmlSaveFile( priv->overridepath, priv->overrides );
+}
+
+
+
+const gchar *screem_dtd_db_parse_doctype( ScreemDTDDB *db,
+		const gchar *doctype,
+		gchar **publicid, gchar **systemid, gchar **root )
 {
 	GString *tmp;
 	gunichar c;
 	gunichar term;
 	gboolean istag;
-
+	guint len;
 	
-	g_return_if_fail( SCREEM_IS_DTD_DB( db ) );
-	g_return_if_fail( publicid != NULL );
-	g_return_if_fail( systemid != NULL );
+	g_return_val_if_fail( SCREEM_IS_DTD_DB( db ), NULL );
+	g_return_val_if_fail( publicid != NULL, NULL );
+	g_return_val_if_fail( systemid != NULL, NULL );
 
 	istag = ! g_strncasecmp( "<!DOCTYPE ", doctype,
 				 strlen( "<!DOCTYPE" ) );
@@ -920,39 +1110,45 @@ void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
 	}
 
 	c = '\0';
+	term = '\0';
+
+	/* max size we need for strings, just alloc this much
+	 * to avoid lots of reallocs */
+	len = strlen( doctype );
+	tmp = g_string_sized_new( len );
+	
 	if( istag ) {
 		doctype += strlen( "<!DOCTYPE " );
-		doctype = g_utf8_skip_space( doctype );
+		doctype = screem_utf8_skip_space( doctype );
 		c = g_utf8_get_char( doctype );
 
-		tmp = g_string_new( NULL );
-	
 		/* get root name */
-		while( *doctype && g_unichar_isalnum( c ) ) {
+		while( *doctype && 
+			( g_unichar_isalnum( c ) ||
+			  c == '.' || c == '_' || c == '-' ) ) {
 			g_string_append_unichar( tmp, c );
 			doctype = g_utf8_next_char( doctype );
 			c = g_utf8_get_char( doctype );
 		}
-		*root = tmp->str;
-		g_string_free( tmp, FALSE );
-	
+		*root = g_strdup( tmp->str );
+		g_string_truncate( tmp, 0 );
+		
 		/* incase we didn't hit whitespace, skip to it */
 		while( ! g_unichar_isspace( c ) ) {
 			doctype = g_utf8_next_char( doctype );
 			c = g_utf8_get_char( doctype );
 		}
 	}
-	doctype = g_utf8_skip_space( doctype );
+	doctype = screem_utf8_skip_space( doctype );
 	if( ( ! strncmp( "PUBLIC", doctype, strlen( "PUBLIC" ) ) ) ||
 	    ( ! strncmp( "public", doctype, strlen( "public" ) ) ) ) {
 		/* public id found */
 		doctype += strlen( "PUBLIC" );
 
-		doctype = g_utf8_skip_space( doctype );
+		doctype = screem_utf8_skip_space( doctype );
 	
 		term = g_utf8_get_char( doctype );
 		if( term == '"' || term == '\'' ) {
-			tmp = g_string_new( NULL );
 			do {
 				c = g_utf8_get_char( doctype );
 				if( c != term ) {
@@ -963,17 +1159,16 @@ void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
 			} while( *doctype && *doctype != term );
 		
 			/* doctype should be the char after the " */
-			*publicid = tmp->str;
-			g_string_free( tmp, FALSE );
+			*publicid = g_strdup( tmp->str );
+			g_string_truncate( tmp, 0 );
 		
 			if( *doctype == term ) {
 				doctype = g_utf8_next_char( doctype );
 			}
-			doctype = g_utf8_skip_space( doctype );
+			doctype = screem_utf8_skip_space( doctype );
 			term = g_utf8_get_char( doctype );
 		}
 		if( term == '"' || term == '\'' ) {
-			tmp = g_string_new( NULL );
 			do {
 				c = g_utf8_get_char( doctype );
 				if( c != term ) {
@@ -984,13 +1179,13 @@ void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
 			} while( *doctype && *doctype != term );
 		
 			/* doctype should be the char after the " */
-			*systemid = tmp->str;
-			g_string_free( tmp, FALSE );
+			*systemid = g_strdup( tmp->str );
+			g_string_truncate( tmp, 0 );
 
 			if( *doctype == term ) {
 				doctype = g_utf8_next_char( doctype );
 			}			
-			doctype = g_utf8_skip_space( doctype );
+			doctype = screem_utf8_skip_space( doctype );
 			term = g_utf8_get_char( doctype );
 		}	
 	} else if( ( ! strncmp( "SYSTEM", doctype, strlen( "SYSTEM" ) ) ||
@@ -999,11 +1194,10 @@ void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
 	
 		doctype += strlen( "SYSTEM" );
 
-		doctype = g_utf8_skip_space( doctype );
+		doctype = screem_utf8_skip_space( doctype );
 	
 		term = g_utf8_get_char( doctype );
 		if( term == '"' || term == '\'' ) {
-			tmp = g_string_new( NULL );
 			do {
 				c = g_utf8_get_char( doctype );
 				if( c != term ) {
@@ -1014,11 +1208,23 @@ void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
 			} while( *doctype && *doctype != term );
 		
 			/* doctype should be the char after the " */
-			*systemid = tmp->str;
-			g_string_free( tmp, FALSE );
-		}
+			*systemid = g_strdup( tmp->str );
+			g_string_truncate( tmp, 0 );
 		
+			if( *doctype == term ) {
+				doctype = g_utf8_next_char( doctype );
+			}			
+			doctype = screem_utf8_skip_space( doctype );
+			term = g_utf8_get_char( doctype );
+		}
+	}	
+	if( term != '[' || ! istag ) {
+		doctype = NULL;
 	}
+			
+	g_string_free( tmp, TRUE );
+
+	return doctype;
 }
 
 ScreemDTD *screem_dtd_db_get_dtd_from_doctype( ScreemDTDDB *db,
@@ -1029,12 +1235,71 @@ ScreemDTD *screem_dtd_db_get_dtd_from_doctype( ScreemDTDDB *db,
 	gchar *systemid;
 	gchar *root;
 
-	screem_dtd_db_parse_doctype( db, doctype, 
-				     &publicid, &systemid, &root );
+	gchar *temp;
+	gchar *temp2;
+	gchar *data;
+	ScreemDTDParse parse;
+
+	gboolean success;
+	
+	doctype = screem_dtd_db_parse_doctype( db, doctype, 
+			&publicid, &systemid, &root );
 	dtd = NULL;
 
+	/* only give a dtd back if we have a root element and
+	 * a public or system identifier
+	 *
+	 * check if the doctype contains any entities, if so we
+	 * always want a new ScreemDTD object which contains them
+	 *
+	 */
 	if( ( publicid || systemid ) && root ) {
-		dtd = screem_dtd_db_get_dtd( db, publicid, systemid );
+		if( doctype ) {
+			doctype = g_utf8_next_char( doctype );
+			temp = strstr( doctype, "]>" );
+			if( temp ) {
+				doctype = g_strndup( doctype,
+						temp - doctype );
+				temp = (gchar*)doctype;
+			} else {
+				doctype = NULL;
+			}
+		}
+		if( ! doctype ) {
+			/* standard doctype */
+			dtd = screem_dtd_db_get_dtd( db, publicid, systemid );
+		} else {
+			success = FALSE;
+			
+			/* document contains entities */
+			data = screem_dtd_db_resolve_entity( db, 
+					publicid, systemid );
+			if( data ) {
+				dtd = screem_dtd_new( publicid, 
+						systemid );
+
+				/* parse data, adding to dtd */
+				parse.dtd = dtd;
+				parse.resolve_entity = screem_dtd_db_resolve_entity;
+				parse.userdata = db;
+				temp2 = g_strconcat( doctype, data,
+						NULL );
+				g_free( data );
+				data = temp2;
+				success = screem_dtd_parse( dtd, 
+						&parse, data );
+				g_free( data );
+			}
+			g_free( temp );
+			if( success ) {
+				g_object_set_data( G_OBJECT( dtd ),
+						"NO_CACHE",
+						GINT_TO_POINTER( 1 ) );
+			} else if( dtd ) {
+				g_object_unref( dtd );
+				dtd = NULL;
+			}
+		}
 	}
 
 	g_free( publicid );
@@ -1078,3 +1343,156 @@ ScreemDTD *screem_dtd_db_get_dtd( ScreemDTDDB *db,
 	return dtd;
 }
 
+GSList *screem_dtd_db_get_default_conflicts( ScreemDTDDB *db,
+	const gchar *publicid, const gchar *mime, gboolean rootmatch )
+{
+	GSList *ret;
+	ScreemDTDDBPrivate *priv;
+	gchar *root;
+	xmlNodePtr node;
+	
+	g_return_val_if_fail( SCREEM_IS_DTD_DB( db ), NULL );
+	g_return_val_if_fail( publicid != NULL, NULL );
+	g_return_val_if_fail( mime != NULL, NULL );
+
+	ret = NULL;
+	
+	priv = db->priv;
+
+	root = screem_dtd_db_get_root( db, publicid, NULL );
+
+	node = xmlDocGetRootElement( priv->overrides );
+	node = node->children;
+	while( node ) {
+		xmlChar *cpublicid;
+		xmlChar *cmime;
+		xmlChar *crootmatch;
+		gboolean crootmatchflag;
+		gchar *croot;
+		
+		cpublicid = xmlGetProp( node, (const xmlChar*)"public" );
+		cmime = xmlGetProp( node, (const xmlChar*)"mime" );
+		crootmatch = xmlGetProp( node, 
+				(const xmlChar*)"rootmatch" );
+
+		crootmatchflag = ( crootmatch && *crootmatch == '1' );
+		
+		/* should probably assert cpublicid != NULL, 
+		   although the dtd says public is optional as we 
+		   can have system instead there is no way for that
+		   to work properly at the moment 
+		
+		   only check different public id's + those with
+		   a matching mime type set, if they have no mime type they
+		   are not the default for anything.   
+		*/
+		if( ( cpublicid && strcmp( publicid, (gchar*)cpublicid ) ) && 
+		    ( cmime && ! strcmp( mime, (gchar*)cmime ) ) ) {
+			
+			if( ( ! rootmatch ) && ( ! crootmatchflag ) ) {
+				/* conflict */
+				ret = g_slist_prepend( ret, g_strdup( (gchar*)cpublicid ) );
+			} else if( rootmatch && crootmatchflag ) {
+				croot = screem_dtd_db_get_root( db, (gchar*)cpublicid, NULL );
+				if( root && croot && ! strcmp( root, (gchar*)croot ) ) {
+					/* conflict */
+					ret = g_slist_prepend( ret, g_strdup( (gchar*)cpublicid ) );
+				}
+				g_free( croot );
+			}
+		}
+
+		if( cpublicid ) {
+			xmlFree( cpublicid );		
+		}
+		if( cmime ) {
+			xmlFree( cmime );	
+		}
+		if( crootmatch ) {
+			xmlFree( crootmatch );		
+		}
+		
+		node = node->next;
+	}
+	
+	g_free( root );
+	
+	return ret;
+}
+
+gchar *screem_dtd_db_lookup_mime_type( ScreemDTDDB *db,
+		const gchar *mime, const gchar *root )
+{
+	ScreemDTDDBPrivate *priv;
+	gchar *ret;
+	xmlNodePtr node;
+
+	gchar *fallback;
+
+	priv = db->priv;
+
+	if( ! mime ) {
+		mime = "*";
+		g_warning( "NULL mime type sent to screem_dtd_db_lookup_mime_type\n" );
+	}
+
+	ret = NULL;
+		
+	node = xmlDocGetRootElement( priv->overrides );
+	node = node->children;
+
+	fallback = NULL;
+	while( node && ! ret ) {
+		xmlChar *cpublic;
+		xmlChar *cmime;
+		xmlChar *crootmatch;
+		gboolean crootmatchflag;
+		gchar *croot;
+
+		cpublic = xmlGetProp( node, (const xmlChar*)"public" );
+		cmime = xmlGetProp( node, (const xmlChar*)"mime" );
+		crootmatch = xmlGetProp( node, 
+				(const xmlChar*)"rootmatch" );
+		crootmatchflag = ( crootmatch && *crootmatch == '1' );
+		
+		if( mime && cmime && ! strcmp( mime, (gchar*)cmime ) ) {
+
+			if( ! crootmatchflag ) {
+				/* match */
+				if( ! fallback ) {
+					fallback = g_strdup( (gchar*)cpublic );				
+				}
+			} else if( root ) {
+				croot = screem_dtd_db_get_root( db,
+						(gchar*)cpublic, NULL );
+				if( croot && ! strcmp( root, (gchar*)croot ) ) {
+					ret = g_strdup( (gchar*)cpublic );
+				}
+				g_free( croot );
+			}
+		}		
+		if( cpublic ) {
+			xmlFree( cpublic );
+		}
+		if( cmime ) {
+			xmlFree( cmime );
+		}
+		if( crootmatch ) {
+			xmlFree( crootmatch );
+		}
+		
+		node = node->next;
+	}
+	
+	if( fallback && ! ret ) {
+		ret = g_strdup( fallback );
+	}	
+	g_free( fallback );
+
+	if( ( ! ret ) && mime[ 0 ] != '*' && mime[ 1 ] != '\0' ) {
+		ret = screem_dtd_db_lookup_mime_type( db, "*", root );
+	}	
+	return ret;
+}
+
+
