@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -36,14 +36,7 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-#ifdef HAVE_ICON_THEME
-#include <libgnomeui/gnome-icon-theme.h>
-#include <libgnomeui/gnome-icon-lookup.h>
-#endif
-
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtktreestore.h>
-#include <gtk/gtktreemodelsort.h>
+#include <gtk/gtk.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
@@ -55,23 +48,15 @@
 
 #include "support.h"
 
-#define ICON_X_SIZE 16
-#define ICON_Y_SIZE 16
-
 enum {
 	ADDED = 0,
 	REMOVED,
 	ICON_CHANGE,
-	THEME_CHANGE,
 	LAST_SIGNAL
 };
 
 static guint screem_file_browser_signals[ LAST_SIGNAL ] = { 0 };
 
-static GtkIconTheme *gtk_theme = NULL;
-
-/* static icon cache */
-static GHashTable *icon_cache = NULL;
 /* lock access to the icon cache */
 static GMutex *cache_mutex = NULL;
 
@@ -91,21 +76,26 @@ typedef struct {
 	gchar *pathname;
 	gchar *basename;
 
-	gchar *current; /* only set if basename is ".." */
-
-	GdkPixbuf *close;
-
-	gchar *mime_type;
+	const gchar *mime_type;
 
 	GnomeVFSMonitorHandle *monitor;
 } BrowserNode;
 
 struct ScreemFileBrowserDetails {
 	GtkTreeStore *model;
+	
+	GtkTreeModel *filtered;
 
 	GtkTreeModel *sorted;
 
+	
 	int mode;
+
+	ScreemIconCache *cache;
+
+	gboolean show_hidden;
+	gboolean show_backups;
+	gchar *backup_suffix;
 };
 
 static BrowserNode *browser_node_new( ScreemFileBrowser *browser, 
@@ -131,10 +121,8 @@ static void screem_file_browser_set_icons( ScreemFileBrowser *browser,
 					     GtkTreeIter *it );
 
 
-#ifdef HAVE_ICON_THEME
-static void screem_file_browser_theme_change( GnomeIconTheme *theme,
-						gpointer data );
-#endif
+static void screem_file_browser_theme_change( ScreemIconCache *cache,
+		gpointer data );
 
 static gint screem_file_browser_compare_func( GtkTreeModel *model,
 						GtkTreeIter *a,
@@ -147,22 +135,22 @@ static gboolean screem_file_browser_clear( GtkTreeModel *model,
 					     GtkTreeIter *it,
 					     gpointer data );
 
-static void screem_file_browser_class_init( ScreemFileBrowserClass *klass );
-static void screem_file_browser_init( ScreemFileBrowser *file_browser );
-static void screem_file_browser_finalize( GObject *object );
-static void screem_file_browser_set_prop( GObject *object, guint prop_id, 
-					    const GValue *value, GParamSpec *spec );
-static void screem_file_browser_get_prop( GObject *object, guint prop_id, 
-					    GValue *value, GParamSpec *spec );
+static gboolean screem_file_browser_filter( GtkTreeModel *model,
+		GtkTreeIter *it, ScreemFileBrowser *browser );
+static gboolean refilter( GtkTreeModel *model, GtkTreePath *path,
+		GtkTreeIter *iter, ScreemFileBrowser *browser );
 
-
-ScreemFileBrowser *screem_file_browser_new()
+ScreemFileBrowser *screem_file_browser_new( ScreemIconCache *cache )
 {
 	ScreemFileBrowser *browser;
 
 	browser = SCREEM_FILE_BROWSER( g_object_new( screem_file_browser_get_type(),
 						  NULL ) );
-
+	g_object_ref( cache );
+	browser->details->cache = cache;
+	g_signal_connect( G_OBJECT( cache ), "changed",
+			G_CALLBACK( screem_file_browser_theme_change ),
+			browser );
 
 	return browser;
 }
@@ -199,7 +187,16 @@ void screem_file_browser_scan_directory( ScreemFileBrowser *browser,
 	/* try and find uri in the current model, and set
 	   parent to the iter's parent if found */
 	parent = NULL;
-	if( find_iter( model, uri, NULL, &it ) ) {
+
+	/* if uri at depth 0 is the root of our tree then
+	 * clear all nodes */
+	if( ( mode & FILE_BROWSE_ROOT ) ) {
+		parent = NULL;
+		gtk_tree_model_foreach( model, 
+				screem_file_browser_clear,
+				browser );
+		gtk_tree_store_clear( GTK_TREE_STORE( model ) );
+	} else if( find_iter( model, uri, NULL, &it ) ) {
 		if( gtk_tree_model_iter_parent( model, &pit, &it ) ) {
 			parent = &pit;
 		}
@@ -208,7 +205,7 @@ void screem_file_browser_scan_directory( ScreemFileBrowser *browser,
 	if( mode == -1 ) {
 		mode = browser->details->mode;
 	}
-
+	
 	scan_directory( browser, parent, uri, "x-directory/normal", 
 			0, mode );
 
@@ -222,140 +219,74 @@ void screem_file_browser_scan_directory( ScreemFileBrowser *browser,
 					    "/", 
 					    -1 );
 		}
-	} else if( mode & FILE_BROWSE_FLAT ) {
-		/* replace first iter with .. or remove completly
-		   if at the top level */
-		GtkTreeIter fit;
-
-		if( gtk_tree_model_get_iter_first( model, &fit ) ) {
-			BrowserNode *node;
-
-			gtk_tree_model_get( model, &fit,
-					  FILE_BROWSER_NODE_COL,
-					  &node, -1 );
-			
-			if( node && ! strcmp( "/", node->pathname ) ) {
-				/* root node, remove it,
-				   we still want to monitor / though,
-				   so we can't free the node so its
-				   getting leaked currently */
-				gtk_tree_store_remove( GTK_TREE_STORE( model ),
-						       &fit );
-			} else if( node ) {
-				/* replace */
-				gchar *temp;
-
-				temp = g_path_get_dirname( node->pathname );
-				g_free( node->pathname );
-				node->pathname = temp;
-				g_free( node->basename );
-				node->basename = g_strdup( ".." );
-
-				/* add monitor for temp */
-
-				gtk_tree_store_set( GTK_TREE_STORE( model ),
-						    &fit,
-						    FILE_BROWSER_NAME_COL,
-						    "..",
-						    FILE_BROWSER_URI_COL,
-						    node->pathname,
-						    -1 );
-			}
-		}
 	}
 }
 
-const gchar* screem_file_browser_scan_iter( ScreemFileBrowser *browser,
-					    GtkTreeIter *iter,
-					    int mode )
+void screem_file_browser_scan_iter( ScreemFileBrowser *browser,
+		GtkTreeIter *iter, int mode )
 {
+	ScreemFileBrowserDetails *details;
 	GtkTreeModel *model;
 	BrowserNode *node;
 	gchar *uri;
+	GtkTreeIter filtered_it;
 	GtkTreeIter realit;
 	GtkTreeIter *parent = NULL;
 	GtkTreeIter pit;
 
-	const gchar *current = NULL;
+	g_return_if_fail( SCREEM_IS_FILE_BROWSER( browser ) );
+	g_return_if_fail( iter != NULL );
 
-	g_return_val_if_fail( SCREEM_IS_FILE_BROWSER( browser ), NULL );
-	g_return_val_if_fail( iter != NULL, NULL );
-
-	model = GTK_TREE_MODEL( browser->details->model );
+	details = browser->details;
+	
+	model = GTK_TREE_MODEL( details->model );
 
+	g_assert( gtk_tree_model_sort_iter_is_valid( GTK_TREE_MODEL_SORT( details->sorted ), iter ) );
+	
 	/* convert iter to a child iter */
-	gtk_tree_model_sort_convert_iter_to_child_iter( GTK_TREE_MODEL_SORT( browser->details->sorted ), &realit, iter );
+	gtk_tree_model_sort_convert_iter_to_child_iter( GTK_TREE_MODEL_SORT( details->sorted ), &filtered_it, iter );
+	gtk_tree_model_filter_convert_iter_to_child_iter( GTK_TREE_MODEL_FILTER( details->filtered ), &realit, &filtered_it );
+	
 	iter = &realit;
 
 	gtk_tree_model_get( model, iter, 
 			FILE_BROWSER_NODE_COL, &node, -1 );
 
-	if( ( ! node ) || node->type != BROWSER_NODE_DIRECTORY ) {
-		return NULL;
-	}
+	if( node && node->type == BROWSER_NODE_DIRECTORY ) {
+		uri = g_strdup( node->pathname );
 
-	uri = g_strdup( node->pathname );
+		if( mode == -1 ) {
+			mode = details->mode;
+		}
 
-	if( gtk_tree_model_iter_parent( model, &pit, iter ) ) {
-		parent = &pit;
-	}
+	
+		if( gtk_tree_model_iter_has_child( model, iter ) ) {
+			GtkTreeIter cit;
+			while( gtk_tree_model_iter_children( model, &cit, iter ) ) {
+				BrowserNode *node;
 
-	if( ( ! ( mode & FILE_BROWSE_FLAT ) ) && ! parent ) {
-		return NULL;
-	}
-
-	if( mode == -1 ) {
-		mode = browser->details->mode;
-	}
-
-	scan_directory( browser, iter, uri,  "x-directory/normal",
-			0, mode );
-
-	current = "/";
-	if( mode & FILE_BROWSE_FLAT ) {
-		/* replace first iter with .. or remove completly
-		   if at the top level */
-		GtkTreeIter fit;
-
-		if( gtk_tree_model_get_iter_first( model, &fit ) ) {
-			BrowserNode *node;
-
-			gtk_tree_model_get( model, &fit,
-					  FILE_BROWSER_NODE_COL,
-					  &node, -1 );
+				gtk_tree_model_get( model, &cit,
+						FILE_BROWSER_NODE_COL, 
+						&node, -1 );
+				if( node ) {
+					browser_node_free( node );
+				}
 			
-			if( node && ! strcmp( "/", node->pathname ) ) {
-				/* root node, remove it */
 				gtk_tree_store_remove( GTK_TREE_STORE( model ),
-						       &fit );
-			} else if( node ) {
-				/* replace */
-				gchar *temp;
-
-				temp = g_path_get_dirname( node->pathname );
-				current = node->current = node->pathname;
-				node->pathname = temp;
-				g_free( node->basename );
-				node->basename = g_strdup( ".." );
-
-				gtk_tree_store_set( GTK_TREE_STORE( model ),
-						    &fit,
-						    FILE_BROWSER_NAME_COL,
-						    "..",
-						    FILE_BROWSER_URI_COL,
-						    node->pathname,
-						    -1 );
+						&cit );
 			}
 		}
+	
+		scan_directory( browser, iter, 
+				uri,  "x-directory/normal", 0, 
+				mode );
+	
+		g_free( uri );
 	}
-
-	g_free( uri );
-
-	return current;
 }
 
 GList *screem_file_browser_get_pathnames( ScreemFileBrowser *browser,
-					  GList *iters, gboolean recurse )
+		GList *iters, gboolean recurse )
 {
 	GList *ret;
 	GtkTreeModel *model;
@@ -370,6 +301,7 @@ GList *screem_file_browser_get_pathnames( ScreemFileBrowser *browser,
 	while( iters ) {
 		GtkTreeIter *iter;
 		GtkTreeIter it;
+		GtkTreeIter filtered_it;
 		BrowserNode *node;
 
 		iter = (GtkTreeIter*)iters->data;
@@ -393,7 +325,8 @@ GList *screem_file_browser_get_pathnames( ScreemFileBrowser *browser,
 			}
 		}
 		
-		gtk_tree_model_sort_convert_iter_to_child_iter( GTK_TREE_MODEL_SORT( browser->details->sorted ), &it, iter );
+		gtk_tree_model_sort_convert_iter_to_child_iter( GTK_TREE_MODEL_SORT( browser->details->sorted ), &filtered_it, iter );
+		gtk_tree_model_filter_convert_iter_to_child_iter( GTK_TREE_MODEL_FILTER( browser->details->filtered ), &it, &filtered_it );
 
 		gtk_tree_model_get( model, &it,
 				  FILE_BROWSER_NODE_COL, &node, -1 );
@@ -401,14 +334,9 @@ GList *screem_file_browser_get_pathnames( ScreemFileBrowser *browser,
 		if( node ) {
 			ret = g_list_append( ret, node->pathname );
 
-			if( recurse && node->type == BROWSER_NODE_DIRECTORY && 
-			    ( browser->details->mode & FILE_BROWSE_FLAT ) ) {
-				/* we need to scan node->pathname */
-				ret = g_list_concat( ret, screem_file_browser_recurse( node->pathname ) );
-			}
 		} else {
 			gchar *uri;
-	
+			
 			uri = NULL;
 			gtk_tree_model_get( model, &it,
 					    FILE_BROWSER_URI_COL,
@@ -440,131 +368,6 @@ void screem_file_browser_set_sort_func( ScreemFileBrowser *browser,
 					      0, GTK_SORT_ASCENDING );
 }
 
-GdkPixbuf *screem_file_browser_get_icon( ScreemFileBrowser *browser,
-					 const gchar *uri,
-					 gint width,
-					 gint height,
-					gboolean ignore_cache,
-					gchar **mtype )
-{
-	gchar *mime_type;
-	GdkPixbuf *ret;
-	const gchar *file;
-	GnomeVFSFileInfo *info;
-	gchar *tmp;
-
-	GtkIconInfo *iinfo;
-	
-	g_return_val_if_fail( SCREEM_IS_FILE_BROWSER( browser ), NULL );
-
-	if( ! uri ) {
-		uri = "";
-	}
-	
-	mime_type = NULL;
-	if( mtype && *mtype ) {
-		mime_type = g_strdup( *mtype );
-	}
-	
-	/* Hack to avoid file access when we are accessing
-	 * directories */
-	if( uri[ strlen( uri ) -1 ] == '/' ) {
-		if( mime_type ) {
-			g_free( mime_type );
-		}
-		mime_type = g_strdup( "x-directory/normal" );
-	} else if( mime_type == NULL ){
-		info = gnome_vfs_file_info_new();
-
-		if( gnome_vfs_get_file_info( uri, info, 
-				 GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
-				 GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
-				 GNOME_VFS_FILE_INFO_FOLLOW_LINKS ) ==
-				GNOME_VFS_OK ) {
-			mime_type = g_strdup( info->mime_type );
-		} else {
-			/* FIXME: application/octet-stream ? */
-			mime_type = g_strdup( "" );
-		}
-		gnome_vfs_file_info_unref( info );
-	}
-	if( mtype && ( *mtype == NULL ) ) {
-		*mtype = g_strdup( mime_type );
-	}
-
-	/* figure out icon filename */
-	tmp = gnome_icon_lookup( gtk_theme, 
-				  NULL,
-				  uri,
-				  NULL,
-				  NULL,
-				  mime_type,
-				  0, NULL );
-	file = NULL;
-	iinfo = NULL;
-	if( tmp ) {
-		iinfo = gtk_icon_theme_lookup_icon( gtk_theme, tmp,
-						     width, 0 );
-		file = gtk_icon_info_get_filename( iinfo );
-		g_free( tmp );
-		if( ! file ||
-		    ! g_file_test( file, G_FILE_TEST_EXISTS ) ) {
-			gtk_icon_info_free( iinfo );
-			info = NULL;
-		}
-	
-	}
-	g_free( mime_type );
-	g_mutex_lock( cache_mutex );
-	
-	ret = NULL;
-	if( iinfo ) {
-		GdkPixbuf *close = NULL;
-	
-		if( ! ignore_cache ) {
-			ret = g_hash_table_lookup( icon_cache, file );
-		}
-		
-		if( ! ret ) {
-			close = gdk_pixbuf_new_from_file( file, NULL );
-		} else {
-			g_object_ref( ret );
-		}
-		if( close ) {
-			gint w;
-			gint h;
-			gdouble f;
-
-			w = gdk_pixbuf_get_width( close );
-			h = gdk_pixbuf_get_height( close );
-
-			if( h > w ) {
-				f = h / (gdouble)height;
-			} else {
-				f = w / (gdouble)width;
-			}
-			h /= f;
-			w /= f;
-
-			ret = gdk_pixbuf_scale_simple( close,
-						       w, h,
-						       GDK_INTERP_BILINEAR );
-			g_object_ref( ret );
-			if( ! ignore_cache ) {
-				g_hash_table_insert( icon_cache, 
-						g_strdup( file ), ret );
-			}
-			g_object_unref( close );
-		}
-
-		gtk_icon_info_free( iinfo );
-	}
-
-	g_mutex_unlock( cache_mutex );
-
-	return ret;
-}
-
 void screem_file_browser_file_mod( ScreemFileBrowser *browser,
 				const gchar *directory,
 				const gchar *file,
@@ -573,14 +376,63 @@ void screem_file_browser_file_mod( ScreemFileBrowser *browser,
 	directory_monitor_cb( NULL, directory, file, type, browser );
 }
 
+void screem_file_browser_set_show_hidden( ScreemFileBrowser *browser,
+		gboolean show )
+{
+	ScreemFileBrowserDetails *details;
+	
+	g_return_if_fail( SCREEM_IS_FILE_BROWSER( browser ) );
+
+	details = browser->details;
+
+	details->show_hidden = show;
+
+	gtk_tree_model_foreach( GTK_TREE_MODEL( details->model ),
+			(GtkTreeModelForeachFunc)refilter,
+			browser );
+}
+
+void screem_file_browser_set_show_backups( ScreemFileBrowser *browser,
+		gboolean show )
+{
+	ScreemFileBrowserDetails *details;
+	
+	g_return_if_fail( SCREEM_IS_FILE_BROWSER( browser ) );
+
+	details = browser->details;
+
+	details->show_backups = show;
+
+	gtk_tree_model_foreach( GTK_TREE_MODEL( details->model ),
+			(GtkTreeModelForeachFunc)refilter,
+			browser );
+}
+
+/* override default backup file suffix of ~ */
+void screem_file_browser_set_backup_suffix( ScreemFileBrowser *browser,
+		const gchar *suffix )
+{
+	ScreemFileBrowserDetails *details;
+	
+	g_return_if_fail( SCREEM_IS_FILE_BROWSER( browser ) );
+
+	details = browser->details;
+
+	g_free( details->backup_suffix );
+	if( suffix ) {
+		details->backup_suffix = g_strdup( suffix );
+	} else {
+		details->backup_suffix = g_strdup( "~" );
+	}
+	
+	gtk_tree_model_filter_refilter( GTK_TREE_MODEL_FILTER( details->filtered ) );
+}
+
 void screem_file_browser_cleanup( void )
 {
 	if( cache_mutex ) {
 		g_mutex_free( cache_mutex );
 	}
-	if( icon_cache ) {
-		g_hash_table_destroy( icon_cache );
-	}
 	if( mime_cache ) {
 		g_hash_table_destroy( mime_cache );
 	}
@@ -592,7 +444,6 @@ static BrowserNode *browser_node_new( ScreemFileBrowser *browser,
 		const gchar *mime_type )
 {
 	BrowserNode *node;
-	const gchar *lookup_uri;
 	gchar *temp;
 
 	node = g_new0( BrowserNode, 1 );
@@ -603,12 +454,13 @@ static BrowserNode *browser_node_new( ScreemFileBrowser *browser,
 	g_assert( uri != NULL );
 	
 	if( mime_type ) {
-		node->mime_type = g_strdup( mime_type );
+		node->mime_type = mime_type;
 	}
 	
 		
-	node->pathname = g_strdup( uri );
-
+	node->pathname = g_filename_to_utf8( uri, 
+			-1, NULL, NULL, NULL );
+		
 	temp = g_path_get_basename( node->pathname );
 	if( temp ) {
 		node->basename = gnome_vfs_unescape_string_for_display( temp );
@@ -619,36 +471,25 @@ static BrowserNode *browser_node_new( ScreemFileBrowser *browser,
 		g_free( node->basename );
 		node->basename = g_strdup( G_DIR_SEPARATOR_S );
 	}
-
-	lookup_uri = uri;
-	if( type == BROWSER_NODE_DIRECTORY ) {
-		lookup_uri = "/";
-	}
-		
-	node->close = screem_file_browser_get_icon( browser, 
-						lookup_uri,
-						ICON_X_SIZE,
-						ICON_Y_SIZE,
-						FALSE,
-						&node->mime_type );
 	
 	g_mutex_lock( cache_mutex );
 
 	mime_type = g_hash_table_lookup( mime_cache, node->mime_type );
 	if( ! mime_type ) {
-		g_hash_table_insert( mime_cache, node->mime_type,
-				node->mime_type );
+		node->mime_type = g_strdup( node->mime_type );
+		/* safe to cast to gpointer, node->mime_type will
+		 * stay valid while mime_cache is around */
+		g_hash_table_insert( mime_cache, 
+				(gpointer)node->mime_type,
+				(gpointer)node->mime_type );
 		mime_type = node->mime_type;
 	} else if( mime_type != node->mime_type ) {
-		g_free( node->mime_type );
 		node->mime_type = mime_type;
 	}
 	
 	g_mutex_unlock( cache_mutex );
 
-       	if( type == BROWSER_NODE_DIRECTORY &&
-	    ( ( ! ( browser->details->mode & FILE_BROWSE_FLAT ) ) ||
-	      depth == 0 ) ) {
+       	if( type == BROWSER_NODE_DIRECTORY && depth == 0 ) {
 		/* don't monitor /dev, /proc, or /sys */
 		gchar *monuri;
 
@@ -659,6 +500,7 @@ static BrowserNode *browser_node_new( ScreemFileBrowser *browser,
 		if( strncmp( "/dev", monuri, strlen( "/dev" ) ) &&
 		    strncmp( "/sys", monuri, strlen( "/sys" ) ) &&
 		    strncmp( "/proc", monuri, strlen( "/proc" ) ) ) {
+
 			gnome_vfs_monitor_add( &node->monitor, uri,
 					       GNOME_VFS_MONITOR_DIRECTORY,
 					       directory_monitor_cb, 
@@ -675,10 +517,6 @@ static void browser_node_free( BrowserNode *node )
 	g_return_if_fail( node != NULL );
 
 	g_free( node->pathname );
-	g_free( node->current );
-	if( node->close ) {
-		g_object_unref( node->close );
-	}
 	g_free( node->basename );
 
 	if( node->monitor ) {
@@ -688,6 +526,73 @@ static void browser_node_free( BrowserNode *node )
 	g_free( node );
 }
 
+static void scan_directory_add_node( ScreemFileBrowser *browser,
+		GtkTreeIter *parent, const gchar *uri,
+		const gchar *mime_type, guint depth,
+		BrowserNodeType type, GtkTreeIter *it )
+{
+	ScreemFileBrowserDetails *details;
+	BrowserNode *node;
+	GdkPixbuf *pixbuf;
+	GtkTreeStore *model;
+	gboolean visible;
+	
+	g_return_if_fail( SCREEM_IS_FILE_BROWSER( browser ) );
+	g_return_if_fail( parent != NULL || type == BROWSER_NODE_DIRECTORY);
+	g_return_if_fail( it != NULL );
+	g_return_if_fail( uri != NULL );
+	g_return_if_fail( mime_type != NULL );
+	
+	details = browser->details;
+	model = details->model;
+	
+	gtk_tree_store_append( model, it, parent );
+	
+	pixbuf = screem_icon_cache_get_pixbuf( details->cache,
+			uri, mime_type,
+			SCREEM_ICON_CACHE_DEFAULT_SIZE );
+
+	node = browser_node_new( browser, depth,
+			type, uri, mime_type );
+	
+	gtk_tree_store_set( model, it,
+			    FILE_BROWSER_NAME_COL, node->basename,
+			    FILE_BROWSER_NODE_COL, node,
+			    FILE_BROWSER_ICON_COL, pixbuf,
+			    FILE_BROWSER_URI_COL, node->pathname,
+			    FILE_BROWSER_MIME_COL, node->mime_type,
+			    -1 );
+	visible = screem_file_browser_filter( GTK_TREE_MODEL( model ),
+			it, browser );
+	gtk_tree_store_set( model, it, FILE_BROWSER_SHOW_COL, visible,
+			-1 );
+
+	if( pixbuf ) {
+		g_object_unref( pixbuf );
+	}	
+	g_signal_emit( G_OBJECT( browser ),
+		       screem_file_browser_signals[ ADDED ], 0,
+		       node->pathname, node->mime_type, it );
+}
+
+static void scan_directory_add_file( ScreemFileBrowser *browser,
+		GtkTreeIter *parent, const gchar *uri,
+		const gchar *mime_type, guint depth )
+{
+	GtkTreeIter it;
+	
+	scan_directory_add_node( browser, parent, uri, mime_type,
+			depth, BROWSER_NODE_FILE, &it );
+}
+
+static void scan_directory_add_dir( ScreemFileBrowser *browser,
+		GtkTreeIter *parent, const gchar *uri,
+		const gchar *mime_type, guint depth, GtkTreeIter *it )
+{
+	scan_directory_add_node( browser, parent, uri, mime_type,
+			depth, BROWSER_NODE_DIRECTORY, it );
+}
+
 static void scan_directory( ScreemFileBrowser *browser, 
 			    GtkTreeIter *parent,
 			    const gchar *uri,
@@ -698,7 +603,6 @@ static void scan_directory( ScreemFileBrowser *browser,
 	GnomeVFSDirectoryHandle *handle;
 	GnomeVFSFileInfo *info;
 
-	GtkTreeIter clear;
 	GtkTreeIter *root;
 
 	gchar *parent_uri;
@@ -712,24 +616,9 @@ static void scan_directory( ScreemFileBrowser *browser,
 	g_return_if_fail( GTK_IS_TREE_MODEL( model ) );
 	g_return_if_fail( uri != NULL );
 
-	if( mode & FILE_BROWSE_FLAT ||
-	    ( mode & FILE_BROWSE_ROOT && ( depth == 0 ) ) ) {
-		parent = NULL;
-		if( depth == 0 ) {
-			gtk_tree_model_foreach( model, 
-						screem_file_browser_clear,
-						browser );
-			gtk_tree_store_clear( GTK_TREE_STORE( model ) );
-		}
-	}
-
 	root = parent;
 	parent_uri = NULL;
-	if( ! parent ) {
-		if( gtk_tree_model_get_iter_first( model, &clear ) ) {
-			root = &clear;
-		}
-	} else {
+	if( parent ) {
 		BrowserNode *node;
 
 		gtk_tree_model_get( model, parent,
@@ -739,25 +628,6 @@ static void scan_directory( ScreemFileBrowser *browser,
 		}
 	}
 
-	if( depth == 0 && root && 
-		gtk_tree_model_iter_has_child( model, root ) ) {
-		/* scanning at depth 0, remove current children */
-		GtkTreeIter cit;
-		while( gtk_tree_model_iter_children( model, &cit, root ) ) {
-			BrowserNode *node;
-
-			gtk_tree_model_get( model, &cit,
-					FILE_BROWSER_NODE_COL, &node,
-					-1 );
-			if( node ) {
-				browser_node_free( node );
-			}
-
-			gtk_tree_store_remove( GTK_TREE_STORE( model ),
-					       &cit );
-		}
-	}
-
 	result = GNOME_VFS_ERROR_NOT_A_DIRECTORY;
 	handle = NULL;
 	info = NULL;
@@ -771,51 +641,15 @@ static void scan_directory( ScreemFileBrowser *browser,
 	}
 	if( result == GNOME_VFS_ERROR_NOT_A_DIRECTORY ) {
 		/* normal file */
-		GtkTreeIter it;
-		BrowserNode *node;
-
-		gtk_tree_store_append( GTK_TREE_STORE( model ), &it, parent );
-
-		node = browser_node_new( browser, depth,
-					 BROWSER_NODE_FILE, uri,
-					 mime_type );
-		gtk_tree_store_set( GTK_TREE_STORE( model ), &it,
-				    FILE_BROWSER_NAME_COL, node->basename,
-				    FILE_BROWSER_NODE_COL, node,
-				    FILE_BROWSER_ICON_COL, node->close,
-				    FILE_BROWSER_URI_COL, node->pathname,
-				    FILE_BROWSER_MIME_COL, node->mime_type,
-				    -1 );
-
-		g_signal_emit( G_OBJECT( browser ),
-			       screem_file_browser_signals[ ADDED ], 0,
-			       node->pathname, node->mime_type, &it );
+		scan_directory_add_file( browser, parent, uri,
+				mime_type, depth );
 	}  else if( result == GNOME_VFS_OK ){
 		GtkTreeIter it;
-		BrowserNode *node;
 
 		/* append if parent uri != uri */
 		if( ! parent_uri || strcmp( uri, parent_uri ) ) {
-			gtk_tree_store_append( GTK_TREE_STORE( model ), &it, 
-					       parent );
-			node = browser_node_new( browser, depth,
-						 BROWSER_NODE_DIRECTORY,
-						 uri, mime_type );
-			gtk_tree_store_set( GTK_TREE_STORE( model ), &it,
-					    FILE_BROWSER_NAME_COL,
-					    node->basename,
-					    FILE_BROWSER_NODE_COL, node, 
-					    FILE_BROWSER_ICON_COL,
-					    node->close,
-					    FILE_BROWSER_URI_COL,
-					    node->pathname,
-				    	    FILE_BROWSER_MIME_COL, 
-					    node->mime_type,
-					    -1 );
-			g_signal_emit( G_OBJECT( browser ),
-				       screem_file_browser_signals[ ADDED ],
-				       0,
-				       node->pathname, node->mime_type, &it );
+			scan_directory_add_dir( browser, parent,
+					uri, mime_type, depth, &it );
 		} else if( parent_uri ) {
 			it = *root;
 		}
@@ -826,8 +660,11 @@ static void scan_directory( ScreemFileBrowser *browser,
 
 			gtk_tree_store_append( GTK_TREE_STORE( model ),
 					&iter, &it );
-		} else if( ( ! ( mode & FILE_BROWSE_FLAT ) ) ||
-		    ( ( mode & FILE_BROWSE_FLAT ) && depth < 1 ) ) {
+			gtk_tree_store_set( GTK_TREE_STORE( model ),
+					&iter,
+					FILE_BROWSER_SHOW_COL, TRUE,
+					-1 );
+		} else {
 			const gchar *ccat;
 			guint ndepth;
 
@@ -836,30 +673,16 @@ static void scan_directory( ScreemFileBrowser *browser,
 				ccat = "/";
 			}
 			ndepth = depth + 1;
-			
-			do {
-				result = gnome_vfs_directory_read_next( handle,
-									info );
-				if( result == GNOME_VFS_OK &&
-				    *info->name != '.' ) {
-					gchar *utfuri;
+		
+			while( ( result = gnome_vfs_directory_read_next( handle, info ) == GNOME_VFS_OK ) ) {
+				if( strcmp( info->name, "." ) &&
+				    strcmp( info->name, ".." ) ) {
 					gchar *nexturi;
 
-					if( ! g_utf8_validate( info->name, -1, NULL ) ) {
-						utfuri = g_filename_to_utf8( info->name, -1, NULL, NULL, NULL );
-					} else {
-						utfuri = g_strdup( info->name );
-					}
-					/* yuk */
-					if( ! utfuri ) {
-						continue;
-					}
-					
 					nexturi = g_strconcat( uri,
 							ccat,
-							utfuri,
+							info->name,
 							NULL );
-					g_free( utfuri );
 					scan_directory( browser, &it,
 							nexturi, 
 							info->mime_type,
@@ -867,13 +690,13 @@ static void scan_directory( ScreemFileBrowser *browser,
 					
 					g_free( nexturi );
 				}
-			} while( result == GNOME_VFS_OK ); 
-		}
-		if( handle ) {
+			} 
 			gnome_vfs_directory_close( handle );
-			gnome_vfs_file_info_unref( info );
 		}
 	}
+	if( info ) {
+		gnome_vfs_file_info_unref( info );
+	}
 	g_free( parent_uri );
 
 }
@@ -892,9 +715,6 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 	GtkTreeIter *parent;
 	GtkTreeModel *model;
 	GnomeVFSFileInfo *info;
-	BrowserNodeType btype;
-	gchar *base;
-	gboolean hidden;
 	int mode;
 	BrowserNode *node;
 	GnomeVFSResult res;
@@ -902,17 +722,9 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 	if( ! data ) {
 		return;
 	}
-
 	browser = SCREEM_FILE_BROWSER( data );
 	mode = browser->details->mode;
 	
-	base = g_path_get_basename( info_uri );
-	hidden = FALSE;
-	if( base ) {
-		hidden = ( *base == '.' );
-		g_free( base );
-	}
-	
 	switch( type ) {
 	case GNOME_VFS_MONITOR_EVENT_CHANGED:
 		break;
@@ -930,7 +742,9 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 		
 			gtk_tree_store_remove( GTK_TREE_STORE( model ),
 					       &it );
-			browser_node_free( node );
+			if( node ) {
+				browser_node_free( node );
+			}
 		}
 		break;
 	case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
@@ -947,8 +761,7 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 		}
 	
 		parent = NULL;
-		if( ( ! ( mode & FILE_BROWSE_FLAT ) ) &&
-			find_iter( model, monitor_uri, NULL, &pit ) ) {
+		if( find_iter( model, monitor_uri, NULL, &pit ) ) {
 			/* got the parent */
 			parent = &pit;
 		}
@@ -966,19 +779,18 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 		 * signal being sent, and us attempting to get file
 		 * info */
 		if( res != GNOME_VFS_OK ) {
+			gnome_vfs_file_info_unref( info );
 			break;
 		}
 		
 		if( info->type == GNOME_VFS_FILE_TYPE_DIRECTORY ) {
-			btype = BROWSER_NODE_DIRECTORY;
-		} else {
-			btype = BROWSER_NODE_FILE;
-		}
-	
-		if( btype == BROWSER_NODE_DIRECTORY &&
-		    ( ! (mode & FILE_BROWSE_FLAT ) ) && ! hidden ) {
 			if( ! (mode & FILE_BROWSE_RECURSE) ) {
+				/* dummy node */
 				gtk_tree_store_append( GTK_TREE_STORE( model ), &iter, parent );
+				gtk_tree_store_set( GTK_TREE_STORE( model ),
+					&iter,
+					FILE_BROWSER_SHOW_COL, TRUE,
+					-1 );
 	
 			} else {
 				/* scan at depth 1 to
@@ -989,38 +801,12 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 						"x-directory/normal", 
 							1, mode );
 			}
-		} else if( ! hidden ) {
-			gtk_tree_store_append( GTK_TREE_STORE( model ),
-					       &it, parent );
-			
-			/* we set a depth of 1 so that in flat mode we 
-			 * don't try and monitor info_uri */
-			node = browser_node_new( browser, 1, btype, 
-					info_uri, info->mime_type );
-			gtk_tree_store_set(GTK_TREE_STORE( model ),
-					   &it,
-					   FILE_BROWSER_NAME_COL,
-					   node->basename,
-					   FILE_BROWSER_NODE_COL, node,
-					   FILE_BROWSER_ICON_COL,
-				 	   node->close,
-				 	   FILE_BROWSER_URI_COL,
-				 	   node->pathname,
-				    	   FILE_BROWSER_MIME_COL, 
-					   node->mime_type,
-					   -1 );
-
-			g_signal_emit( G_OBJECT( browser ),
-				       screem_file_browser_signals[ ADDED ],
-				       0,
-				       info_uri, node->mime_type, &it );
 		} else {
-			g_signal_emit( G_OBJECT( browser ),
-					screem_file_browser_signals[ ADDED ],
-					0,
-					info_uri, NULL, NULL );
-		}
-			
+			scan_directory_add_file( browser,
+					parent, info_uri,
+					info->mime_type, 1 );
+		} 
+
 		gnome_vfs_file_info_unref( info );
 		break;
 	case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
@@ -1117,23 +903,17 @@ static void screem_file_browser_set_icons( ScreemFileBrowser *browser,
 		}
 		
 		if( uri && node ) {
-			if( node->close ) {
-				g_object_unref( node->close );
-			} 
-			pixbuf = screem_file_browser_get_icon( browser,
-								 uri,
-								 ICON_X_SIZE,
-								 ICON_Y_SIZE,
-								FALSE,
-								NULL );
-			
-			node->close = pixbuf;
-			
+			pixbuf = screem_icon_cache_get_pixbuf( browser->details->cache,
+					uri, mimetype,
+					SCREEM_ICON_CACHE_DEFAULT_SIZE );
 			gtk_tree_store_set( GTK_TREE_STORE( model ), it,
 				    	    FILE_BROWSER_MIME_COL, 
 					    mimetype,
 					    FILE_BROWSER_ICON_COL, 
 					    pixbuf, -1 );
+			if( pixbuf ) {
+				g_object_unref( pixbuf );
+			}
 		}		
 		g_signal_emit( G_OBJECT( browser ),
 			       screem_file_browser_signals[ ICON_CHANGE ], 0,
@@ -1152,8 +932,7 @@ static void screem_file_browser_set_icons( ScreemFileBrowser *browser,
 }
 
 
-#ifdef HAVE_ICON_THEME
-static void screem_file_browser_theme_change( GnomeIconTheme *theme,
+static void screem_file_browser_theme_change( ScreemIconCache *cache,
 						gpointer data )
 {
 	ScreemFileBrowser *browser;
@@ -1161,11 +940,7 @@ static void screem_file_browser_theme_change( GnomeIconTheme *theme,
 	browser = SCREEM_FILE_BROWSER( data );
 
 	screem_file_browser_set_icons( browser, NULL );
-
-	g_signal_emit( G_OBJECT( browser ),
-		       screem_file_browser_signals[ THEME_CHANGE ], 0 );
 }
-#endif
 
 static gint screem_file_browser_compare_func( GtkTreeModel *model,
 					      GtkTreeIter *a,
@@ -1200,11 +975,6 @@ static gint screem_file_browser_compare_func( GtkTreeModel *model,
                 ret = strcmp( anode->pathname, bnode->pathname );
 	}
 
-	if( ( anode && anode->basename && ! strcmp( "..", anode->basename ) ) ||
-	    ( bnode &&  bnode->basename && ! strcmp( "..", bnode->basename ) ) ) {
-		ret = 1;
-	}
-
         return ret;
 }
 
@@ -1249,31 +1019,105 @@ static gboolean screem_file_browser_clear( GtkTreeModel *model,
 					   GtkTreeIter *it,
 					   gpointer data )
 {
+	ScreemFileBrowser *browser;
+	ScreemFileBrowserDetails *details;
         BrowserNode *node;
+
+        gtk_tree_model_get( model, it, 
+				FILE_BROWSER_NODE_COL, &node, -1 );
+       
+	if( node ) {
+		browser_node_free( node );
+	}
+	
+	browser = SCREEM_FILE_BROWSER( data );
+	details = browser->details;
+
+	if( details->filtered ) {
+		gtk_tree_store_set( GTK_TREE_STORE( model ), 
+				it, FILE_BROWSER_NODE_COL,
+				NULL, -1 );
+	}
+	
+	return FALSE;
+}
+
+static gboolean screem_file_browser_filter( GtkTreeModel *model,
+		GtkTreeIter *it, ScreemFileBrowser *browser )
+{
+	ScreemFileBrowserDetails *details;
+        BrowserNode *node;
+	const gchar *base;
+	gboolean visible;
+	const gchar *suffix;
+
+	g_return_val_if_fail( SCREEM_IS_FILE_BROWSER( browser ), FALSE );
+	g_return_val_if_fail( GTK_IS_TREE_MODEL( model ), FALSE );
+	g_return_val_if_fail( it != NULL, FALSE );
+	
+	details = browser->details;
+
+	if( ! details->filtered ) {
+		return FALSE;
+	}
 	
         gtk_tree_model_get( model, it, 
 			FILE_BROWSER_NODE_COL, &node, -1 );
        
-	if( node ) {
-		browser_node_free( node );
+	visible = TRUE;
+		
+	if( node && ( base = node->basename ) ) {
+		if( ! details->show_hidden ) {
+			visible &= *base != '.';
+		}
+		if( ! details->show_backups ) {
+			suffix = strstr( base, details->backup_suffix );
+			
+			if( suffix ) {
+				visible &= strcmp( suffix,
+						details->backup_suffix );
+			}	
+		}
+	} 	
+
+	return visible;
+}
+
+static gboolean refilter( GtkTreeModel *model, GtkTreePath *path,
+		GtkTreeIter *iter, ScreemFileBrowser *browser )
+{
+	gboolean cur;
+	gboolean visible;
+
+	gtk_tree_model_get( model, iter, FILE_BROWSER_SHOW_COL, &cur,
+			-1 );
+	visible = screem_file_browser_filter( model, iter, browser );
+	if( visible != cur ) {
+		gtk_tree_store_set( GTK_TREE_STORE( model ), iter,
+				FILE_BROWSER_SHOW_COL, visible, -1 );
 	}
 
 	return FALSE;
 }
 
+
+
 /* G Object stuff */
 
-#define PARENT_TYPE G_TYPE_OBJECT
+G_DEFINE_TYPE( ScreemFileBrowser, screem_file_browser, G_TYPE_OBJECT )
+
+static void screem_file_browser_finalize( GObject *object );
+static void screem_file_browser_set_prop( GObject *object, 
+		guint prop_id, const GValue *value, GParamSpec *spec );
+static void screem_file_browser_get_prop( GObject *object, 
+		guint prop_id, GValue *value, GParamSpec *spec );
 
-static gpointer parent_class;
 
 static void
 screem_file_browser_class_init( ScreemFileBrowserClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS( klass );
 
-	parent_class = g_type_class_peek_parent( klass );
-
 	object_class->finalize = screem_file_browser_finalize;
 	object_class->get_property = screem_file_browser_get_prop;
 	object_class->set_property = screem_file_browser_set_prop;
@@ -1312,30 +1156,17 @@ screem_file_browser_class_init( ScreemFileBrowserClass *klass)
 			      G_TYPE_STRING,
 			      G_TYPE_STRING,
 			      G_TYPE_POINTER );
-	screem_file_browser_signals[ THEME_CHANGE ] = 
-		g_signal_new( "theme_change",
-			      G_OBJECT_CLASS_TYPE( object_class ),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET( ScreemFileBrowserClass, 
-					       theme_change ),
-			      NULL, NULL,
-			      screem_marshal_VOID__VOID,
-			      G_TYPE_NONE, 0  );
 }
 
 static void
 screem_file_browser_init( ScreemFileBrowser *browser )
 {
+	ScreemFileBrowserDetails *details;
+	
 	if( ! cache_mutex ) {
 		cache_mutex = g_mutex_new();
 	}
 	g_mutex_lock( cache_mutex );
-	if( ! icon_cache ) {
-		icon_cache = g_hash_table_new_full( g_str_hash, 
-				g_str_equal, 
-				(GDestroyNotify)g_free, 
-				(GDestroyNotify)g_object_unref );
-	}
 	if( ! mime_cache ) {
 		mime_cache = g_hash_table_new_full( g_str_hash, 
 				g_str_equal, 
@@ -1345,32 +1176,41 @@ screem_file_browser_init( ScreemFileBrowser *browser )
 
 	g_mutex_unlock( cache_mutex );
 
-	browser->details = g_new0( ScreemFileBrowserDetails, 1 );
+	details = browser->details = g_new0( ScreemFileBrowserDetails, 1 );
 
-	browser->details->model = gtk_tree_store_new( FILE_BROWSER_MAX_COL,
-						      G_TYPE_STRING,
-						      GDK_TYPE_PIXBUF,
-						      G_TYPE_POINTER,
-						      G_TYPE_STRING,
-						      G_TYPE_POINTER,
-						      G_TYPE_STRING,
-						      G_TYPE_POINTER);
+	details->show_hidden = FALSE;
+	details->show_backups = FALSE;
+	details->backup_suffix = g_strdup( "~" );
+	
+	details->model = gtk_tree_store_new( FILE_BROWSER_MAX_COL,
+			G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_POINTER,
+			G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING, 
+			G_TYPE_POINTER, G_TYPE_BOOLEAN );
 
-	browser->details->sorted = 
-		GTK_TREE_MODEL( gtk_tree_model_sort_new_with_model( GTK_TREE_MODEL( browser->details->model ) ) );
+	details->filtered = gtk_tree_model_filter_new( GTK_TREE_MODEL( details->model ), NULL );
+
+#if 0
+	gtk_tree_model_filter_set_visible_func( GTK_TREE_MODEL_FILTER( details->filtered ),
+			(GtkTreeModelFilterVisibleFunc)screem_file_browser_filter,
+			browser, NULL );
+#else
+	gtk_tree_model_filter_set_visible_column( GTK_TREE_MODEL_FILTER( details->filtered ), FILE_BROWSER_SHOW_COL );
+#endif
+	
+
+	details->sorted = 
+		GTK_TREE_MODEL( gtk_tree_model_sort_new_with_model( GTK_TREE_MODEL( details->filtered ) ) );
 
 	screem_file_browser_set_sort_func( browser, 
 					   screem_file_browser_compare_func,
 					   browser );
 
-
-	if( ! gtk_theme ) {
-		gtk_theme = gtk_icon_theme_get_default();
-	}
-	g_signal_connect( G_OBJECT( gtk_theme ),
-			  "changed",
-			  G_CALLBACK( screem_file_browser_theme_change ),
-			  browser );
+	/* quick hack so we can get at the filtered / sorted models
+	 * easily from the base model */
+	g_object_set_data( G_OBJECT( details->model ), "filtered",
+			details->filtered );
+	g_object_set_data( G_OBJECT( details->model ), "sorted",
+			details->sorted );
 }
 
 static void
@@ -1405,50 +1245,38 @@ static void
 screem_file_browser_finalize( GObject *object )
 {
 	ScreemFileBrowser *browser;
-
+	ScreemFileBrowserDetails *details;
+	ScreemIconCache *cache;
+	
 	browser = SCREEM_FILE_BROWSER( object );
+	details = browser->details;
 
-	g_signal_handlers_disconnect_matched( G_OBJECT( gtk_theme ),
-					      G_SIGNAL_MATCH_DATA,
-					      0, 0, NULL,
-					      NULL,
-					      browser );
+	g_object_unref( details->sorted );
+	g_object_unref( details->filtered );
 
-	g_object_unref( browser->details->sorted );
-
-	gtk_tree_model_foreach( GTK_TREE_MODEL( browser->details->model ), 
+	details->filtered = NULL;
+	
+	gtk_tree_model_foreach( GTK_TREE_MODEL( details->model ), 
 				screem_file_browser_clear,
 				browser );
-	gtk_tree_store_clear( GTK_TREE_STORE( browser->details->model ) );
+	gtk_tree_store_clear( GTK_TREE_STORE( details->model ) );
 
-	g_object_unref( browser->details->model );
+	g_object_unref( details->model );
 
-	g_free( browser->details );
-
-	G_OBJECT_CLASS( parent_class )->finalize( object );
-}
-
-GType screem_file_browser_get_type()
-{
-	static GType type = 0;
-
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemFileBrowserClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_file_browser_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemFileBrowser ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_file_browser_init
-		};
-
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemFileBrowser",
-					       &info, 0 );
+	cache = details->cache;
+	if( cache ) {
+		g_signal_handlers_disconnect_matched( G_OBJECT( cache ),
+				G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, 
+				browser );
+		g_object_unref( G_OBJECT( details->cache ) );
 	}
 
-	return type;
+	if( details->backup_suffix ) {
+		g_free( details->backup_suffix );
+	}
+	
+	g_free( details );
+
+	G_OBJECT_CLASS( screem_file_browser_parent_class )->finalize( object );
 }
+
