@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -187,8 +187,6 @@ void screem_link_view_depth( GtkWidget *widget );
 static void screem_link_view_class_init( ScreemLinkViewClass *klass );
 static void screem_link_view_init( ScreemLinkView *link_view );
 static void screem_link_view_finalize( GObject *link_view );
-static void screem_link_view_size_request( GtkWidget *widget, GtkRequisition *req );
-static void screem_link_view_realize( GtkWidget *widget );
 static void screem_link_view_hide( GtkWidget *widget );
 static void screem_link_view_show( GtkWidget *widget );
 static void screem_link_view_window_set( ScreemView *view );
@@ -245,21 +243,22 @@ static void screem_link_view_display( ScreemLinkView *link_view,
 				      ScreemPage *page,
 				      gboolean online )
 {
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 	GtkWidget *canvas;
 	GtkType type;
 	ScreemApplication *application;
-	GModule *uploadWiz;
+	ScreemPluginManager *manager;
+	ScreemPluginDetails *uploadWiz;
 	SiteSyncState screem_site_sync_status;
 	SiteSyncState *status;
 	ScreemWindow *window;
 
-	private = link_view->private;
+	priv = link_view->priv;
 
-	canvas = private->canvas;
+	canvas = priv->canvas;
 
-	private->checked_files = NULL;
-	private->upload_table = NULL;
+	priv->checked_files = NULL;
+	priv->upload_table = NULL;
 
 	uploadWiz = NULL;
 
@@ -268,18 +267,20 @@ static void screem_link_view_display( ScreemLinkView *link_view,
 		      NULL );
 	
 	application = window->application;
-	uploadWiz = screem_plugin_get_sitecopy( application );
+	manager = screem_application_get_plugin_manager( application );
+	uploadWiz = screem_plugin_manager_get_plugin_by_name( manager,
+			"ScreemSitecopy" );
 	
-	if( private->link_view_tree ) {
-		g_node_traverse(private->link_view_tree,
+	if( priv->link_view_tree ) {
+		g_node_traverse(priv->link_view_tree,
 				G_IN_ORDER, G_TRAVERSE_ALL, -1,
 				(GNodeTraverseFunc)screem_link_view_stop_check,
 				link_view );
-		remove_data ( private->link_view_tree );
-		g_node_destroy( private->link_view_tree );
-		private->link_view_tree = NULL;
-		gtk_object_destroy( GTK_OBJECT( private->root ) );
-		private->root = NULL;
+		remove_data ( priv->link_view_tree );
+		g_node_destroy( priv->link_view_tree );
+		priv->link_view_tree = NULL;
+		gtk_object_destroy( GTK_OBJECT( priv->root ) );
+		priv->root = NULL;
 	}
 
 	/* we must have a page open */
@@ -287,15 +288,15 @@ static void screem_link_view_display( ScreemLinkView *link_view,
 		return;
 	}
 
-	private->site = site;
-	private->page = page;
+	priv->site = site;
+	priv->page = page;
 
 	if( uploadWiz ) {
 		status = &screem_site_sync_status;
-		g_module_symbol( uploadWiz, "screem_site_get_sync_status", 
+		g_module_symbol( uploadWiz->mod, "screem_site_get_sync_status", 
 				 (gpointer*)status );
 		
-		screem_site_sync_status( site, &private->upload_table );
+		screem_site_sync_status( site, &priv->upload_table );
 
 		/* if failed table will be NULL, otherwise the status
 		   of a file can be obtained via 
@@ -304,23 +305,23 @@ static void screem_link_view_display( ScreemLinkView *link_view,
 	}
 
 	gnome_canvas_set_pixels_per_unit( GNOME_CANVAS( canvas ),
-					  private->zoom/100.0 );
+					  priv->zoom/100.0 );
 
-	private->max_x = 0.0;
-	private->max_y = 0.0;
-	private->min_x = 0.0;
-	private->min_y = 0.0;
+	priv->max_x = 0.0;
+	priv->max_y = 0.0;
+	priv->min_x = 0.0;
+	priv->min_y = 0.0;
 
 	type = gnome_canvas_group_get_type();
-	private->root = G_OBJECT( gnome_canvas_item_new( GNOME_CANVAS_GROUP( private->mainroot ),
+	priv->root = G_OBJECT( gnome_canvas_item_new( GNOME_CANVAS_GROUP( priv->mainroot ),
 							 type,
 							 "x", 0.0,
 							 "y", 0.0,
 							 NULL ) );
 
 	/* we now need to build up the tree of links */
-	private->checked_files = g_hash_table_new( g_str_hash, g_str_equal );
-	private->link_view_tree = build_link_tree( link_view,
+	priv->checked_files = g_hash_table_new( g_str_hash, g_str_equal );
+	priv->link_view_tree = build_link_tree( link_view,
 						   window,
 						   online,
 						   screem_page_get_pathname( page ),
@@ -328,43 +329,43 @@ static void screem_link_view_display( ScreemLinkView *link_view,
 	
 	/* we can destroy the checked_files hash table now
 	   as we no longer need it */
-	g_hash_table_destroy( private->checked_files );
+	g_hash_table_destroy( priv->checked_files );
 
 	/* GNode tree built, now create the icons */
-	build_icons( link_view, private->link_view_tree,
-		     GNOME_CANVAS_GROUP( private->root ),
+	build_icons( link_view, priv->link_view_tree,
+		     GNOME_CANVAS_GROUP( priv->root ),
 		     0.0, 0.0, 0,0, 1, 0 );
 
 	type = gnome_canvas_rect_get_type();
 	
-	gnome_canvas_item_raise_to_top( GNOME_CANVAS_ITEM( private->root ) );
+	gnome_canvas_item_raise_to_top( GNOME_CANVAS_ITEM( priv->root ) );
 	
 	gnome_canvas_set_scroll_region( GNOME_CANVAS( canvas ),
-					private->min_x - 20,
-					private->min_y - 20,
-					private->max_x + 20,
-					private->max_y + 20 );
+					priv->min_x - 20,
+					priv->min_y - 20,
+					priv->max_x + 20,
+					priv->max_y + 20 );
 
 	/* if we aren't showing text labels hide them now */
 
-	g_node_traverse( private->link_view_tree,G_IN_ORDER,G_TRAVERSE_ALL,-1,
+	g_node_traverse( priv->link_view_tree,G_IN_ORDER,G_TRAVERSE_ALL,-1,
 			 (GNodeTraverseFunc)traverse_for_remote, link_view );
 }
 
 static gboolean traverse_for_remote( GNode *node, ScreemLinkView *link_view )
 {
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 	ScreemLinkViewHtml *html;
 	gboolean online;
 
 	g_object_get( G_OBJECT( link_view ), "online", &online, NULL );
 
-	private = link_view->private;
+	priv = link_view->priv;
 	html = SCREEM_LINK_VIEW_HTML( node->data );
 
 	html->status = REMOTE_NOT_CHECKED;
 
-	if( html->remoteAddr && ( ! online || ! private->check_external ) ) {
+	if( html->remoteAddr && ( ! online || ! priv->check_external ) ) {
 		gnome_canvas_item_set( html->icon, "fill_color", "grey", NULL );
 	} else {
 		gnome_canvas_item_set( html->icon, "fill_color", "orange",
@@ -386,7 +387,7 @@ screem_link_view_add_key( ScreemLinkView *link_view,
 			  GnomeCanvas *legend, GnomeCanvasGroup *root, 
 			  gdouble x, gdouble y )
 {
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 	GnomeCanvasGroup *group;
 	GnomeCanvasItem *image;
 	GnomeCanvasItem *text;
@@ -419,7 +420,7 @@ screem_link_view_add_key( ScreemLinkView *link_view,
 	gint i;
 
 
-	private = link_view->private;
+	priv = link_view->priv;
 
 	/* create the group */
 	type = gnome_canvas_group_get_type();
@@ -430,7 +431,7 @@ screem_link_view_add_key( ScreemLinkView *link_view,
    
 	/* create the image */
 	y2 = 4.0;
-	font = gconf_client_get_string( private->client,
+	font = gconf_client_get_string( priv->client,
 					"/apps/screem/editor/font", NULL );
 	w = 0;
 	for( i = 0; icons[ i ] != -1; ++ i ) {
@@ -445,7 +446,7 @@ screem_link_view_add_key( ScreemLinkView *link_view,
 					      gettext( names[ i ] ),
 					       "fill_color", "black",
 					      NULL );
-		if( font ) {
+		if( 0 && font ) {
 			g_object_set( G_OBJECT( text ), "font", font,
 					NULL );
 		}
@@ -478,7 +479,7 @@ static GNode *build_link_tree( ScreemLinkView *view,
 			       const gchar *pathname,
 			       GNode *parent, gint level )
 {
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 	ScreemSite *site;
 	const gchar *sitepath;
 	GNode *ret = NULL;
@@ -488,9 +489,9 @@ static GNode *build_link_tree( ScreemLinkView *view,
 	GNode *node = NULL;
 	ScreemDTD *dtd;
 
-	private = view->private;
+	priv = view->priv;
 
-	site = SCREEM_SITE( private->site );
+	site = SCREEM_SITE( priv->site );
 	sitepath = screem_site_get_pathname( site );
 
 	/* pathname will be NULL for a new blank document, hence
@@ -564,11 +565,11 @@ static GNode *build_link_tree( ScreemLinkView *view,
 		}
 		
 		/* have we already processed this page before? */
-		processed = GPOINTER_TO_INT( g_hash_table_lookup( private->checked_files, pathname ) );
+		processed = GPOINTER_TO_INT( g_hash_table_lookup( priv->checked_files, pathname ) );
 		if( processed <= 0 ) {
 			gchar *text;
 			
-			g_hash_table_replace( private->checked_files,
+			g_hash_table_replace( priv->checked_files,
 					      g_strdup( pathname ),
 					      GINT_TO_POINTER( level + 1 ) );
 			
@@ -614,15 +615,15 @@ static GNode *build_link_tree( ScreemLinkView *view,
 				
 				gnome_vfs_uri_unref( uri );
 				
-				processed = GPOINTER_TO_INT( g_hash_table_lookup( private->checked_files, full ) );
+				processed = GPOINTER_TO_INT( g_hash_table_lookup( priv->checked_files, full ) );
 				
-				if( *full && level < private->depth &&
+				if( *full && level < priv->depth &&
 				    ! processed ) {
 					this_level = g_slist_prepend(this_level,
 								     full );
 					
-					if( ! g_hash_table_lookup( private->checked_files, full ) ) {
-						g_hash_table_insert( private->checked_files, full, GINT_TO_POINTER( -1 ) );
+					if( ! g_hash_table_lookup( priv->checked_files, full ) ) {
+						g_hash_table_insert( priv->checked_files, full, GINT_TO_POINTER( -1 ) );
 					}
 				} else {
 					g_free( full );
@@ -649,7 +650,7 @@ static void build_icons( ScreemLinkView *link_view,
 			 gdouble root_y, gdouble root_angle, gint level, 
 			 gint numOfNodesAtLevel, gint nthNode )
 {
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 	gint siblings;
 	gint currentNode = 0;
 	GNode *temp;
@@ -669,7 +670,7 @@ static void build_icons( ScreemLinkView *link_view,
 	
 	gdouble angle = 0.0;
 
-	private = link_view->private;
+	priv = link_view->priv;
 
 	g_assert( node );
 
@@ -697,7 +698,7 @@ static void build_icons( ScreemLinkView *link_view,
 		screem_link_view_html_build_icon(SCREEM_LINK_VIEW_HTML(node->data),
 						 root_x, root_y, angle, &width, 
 						 root );
-	} else if( level <= private->depth ) {
+	} else if( level <= priv->depth ) {
 		/* this is the links from the root icon, create an icon
 		   then call build_icons again */
 							   
@@ -795,7 +796,7 @@ static void build_icons( ScreemLinkView *link_view,
 static gint background_event( GtkWidget *canvas, GdkEventButton *event, 
 			      ScreemLinkView *link_view )
 {
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 	GtkWidget *menu;
 	GnomeCanvasItem *item;
 	gdouble x;
@@ -805,7 +806,7 @@ static gint background_event( GtkWidget *canvas, GdkEventButton *event,
 
 	g_object_get( G_OBJECT( link_view ), "window", &window, NULL );
 
-	private = link_view->private;
+	priv = link_view->priv;
 
 	gnome_canvas_window_to_world( GNOME_CANVAS( canvas ),
 				      event->x, event->y,
@@ -871,10 +872,10 @@ static void free_node_list( GNode* node, gpointer data )
 static gboolean screem_link_view_stop_check( GNode *node,
 					     ScreemLinkView *link_view )
 {
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 	ScreemLinkViewHtml *html;
 	
-	private = link_view->private;
+	priv = link_view->priv;
 	html = SCREEM_LINK_VIEW_HTML( node->data );
 	
 	if( html->handle ) {
@@ -990,7 +991,7 @@ static void screem_link_view_async_close_cb( GnomeVFSAsyncHandle *handle,
 
 static gboolean link_view_toggle_labels(GNode *node, ScreemLinkView *link_view)
 {
-	if( link_view->private->show_links ) {
+	if( link_view->priv->show_links ) {
 		gnome_canvas_item_show(((ScreemLinkViewHtml*)node->data)->label);
 	} else {
 		gnome_canvas_item_hide(((ScreemLinkViewHtml*)node->data)->label);
@@ -1004,14 +1005,14 @@ static gboolean link_view_toggle_labels(GNode *node, ScreemLinkView *link_view)
 static void screem_link_view_sidebar_switch( GtkWidget *widget, gpointer data )
 {
 	ScreemLinkView *link_view;
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 
 	link_view = SCREEM_LINK_VIEW( widget );
-	private = link_view->private;
+	priv = link_view->priv;
 
-	private->show_links =gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( data ) );
+	priv->show_links =gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( data ) );
 		
-	g_node_traverse( private->link_view_tree, G_IN_ORDER, G_TRAVERSE_ALL, -1,
+	g_node_traverse( priv->link_view_tree, G_IN_ORDER, G_TRAVERSE_ALL, -1,
 			 (GNodeTraverseFunc)link_view_toggle_labels,
 			 link_view );
 }
@@ -1019,42 +1020,42 @@ static void screem_link_view_sidebar_switch( GtkWidget *widget, gpointer data )
 static void screem_link_view_sidebar_zoom( GtkWidget *widget )
 {
 	ScreemLinkView *link_view;
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 
 	link_view = SCREEM_LINK_VIEW( widget );
-	private = link_view->private;
+	priv = link_view->priv;
 
-	private->zoom = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( private->zoom_control ) );
-	gnome_canvas_set_pixels_per_unit( GNOME_CANVAS( private->canvas ), 
-					  private->zoom/100.0 );
+	priv->zoom = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( priv->zoom_control ) );
+	gnome_canvas_set_pixels_per_unit( GNOME_CANVAS( priv->canvas ), 
+					  priv->zoom/100.0 );
 }
 
 static void screem_link_view_check_toggle( GtkWidget *widget )
 {
 	ScreemLinkView *link_view;
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 	GConfClient *client;
 
 	link_view = SCREEM_LINK_VIEW( widget );
-	private = link_view->private;
+	priv = link_view->priv;
 
 	client = gconf_client_get_default();
 
-	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( private->check_links ) ) ) {
+	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( priv->check_links ) ) ) {
 		/* yes */
-		private->check_external = TRUE;
+		priv->check_external = TRUE;
 		gconf_client_set_bool( client, "/apps/screem/linkview/check_external",
 				       TRUE, NULL );
-		g_node_traverse(private->link_view_tree,
+		g_node_traverse(priv->link_view_tree,
 				G_IN_ORDER, G_TRAVERSE_ALL, -1,
 				(GNodeTraverseFunc)screem_link_view_stop_check,
 				link_view );
-		g_node_traverse( private->link_view_tree,G_IN_ORDER,G_TRAVERSE_ALL,-1,
+		g_node_traverse( priv->link_view_tree,G_IN_ORDER,G_TRAVERSE_ALL,-1,
 				 (GNodeTraverseFunc)traverse_for_remote, link_view );
 	} else {
 		/* no */
-		private->check_external = FALSE;
-		g_node_traverse(private->link_view_tree,
+		priv->check_external = FALSE;
+		g_node_traverse(priv->link_view_tree,
 				G_IN_ORDER, G_TRAVERSE_ALL, -1,
 				(GNodeTraverseFunc)screem_link_view_stop_check,
 				link_view );
@@ -1068,12 +1069,12 @@ static void screem_link_view_check_toggle( GtkWidget *widget )
 void screem_link_view_depth( GtkWidget *widget )
 {
 	ScreemLinkView *link_view;
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 
 	link_view = SCREEM_LINK_VIEW( widget );
-	private = link_view->private;
+	priv = link_view->priv;
 
-	private->depth = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( private->depth_control ) );
+	priv->depth = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( priv->depth_control ) );
 
 	screem_view_display( SCREEM_VIEW( link_view ) );
 }
@@ -1084,24 +1085,24 @@ static void screem_link_view_check_external_notify( GConfClient *client,
 						    gpointer data )
 {
 	ScreemLinkView *view;
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 
 	view = SCREEM_LINK_VIEW( data );
-	private = view->private;
+	priv = view->priv;
 
 	if( entry->value->type == GCONF_VALUE_BOOL ) {
-		gboolean state = private->check_external;
+		gboolean state = priv->check_external;
 
-		private->check_external = gconf_value_get_bool( entry->value );
+		priv->check_external = gconf_value_get_bool( entry->value );
 
-		if( private->check_external != state ) {
+		if( priv->check_external != state ) {
 			GSList *list;
 			GtkWidget *radio;
 
-			list = gtk_radio_button_get_group( GTK_RADIO_BUTTON( private->check_links ) );
-			radio = private->check_links;
+			list = gtk_radio_button_get_group( GTK_RADIO_BUTTON( priv->check_links ) );
+			radio = priv->check_links;
 
-			if( ! private->check_external ) {
+			if( ! priv->check_external ) {
 				radio = GTK_WIDGET( list->data );
 			}
 
@@ -1112,9 +1113,10 @@ static void screem_link_view_check_external_notify( GConfClient *client,
 }
 
 /* G Object stuff */
-#define PARENT_TYPE SCREEM_TYPE_VIEW
 
-static gpointer parent_class;
+static void screem_view_iface_init( ScreemView *iface );
+
+G_DEFINE_TYPE( ScreemLinkView, screem_link_view, SCREEM_TYPE_VIEW )
 
 static void screem_link_view_class_init( ScreemLinkViewClass *klass )
 {
@@ -1124,7 +1126,6 @@ static void screem_link_view_class_init( ScreemLinkViewClass *klass )
 	GParamSpec *pspec;
 
 	object_class = G_OBJECT_CLASS( klass );
-	parent_class = g_type_class_peek_parent( klass );
 	widget_class = (GtkWidgetClass *)klass;
 
 	G_OBJECT_CLASS( object_class )->get_property = screem_link_view_get_prop;
@@ -1170,13 +1171,16 @@ static void screem_link_view_class_init( ScreemLinkViewClass *klass )
 
 	widget_class->hide = screem_link_view_hide;
 	widget_class->show = screem_link_view_show;
-	widget_class->realize = screem_link_view_realize;
-	widget_class->size_request = screem_link_view_size_request;
+}
+
+static void screem_view_iface_init( ScreemView *iface )
+{
+	iface->display = screem_link_view_display_view;
 }
 
 static void screem_link_view_init( ScreemLinkView *link_view )
 {
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 
 	GtkWidget *label;
 	GtkWidget *legend;
@@ -1187,24 +1191,27 @@ static void screem_link_view_init( ScreemLinkView *link_view )
 	GdkColor back;
 	GtkWidget *window;
 
-	SCREEM_VIEW( link_view )->window_set = screem_link_view_window_set;
-	SCREEM_VIEW( link_view )->display = screem_link_view_display_view;
-
-	link_view->private = private = g_new0( ScreemLinkViewPrivate, 1 );
+	screem_view_iface_init( SCREEM_VIEW( link_view ) );
 	
-	link_view->private->mainbox = gtk_vbox_new( FALSE, 0 );
-	
-	private->client = gconf_client_get_default();
+	g_signal_connect( G_OBJECT( link_view ), "notify::window",
+			G_CALLBACK( screem_link_view_window_set ),
+			NULL );
 
-	gconf_client_add_dir( private->client, "/apps/screem/editor",
+	link_view->priv = priv = g_new0( ScreemLinkViewPrivate, 1 );
+	
+	link_view->priv->mainbox = gtk_vbox_new( FALSE, 0 );
+	
+	priv->client = gconf_client_get_default();
+
+	gconf_client_add_dir( priv->client, "/apps/screem/editor",
 			      GCONF_CLIENT_PRELOAD_NONE, NULL );
-	gconf_client_add_dir( private->client, "/apps/screem/linkview",
+	gconf_client_add_dir( priv->client, "/apps/screem/linkview",
 			      GCONF_CLIENT_PRELOAD_NONE, NULL );
 
 
-       	private->zoom = 100;
-	private->depth = 3;
-	private->show_links = TRUE;
+       	priv->zoom = 100;
+	priv->depth = 3;
+	priv->show_links = TRUE;
 
 	window = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( window ),
@@ -1212,39 +1219,39 @@ static void screem_link_view_init( ScreemLinkView *link_view )
 					GTK_POLICY_AUTOMATIC );
 	gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW( window ),
 						GTK_SHADOW_IN );
-	private->canvas = gnome_canvas_new(); /*_aa();*/
+	priv->canvas = gnome_canvas_new(); /*_aa();*/
 
 	back.red = 0xffff;
 	back.green = 0xffff;
 	back.blue = 0xffff;
-	gtk_widget_modify_bg( GTK_WIDGET( private->canvas ),
+	gtk_widget_modify_bg( GTK_WIDGET( priv->canvas ),
 			      GTK_STATE_NORMAL, &back );
-	gtk_widget_show( private->canvas );
-	private->mainroot = 
-		G_OBJECT( gnome_canvas_root( GNOME_CANVAS( private->canvas ) ) );
+	gtk_widget_show( priv->canvas );
+	priv->mainroot = 
+		G_OBJECT( gnome_canvas_root( GNOME_CANVAS( priv->canvas ) ) );
 
-	gtk_widget_add_events( private->canvas, 
+	gtk_widget_add_events( priv->canvas, 
 			       GDK_BUTTON_PRESS_MASK );
 
-	g_signal_connect( G_OBJECT( private->canvas ), "button_press_event",
+	g_signal_connect( G_OBJECT( priv->canvas ), "button_press_event",
 			  G_CALLBACK( background_event ), 
 			  link_view );
 
 	/* should we be checking external links? */
-	private->check_external = 
-		gconf_client_get_bool( private->client, 
+	priv->check_external = 
+		gconf_client_get_bool( priv->client, 
 				       "/apps/screem/linkview/check_external",
 				       NULL );
 	
-	private->check_external_notify = 
-		gconf_client_notify_add( private->client,
+	priv->check_external_notify = 
+		gconf_client_notify_add( priv->client,
 					 "/apps/screem/linkview/check_external",
 					 screem_link_view_check_external_notify,
 					 link_view, NULL, NULL );
 
 
 	/* create the side panel */
-	private->panel = gtk_vbox_new( FALSE, 0 );
+	priv->panel = gtk_vbox_new( FALSE, 0 );
 
 	legend = gnome_canvas_new();
 	gnome_canvas_set_center_scroll_region( GNOME_CANVAS( legend ), FALSE );
@@ -1254,24 +1261,24 @@ static void screem_link_view_init( ScreemLinkView *link_view )
 							  GNOME_CANVAS( legend ), 
 							  legend_root, 6.0, 0 );
 	gnome_canvas_item_raise_to_top( key );
-	gtk_box_pack_start( GTK_BOX( private->panel ), legend, TRUE, TRUE, 0 );
+	gtk_box_pack_start( GTK_BOX( priv->panel ), legend, TRUE, TRUE, 0 );
 
 	box = gtk_hbox_new( FALSE, 6 );
-	gtk_box_pack_start( GTK_BOX( private->panel ), box, TRUE, TRUE, 0 );
+	gtk_box_pack_start( GTK_BOX( priv->panel ), box, TRUE, TRUE, 0 );
 	label = gtk_label_new( _( "Zoom:" ) );
 	gtk_widget_show( label );
 	gtk_box_pack_start( GTK_BOX( box ), label, FALSE, FALSE, 0 );
 
 	adjustment = GTK_ADJUSTMENT( gtk_adjustment_new( 100.0, 20.0, 400.0, 20.0, 10.0, 10.0 ) );
-	private->zoom_control = gtk_spin_button_new( adjustment,
+	priv->zoom_control = gtk_spin_button_new( adjustment,
 						     1.0, /* climb rate */
 						     0 /* digits */ );
-	g_signal_connect_object( G_OBJECT( private->zoom_control ),
+	g_signal_connect_object( G_OBJECT( priv->zoom_control ),
 				 "changed",
 				 G_CALLBACK( screem_link_view_sidebar_zoom ),
 				 G_OBJECT( link_view ), G_CONNECT_SWAPPED );
-	gtk_widget_show( private->zoom_control );
-	gtk_box_pack_start( GTK_BOX( box ), private->zoom_control, 
+	gtk_widget_show( priv->zoom_control );
+	gtk_box_pack_start( GTK_BOX( box ), priv->zoom_control, 
 			    FALSE, FALSE, 0 );
 
 	label = gtk_label_new( _( "Depth:" ) );
@@ -1280,27 +1287,27 @@ static void screem_link_view_init( ScreemLinkView *link_view )
 
 	adjustment = GTK_ADJUSTMENT( gtk_adjustment_new( 3.0, 1.0, 10.0, 1.0, 
 							10.0, 10.0 ) );
-	private->depth_control = gtk_spin_button_new( adjustment,
+	priv->depth_control = gtk_spin_button_new( adjustment,
 						      1.0, /* climb rate */
 						      0 /* digits */ );
-	g_signal_connect_object( G_OBJECT( private->depth_control ),
+	g_signal_connect_object( G_OBJECT( priv->depth_control ),
 				 "changed",
 				 G_CALLBACK( screem_link_view_depth ),
 				 G_OBJECT( link_view ), G_CONNECT_SWAPPED );
-	gtk_widget_show( private->depth_control );
-	gtk_box_pack_start( GTK_BOX( box ), private->depth_control, 
+	gtk_widget_show( priv->depth_control );
+	gtk_box_pack_start( GTK_BOX( box ), priv->depth_control, 
 			    FALSE, FALSE, 0 );
 
-	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( private->depth_control ),
+	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( priv->depth_control ),
 					TRUE );
-	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( private->zoom_control ), 
+	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( priv->zoom_control ), 
 					TRUE );
 
 	label = gtk_check_button_new_with_label( _( "Show Labels" ) );
 	gtk_widget_show( label );
 	gtk_box_pack_start( GTK_BOX( box ), label, FALSE, FALSE, 0 );
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( label ),
-					private->show_links );
+					priv->show_links );
 	
 	g_signal_connect_object( G_OBJECT( label ), "clicked", 
 				 G_CALLBACK( screem_link_view_sidebar_switch ),
@@ -1311,21 +1318,21 @@ static void screem_link_view_init( ScreemLinkView *link_view )
 	gtk_box_pack_start( GTK_BOX( box), label, FALSE, FALSE, 0 );
 
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( label ),
-				      private->check_external );
+				      priv->check_external );
 	g_signal_connect_object( G_OBJECT( label ), "clicked", 
 				 G_CALLBACK( screem_link_view_check_toggle ),
 				 G_OBJECT( link_view ), G_CONNECT_SWAPPED );
 
-	private->check_links = label;
+	priv->check_links = label;
 
 	/* end of side panel */
 
-	gtk_container_add( GTK_CONTAINER( window ), private->canvas );
+	gtk_container_add( GTK_CONTAINER( window ), priv->canvas );
 	
-	gtk_box_pack_start( GTK_BOX( private->mainbox ), window, TRUE, TRUE, 0 );
-	gtk_box_pack_start( GTK_BOX( private->mainbox ), private->panel,
+	gtk_box_pack_start( GTK_BOX( priv->mainbox ), window, TRUE, TRUE, 0 );
+	gtk_box_pack_start( GTK_BOX( priv->mainbox ), priv->panel,
 				FALSE, FALSE, 0 );
-	gtk_container_add( GTK_CONTAINER( link_view ), private->mainbox );
+	gtk_container_add( GTK_CONTAINER( link_view ), priv->mainbox );
 
 	gnome_vfs_async_set_job_limit( 5 );
 }
@@ -1336,9 +1343,9 @@ static void screem_link_view_finalize( GObject *link_view )
 
 	view = SCREEM_LINK_VIEW( link_view );
 
-	g_free( view->private );
+	g_free( view->priv );
 
-	G_OBJECT_CLASS( parent_class )->finalize( G_OBJECT( link_view ) );
+	G_OBJECT_CLASS( screem_link_view_parent_class )->finalize( G_OBJECT( link_view ) );
 }
 
 static void screem_link_view_popup_zoom_callback( GtkAction *action,
@@ -1356,7 +1363,7 @@ static void screem_link_view_popup_zoom_callback( GtkAction *action,
 
 	view = SCREEM_LINK_VIEW( user_data );
 
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(view->private->zoom_control),
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(view->priv->zoom_control),
 				  (gdouble)zoom );
 }
 
@@ -1420,23 +1427,23 @@ static void screem_link_view_set_prop( GObject *object, guint property_id,
 				       const GValue *value, GParamSpec *pspec )
 {
 	ScreemLinkView *linkview;
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 
 	linkview = SCREEM_LINK_VIEW( object );
-	private = linkview->private;
+	priv = linkview->priv;
 
 	switch( property_id ) {
 	case PROP_MIN_X:
-		private->min_x = g_value_get_double( value );
+		priv->min_x = g_value_get_double( value );
 		break;
 	case PROP_MIN_Y:
-		private->min_y = g_value_get_double( value );
+		priv->min_y = g_value_get_double( value );
 		break;
 	case PROP_MAX_X:
-		private->max_x = g_value_get_double( value );
+		priv->max_x = g_value_get_double( value );
 		break;
 	case PROP_MAX_Y:
-		private->max_y = g_value_get_double( value );
+		priv->max_y = g_value_get_double( value );
 		break;
 	}
 }
@@ -1445,50 +1452,39 @@ static void screem_link_view_get_prop( GObject *object, guint property_id,
 				       GValue *value, GParamSpec *pspec)
 {
 	ScreemLinkView *linkview;
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 
 	linkview = SCREEM_LINK_VIEW( object );
-	private = linkview->private;
+	priv = linkview->priv;
 	
 	switch( property_id ) {
 	case PROP_UPLOAD_TABLE:
-		g_value_set_pointer( value, private->upload_table );
+		g_value_set_pointer( value, priv->upload_table );
 		break;
 	case PROP_MIN_X:
-		g_value_set_double( value, private->min_x );
+		g_value_set_double( value, priv->min_x );
 		break;
 	case PROP_MIN_Y:
-		g_value_set_double( value, private->min_y );
+		g_value_set_double( value, priv->min_y );
 		break;
 	case PROP_MAX_X:
-		g_value_set_double( value, private->max_x );
+		g_value_set_double( value, priv->max_x );
 		break;
 	case PROP_MAX_Y:
-		g_value_set_double( value, private->max_y );
+		g_value_set_double( value, priv->max_y );
 		break;
 	}
 }
 
-
-static void screem_link_view_size_request( GtkWidget *widget, GtkRequisition *req )
-{
-	GTK_WIDGET_CLASS( parent_class )->size_request( widget, req );
-}
-
-static void screem_link_view_realize( GtkWidget *widget )
-{
-	GTK_WIDGET_CLASS( parent_class )->realize( widget );
-}
-
 static void screem_link_view_hide( GtkWidget *widget )
 {
 	ScreemLinkView *view;
 
 	view = SCREEM_LINK_VIEW( widget );
 
-	gtk_widget_hide( view->private->panel );
+	gtk_widget_hide( view->priv->panel );
 
-	GTK_WIDGET_CLASS( parent_class )->hide( widget );
+	GTK_WIDGET_CLASS( screem_link_view_parent_class )->hide( widget );
 }
 
 static void screem_link_view_show( GtkWidget *widget )
@@ -1497,32 +1493,9 @@ static void screem_link_view_show( GtkWidget *widget )
 
 	view = SCREEM_LINK_VIEW( widget );
 
-	gtk_widget_show( view->private->panel );
+	gtk_widget_show( view->priv->panel );
 
-	GTK_WIDGET_CLASS( parent_class )->show( widget );
+	GTK_WIDGET_CLASS( screem_link_view_parent_class )->show( widget );
 }
 
-GType screem_link_view_get_type()
-{
-	static guint type = 0;
 
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemLinkViewClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_link_view_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemLinkView ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_link_view_init
-		};
-		
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemLinkView",
-					       &info, 0 );
-	}
-
-	return type;
-}
