@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -29,7 +29,7 @@
 
 #include <glib/gi18n.h>
 #include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-window-icon.h>
+#include <libgnomeui/gnome-authentication-manager.h>
 
 #include <gtk/gtk.h>
 
@@ -79,7 +79,6 @@ int main( int argc, char *argv[] )
 	const gchar** start_files;
 	const gchar *icon;
 	
-	gchar *path;
 	gchar *dotdir;
 	GConfClient *client;
 	
@@ -112,7 +111,7 @@ int main( int argc, char *argv[] )
 	start_files = parse_args( ctx, argc, argv );
 
         gnome_vfs_init();
-	screem_init_vfs_callbacks();
+	gnome_authentication_manager_init();
 
 	glade_require( "gnome" );
 
@@ -120,15 +119,13 @@ int main( int argc, char *argv[] )
 	 * and we have a glade file */
 	client = gconf_client_get_default();
 	dotdir = screem_get_dot_dir();
-	path = screem_get_glade_path();
 	
-	if( ! gconf_client_get_bool( client,
-				"/apps/screem/installed",
+	if( ! gconf_client_get_bool( client, "/apps/screem/installed",
 				NULL ) ) {
 		screem_main_missing( MISSING_SCHEMAS );
 	} else if( ! uri_accessible( dotdir, GNOME_VFS_PERM_USER_ALL ) ) {
 		screem_main_missing( MISSING_CONFIG_DIR );
-	} else if( ! uri_exists( path, NULL ) ) {
+	} else if( ! uri_exists( GLADE_PATH"/screem.glade", NULL ) ) {
 		screem_main_missing( MISSING_GLADE );
 	} else {
 		app = screem_application_new();
@@ -144,13 +141,14 @@ int main( int argc, char *argv[] )
 		g_object_unref( app );
 	}
 
-	g_free( path );
 	g_free( dotdir );
 	g_object_unref( client );
 
 	gnome_vfs_shutdown();
 	
 	g_object_unref( program );
+
+	xmlCleanupParser();
 	
 	return 0;
 }
@@ -193,6 +191,6 @@ static void screem_main_missing( MissingType type )
 			break;
 	}
 	
-	screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+	screem_hig_alert( GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 			primary, secondary, NULL );
 }
