@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -40,6 +40,7 @@
 #include "screem-site.h"
 #include "screem-page.h"
 #include "screem-search.h"
+#include "screem-markup.h"
 
 #include "fileops.h"
 
@@ -289,7 +290,7 @@ const gchar* screem_markup_char_to_ent( guint c )
 		tag = "ordf";
 		break ;
 	case GDK_guillemotleft:
-		tag = "laqo";
+		tag = "laquo";
 		break ;
 	case GDK_notsign:
 		tag = "not";
@@ -370,8 +371,8 @@ const gchar* screem_markup_char_to_ent( guint c )
 	return tag;
 }
 
-gboolean screem_markup_is_entity( const gchar *text, gint pos,
-				  gint *start, gint *end )
+gboolean screem_markup_is_entity( const gchar *text, guint pos,
+				  guint *start, guint *end )
 {
 	gboolean ret = FALSE;
 	gint i;
@@ -425,8 +426,137 @@ gboolean screem_markup_is_entity( const gchar *text, gint pos,
 	return ret;
 }
 
-gboolean screem_markup_is_attribute( const gchar *text, gint pos,
-				     gint *start, gint *end )
+#if 1
+gboolean screem_markup_is_attribute( const gchar *text, guint pos,
+				     guint *start, guint *end )
+{
+	gboolean ret = FALSE;
+
+	gchar *tag;
+	guint offset;
+	gunichar tc;
+	const gchar *tmp;
+	const gchar *tmp2;
+	gunichar c;
+
+	const gchar *pospointer;
+	const gchar *astart;
+	const gchar *aend;
+
+	guint tstart;
+	guint tend;
+	
+	if( pos < 0 ) {
+		pos = 0;
+	}
+
+	/* find tag */
+	if( screem_markup_is_tag( text, pos, &tstart, &tend ) ) {
+		tag = screem_markup_next_tag( text, tstart,
+				&tstart, &tend, NULL );
+		g_return_val_if_fail( tag != NULL, FALSE );
+	
+		offset = tstart;
+
+		pos -= offset;
+
+		pospointer = g_utf8_offset_to_pointer( tag, pos );
+		
+		/* bypass tag name */
+		tmp = screem_utf8_skip_to_space( tag );
+		/* bypass any extra space */
+		tmp = screem_utf8_skip_space( tmp );
+	
+		c = g_utf8_get_char( tmp );
+		
+		/* begin forward scan */
+		while( c != '\0' ) {
+			
+			/* an attribute starts here */
+			astart = tmp;
+			c = g_utf8_get_char( tmp );
+			while( c != '\0' && c != '=' && 
+					! g_unichar_isspace( c ) ) {
+				tmp = g_utf8_next_char( tmp );
+				c = g_utf8_get_char( tmp );
+			}
+			/* allow bonkers  name = "value" not just 
+			 * name="value" */
+			if( g_unichar_isspace( c ) ) {
+				tmp2 = tmp;
+				tmp = screem_utf8_skip_space( tmp );
+				c = g_utf8_get_char( tmp );
+				if( c != '=' ) {
+					tmp = tmp2;
+				}
+			}
+			if( c == '=' ) {
+				tmp = g_utf8_next_char( tmp );
+				tc = g_utf8_get_char( tmp );
+				if( g_unichar_isspace( tc ) ) {
+					tmp = screem_utf8_skip_space( tmp );
+					tc = g_utf8_get_char( tmp );
+				}
+			
+				if( tc != '"' && tc != '\'' ) {
+					/* attr ends at next space */
+					aend = screem_utf8_skip_to_space( tmp );
+					c = g_utf8_get_char( aend );
+					if( c == '\0' ) {
+						/* end of tag, step back to > */
+						aend = g_utf8_prev_char( aend );
+					}
+					/* step back to prev char */
+					aend = g_utf8_prev_char( aend );
+					tmp = aend;
+				} else {
+					/* attr ends at next tc */
+					tmp = g_utf8_next_char( tmp );
+					aend = g_utf8_strchr( tmp, -1,
+							tc );
+					if( aend ) {
+						tmp = aend;
+					} else {
+						aend = g_utf8_offset_to_pointer( tag, tend - offset );
+					}
+				}
+			} else { 
+				if( c == '\0' ) {
+					/* end of tag, step back to > */
+					tmp = g_utf8_prev_char( tmp );
+				}
+				aend = g_utf8_prev_char( tmp );
+			}
+
+			if( astart != aend && pospointer >= astart && pospointer <= aend ) {
+				/* we are inside this attribute */
+				ret = TRUE;
+				if( start ) {
+					*start = g_utf8_pointer_to_offset( tag, astart ) + offset;
+				}
+				if( end ) {
+					*end = g_utf8_pointer_to_offset( tag, aend ) + offset;
+				}
+
+			} 
+
+			tmp = g_utf8_next_char( tmp );
+			c = g_utf8_get_char( tmp );
+			if( tmp[ 0 ] == '>' && tmp[ 1 ] == '\0' ) {
+				c = '\0';
+			}
+		}
+		
+		g_free( tag );
+
+	} 
+
+	return ret;
+}
+#else
+
+gboolean screem_markup_is_attribute( const gchar *text, guint pos,
+				     guint *start, guint *end )
 {
 	gboolean ret = FALSE;
 	gunichar c;
@@ -544,9 +674,10 @@ gboolean screem_markup_is_attribute( const gchar *text, gint pos,
 
 	return ret;
 }
+#endif
 
-gboolean screem_markup_is_tag( const gchar *text, gint pos,
-			       gint *start, gint *end )
+gboolean screem_markup_is_tag( const gchar *text, guint pos,
+			       guint *start, guint *end )
 {
 	gint i;
 	gunichar c;
@@ -652,8 +783,8 @@ gboolean screem_markup_is_tag( const gchar *text, gint pos,
 	return ret;
 }
 
-gchar *screem_markup_next_tag( const gchar *text, gint pos, 
-			       gint *start, gint *end, gchar **name )
+gchar *screem_markup_next_tag( const gchar *text, guint pos, 
+			       guint *start, guint *end, gchar **name )
 {
 	const gchar *tag;
 	const gchar *tend;
@@ -661,8 +792,8 @@ gchar *screem_markup_next_tag( const gchar *text, gint pos,
 	gunichar c;
 	gchar *ret;
 
-	gint s;
-	gint e;
+	guint s;
+	guint e;
 	
 	if( pos < 0 ) {
 		g_warning( "ERROR\n" );
@@ -735,6 +866,8 @@ gchar *screem_markup_next_tag( const gchar *text, gint pos,
 				tend = strstr( tend, "]]>" );
 				if( tend ) {
 					tend += strlen( "]]>" );
+				} else {
+					tend = text + strlen( text ) - 1;
 				}
 				break;
 			}
@@ -780,7 +913,8 @@ gchar *screem_markup_encode_text( const gchar *text )
 	gunichar c;
 	gchar *ret;
 
-	string = g_string_new( NULL );
+	/* result will be at least strlen( text ) */
+	string = g_string_sized_new( strlen( text ) );
 
 	for( c = g_utf8_get_char( text ); c != '\0'; 
 		text = g_utf8_next_char( text ),
@@ -827,7 +961,9 @@ gchar *screem_markup_change_case( const gchar *text, gboolean upper )
 	} else {
 		alter = (Alter)g_unichar_tolower;
 	}
-	str = g_string_new( NULL );
+	
+	/* result will be at least strlen( text ) */
+	str = g_string_sized_new( strlen( text ) );
 	
 	while( ( next = screem_markup_next_tag( text, pos, &start, &end,
 						&name ) ) ) {
@@ -886,7 +1022,7 @@ gchar *screem_markup_change_case( const gchar *text, gboolean upper )
 }
 
 gboolean screem_markup_next_tag_close( const gchar *text,
-				       const gchar *tname, gint pos )
+		const gchar *tname, guint pos )
 {
 	gchar *next;
 	gchar *name;
@@ -908,7 +1044,8 @@ gboolean screem_markup_next_tag_close( const gchar *text,
 
 /* NOTE: this returns the list in reverse order, I.E
    end of tag <-- value, name, value, name  -->  start of tag */
-GSList *screem_markup_build_attributes_list( const gchar *tag, GSList *list )
+GSList *screem_markup_build_attributes_list( const gchar *tag, 
+		GSList *list )
 {
 	const gchar *tmp;
 	gunichar c;
@@ -920,7 +1057,7 @@ GSList *screem_markup_build_attributes_list( const gchar *tag, GSList *list )
 		gchar *value;
 
 		tmp = screem_utf8_skip_to_space( tmp );
-		tmp = g_utf8_skip_space( tmp );
+		tmp = screem_utf8_skip_space( tmp );
 
 		if( *tmp == '\0' || *tmp == '>' || *tmp == '/' ) {
 			break;
@@ -944,7 +1081,7 @@ GSList *screem_markup_build_attributes_list( const gchar *tag, GSList *list )
 			tmp = equals;
 		} else {
 			equals = g_utf8_next_char( equals );
-			equals = g_utf8_skip_space( equals );
+			equals = screem_utf8_skip_space( equals );
 			c = g_utf8_get_char( equals );
 			if( c == '"' || c == '\'' ) {
 				equals = g_utf8_next_char( equals );
@@ -966,7 +1103,13 @@ GSList *screem_markup_build_attributes_list( const gchar *tag, GSList *list )
 					tmp = strchr( equals, ' ' );
 				}
 			} else {
+				const gchar *tmp2;
 				tmp = screem_utf8_skip_to_space( equals );
+				/* could hit end of tag before we hit space */
+				tmp2 = strchr( equals, '>' );
+				if( tmp2 < tmp ) {
+					tmp = tmp2;
+				}
 			}
 
 			if( ! tmp ) {
@@ -992,23 +1135,31 @@ GSList *screem_markup_get_links( ScreemDTD *dtd, const gchar *text )
 {
 	GSList *ret;
 	gchar *next;
-	gint pos;
-	gint end;
+	guint pos;
+	guint end;
 	gchar *name;
+	GCompareFunc compare;
 
 	pos = 0;
 	end = 0;
 	ret = NULL;
 
+	g_return_val_if_fail( SCREEM_IS_DTD( dtd ), NULL );
+
+	g_object_get( G_OBJECT( dtd ), "compare", &compare, NULL );
+	
 	while( ( next = screem_markup_next_tag( text, pos, 
-						NULL, &end, &name ) ) ) {
+					NULL, &end, &name ) ) ) {
 		/* get tag attributes */
 		GSList *attr;
 		GSList *tmp;
 
 		pos = end;
 
-		attr = screem_markup_build_attributes_list( next, NULL );
+		attr = NULL;
+		if( *name != '?' && *name != '!' ) {
+			attr = screem_markup_build_attributes_list( next, NULL );
+		}
 
 		/* any links among them? */
 		for( tmp = attr; tmp; tmp = tmp->next ) {
@@ -1024,13 +1175,13 @@ GSList *screem_markup_get_links( ScreemDTD *dtd, const gchar *text )
 				} else {
 					g_free( temp->data );
 				}
-			} else if( ! g_strcasecmp("src",
+			} else if( ! compare("src",
 					(const gchar*)tmp->data ) ||
-				   ! g_strcasecmp("data",
+				   ! compare("data",
 					(const gchar*)tmp->data ) ||
-				   ! g_strcasecmp("href",
+				   ! compare("href",
 					(const gchar*)tmp->data) ||
-				   ! g_strcasecmp("url",
+				   ! compare("url",
 					(const gchar*)tmp->data ) ) {
 				/* fallback mode if no dtd is passed,
 				   then we check src="" href="" and data="" */
@@ -1108,6 +1259,17 @@ static void process_attrs( GString *str, const gchar *tag,
 	gchar *newpath;
 	gchar *temp;
 	
+	GCompareFunc compare;
+
+	compare = NULL;
+	if( dtd ) {
+		g_object_get( G_OBJECT( dtd ), 
+				"compare", &compare, NULL );
+	}
+	if( ! compare ) {
+		compare = (GCompareFunc)g_strcasecmp;
+	}
+	
 	attrs = screem_markup_build_attributes_list( tag, NULL );
 	attrs = g_slist_reverse( attrs );
 	
@@ -1155,8 +1317,14 @@ static void process_attrs( GString *str, const gchar *tag,
 
 			/* process value if aname is a
 			 * uri attribute */
-			if( screem_dtd_attr_is_uri( dtd, name,
-						aname ) ) {
+			if( *value != '#' &&
+			    ( ( dtd && screem_dtd_attr_is_uri( dtd, 
+							       name,
+							       aname ) ) || 
+			     ( ( ! dtd ) && ( ! compare("src", name ) ||
+			       ! compare("data", name ) ||
+			       ! compare("href", name ) ||
+			       ! compare("url", name ) ) ) ) ) {
 
 				uri = gnome_vfs_uri_new( value );
 				fragment = "";
@@ -1184,7 +1352,7 @@ static void process_attrs( GString *str, const gchar *tag,
 
 				if( match ) {
 					g_free( value );
-					value = newpath;
+					value = g_strdup( newpath );
 				}
 				if( match && fragment ) {
 					temp = value;
@@ -1192,6 +1360,7 @@ static void process_attrs( GString *str, const gchar *tag,
 					g_free( temp );
 				}
 				g_free( match );
+				g_free( newpath );
 				if( uri ) {
 					gnome_vfs_uri_unref( uri );
 				}
@@ -1204,6 +1373,7 @@ static void process_attrs( GString *str, const gchar *tag,
 		g_free( value );
 
 	}
+	g_slist_free( attrs );
 	/* add any extra spacing extra that there was, and also
 	 * the possible / */
 	g_string_append( str, offset );
@@ -1257,6 +1427,10 @@ static void screem_markup_fix_html_links( ScreemSite *site,
 	dtd = screem_page_get_dtd( page );
 	ineditable = FALSE;
 	
+	if( ! dtd ) {
+		g_warning( "Page without assigned dtd\n" );
+	}
+	
 	do {
 		ppos = pos;
 		name = NULL;
@@ -1662,6 +1836,7 @@ static const gchar *screem_markup_get_region( const gchar *text,
 	const gchar *start;
 	gchar *ed_name;
 	const gchar *reg_start;
+	guint len;
 	
 	if( ! blocks ) {
 		return NULL;
@@ -1670,8 +1845,6 @@ static const gchar *screem_markup_get_region( const gchar *text,
 	start = text;
 	
 	while( start ) {
-		gint len;
-
 		start = strstr( start, "<!-- #BeginEditable" );
 		reg_start = start;
 
@@ -1777,7 +1950,7 @@ gchar *screem_markup_basic_html( ScreemDTDDB *db, const gchar *doctype,
 }
 
 gchar *screem_markup_get_charset( const gchar *text, 
-		gint *retpos, gint *retlen )
+		GCompareFunc compare, guint *retpos, guint *retlen )
 {
 	guint len;
 	guint pos;
@@ -1794,6 +1967,10 @@ gchar *screem_markup_get_charset( const gchar *text,
 
 	ret = NULL;
 
+	if( ! compare ) {
+		compare = (GCompareFunc)g_strcasecmp;
+	}
+	
 	/* first we look for <?xml encoding="" ?>
 	   then we check <meta http-equiv="content-type" />
 	   then we check <meta name="charset" /> 
@@ -1832,7 +2009,7 @@ gchar *screem_markup_get_charset( const gchar *text,
 
 			g_slist_foreach( list, (GFunc)g_free, NULL );
 			g_slist_free( list );
-		} else if( ! g_strcasecmp( "meta", name ) ) {
+		} else if( ! compare( "meta", name ) ) {
 			/* meta element */
 			GSList *list;
 			GSList *attr;
@@ -1843,14 +2020,14 @@ gchar *screem_markup_get_charset( const gchar *text,
 			content = FALSE;
 
 			for( attr = list; attr; attr = attr->next ) {
-				if( ! strcmp( "http-equiv", 
+				if( ! compare( "http-equiv", 
 					      attr->next->data ) && 
 				    attr->data &&
 				    ! strcmp( "content-type",
 					      attr->data ) ) {
 					content = TRUE;
 					break;
-				} else if(! strcmp( "name", 
+				} else if(! compare( "name", 
 						    attr->next->data) &&
 					  attr->data && 
 					  ! strcmp( "charset",
@@ -1862,7 +2039,7 @@ gchar *screem_markup_get_charset( const gchar *text,
 			if( attr ) {
 				for( attr = list; attr; 
 				     attr = attr->next ) {
-					if( ! strcmp( "content",
+					if( ! compare( "content",
 						      attr->next->data ) ) {
 						break;
 					}
@@ -1894,15 +2071,15 @@ gchar *screem_markup_get_charset( const gchar *text,
 
 			g_slist_foreach( list, (GFunc)g_free, NULL );
 			g_slist_free( list );
-		} else if( g_strcasecmp( "html", name ) &&
-			   g_strcasecmp( "head", name ) &&
-			   g_strcasecmp( "title", name ) &&
-			   g_strcasecmp( "link", name ) &&
-			   g_strcasecmp( "script", name ) &&
-			   g_strcasecmp( "style", name ) &&
-			   g_strcasecmp( "!--", name ) &&
-			   g_strcasecmp( "base", name ) &&
-			   g_strcasecmp( "isindex", name ) &&
+		} else if( compare( "html", name ) &&
+			   compare( "head", name ) &&
+			   compare( "title", name ) &&
+			   compare( "link", name ) &&
+			   compare( "script", name ) &&
+			   compare( "style", name ) &&
+			   strcmp( "!--", name ) &&
+			   compare( "base", name ) &&
+			   compare( "isindex", name ) &&
 			   g_strcasecmp( "!DOCTYPE", name ) &&
 			   *name != '/' && *name != '?' ) {
 			/* limiting case, bit of a hack for html,
