@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -33,8 +33,7 @@
 #include <glade/glade.h>
 
 #include <glib/gi18n.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/libgnomeui.h>
+#include <libgnomeui/gnome-druid.h>
 
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -45,6 +44,7 @@
 
 #include "screem-application.h"
 #include "screem-window.h"
+#include "screem-page-druid.h"
 #include "screem-preview.h"
 #include "screem-site.h"
 #include "screem-page.h"
@@ -66,16 +66,14 @@ void screem_page_druid_begin( ScreemApplication *application,
 	ScreemPage *page;
 	GladeXML *xml;
 	GtkWidget *druid;
-	gchar *gladepath;
 	ScreemSession *session;
 	
 	session = screem_application_get_session( application );
 
 	site = screem_window_get_current( window );
 
-	gladepath = screem_get_glade_path();
-	xml = glade_xml_new( gladepath, "new_page_druid", NULL );
-	g_free( gladepath );
+	xml = glade_xml_new( GLADE_PATH"/screem.glade", 
+			"new_page_druid", NULL );
 
 	druid = glade_xml_get_widget( xml, "new_page_druid" );
 	page = screem_page_new( G_OBJECT( application ) );
@@ -143,7 +141,7 @@ void screem_page_druid_finish( GnomeDruidPage *druid_page,
 		if( error ) {
 			secondary = error->message;
 		}
-		screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+		screem_hig_alert( GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 				primary, secondary,
 				GTK_WIDGET( window ) );
 		if( error ) {
@@ -179,6 +177,28 @@ void screem_page_druid_cancel( GtkWidget *widget )
 	gtk_widget_destroy( widget );
 }
 
+static gboolean
+screem_page_druid_dtd_filter( GtkTreeModel *model, GtkTreeIter *iter,
+		GladeXML *xml )
+{
+	GtkWidget *all;
+	gboolean show;
+	
+	all = glade_xml_get_widget( xml, "show_all_doctypes" );
+	show = FALSE;
+	if( all ) {
+		show = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( all ) );
+	}
+
+	if( ! show ) {
+		gtk_tree_model_get( model, iter,
+				SCREEM_DTD_DB_ACTIVE_COL, &show, -1 );
+	}
+
+	return show;
+}
+
+
 void screem_page_druid_setup_prepare( GnomeDruidPage *druid_page,
 		GtkWidget *druid )
 {
@@ -193,6 +213,8 @@ void screem_page_druid_setup_prepare( GnomeDruidPage *druid_page,
 	GtkWidget *entry;
 	gchar *default_charset;
 
+	GtkTreeModel *filter;
+	
 	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
 
 	druid = glade_xml_get_widget( xml, "new_page_druid" );
@@ -205,12 +227,10 @@ void screem_page_druid_setup_prepare( GnomeDruidPage *druid_page,
 
 	site = SCREEM_SITE( g_object_get_data( G_OBJECT( druid ),
 				"site" ) );
-/*	sitepath = screem_site_get_pathname( site );*/
-	sitepath = screem_get_local_site_path( site );
+	sitepath = screem_site_get_pathname( site );
 	if( sitepath ) {
 		widget = glade_xml_get_widget( xml, "page_path" );
-		gnome_file_entry_set_default_path( GNOME_FILE_ENTRY( widget ),
-						   sitepath );
+		gtk_file_chooser_set_current_folder_uri( GTK_FILE_CHOOSER( widget ), sitepath );
 	}
 					   
 	widget = glade_xml_get_widget( xml, "doctype" );
@@ -220,13 +240,17 @@ void screem_page_druid_setup_prepare( GnomeDruidPage *druid_page,
 	db = screem_application_get_dtd_db( application );
 
 	list = screem_dtd_db_get_store( db );
-	gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), 
-			GTK_TREE_MODEL( list ) );
-	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ),
-				0 );
+	filter = gtk_tree_model_filter_new( GTK_TREE_MODEL( list ),
+			NULL );
+	gtk_tree_model_filter_set_visible_func( GTK_TREE_MODEL_FILTER( filter ), (GtkTreeModelFilterVisibleFunc)screem_page_druid_dtd_filter,
+			xml, NULL );
+	gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), filter );
+	g_object_unref( G_OBJECT( filter ) );
 
-	default_dtd = gconf_client_get_string( application->client,
-			"/apps/screem/editor/default_dtd", NULL );
+	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ),
+			SCREEM_DTD_DB_PUBLIC_COL );
+	
+	default_dtd = screem_dtd_db_lookup_mime_type( db, "*", NULL );
 	if( ! default_dtd ) {
 		default_dtd = g_strdup( "-//W3C//DTD HTML 4.01 Transitional//EN" );
 	}
@@ -259,27 +283,26 @@ void screem_page_druid_setup_prepare( GnomeDruidPage *druid_page,
 static void charset_changed( GtkWidget *entry, GtkWidget *combo )
 {
 	GtkTreeModel *model;
-	GtkTreeIter *it;
+	GtkTreeIter it;
 	const gchar *value;
+	gboolean found;
 	gchar *colval;
 	gchar *disval;
 	
 	value = gtk_entry_get_text( GTK_ENTRY( entry ) );
 	model = gtk_combo_box_get_model( GTK_COMBO_BOX( combo ) );
-	it = screem_support_find_in_list( GTK_LIST_STORE( model ), 1, 
-			value );
-	if( ! it ) {
-		it = screem_support_find_in_list( GTK_LIST_STORE( model ), 0, value );
+	found = screem_gtk_list_store_find_string( GTK_LIST_STORE( model ), &it, 1, value );
+	if( ! found ) {
+		found = screem_gtk_list_store_find_string( GTK_LIST_STORE( model ), &it, 0, value );
 	}
-	if( it ) {
-		gtk_tree_model_get( model, it,	
+	if( found ) {
+		gtk_tree_model_get( model, &it,	
 				0, &disval,
 				1, &colval, -1 );
 		if( ! strcmp( value, colval ) || 
 		    ! strcmp( value, disval ) ) {
-			gtk_combo_box_set_active_iter( GTK_COMBO_BOX( combo ), it );
+			gtk_combo_box_set_active_iter( GTK_COMBO_BOX( combo ), &it );
 		}
-		gtk_tree_iter_free( it );
 		g_free( disval );
 		g_free( colval );
 	}
@@ -302,8 +325,18 @@ void screem_page_druid_details_prepare( GnomeDruidPage *druid_page,
 			GINT_TO_POINTER( 1 ) );
 
 	widget = glade_xml_get_widget( xml, "page_author" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY(widget) );
+	screem_gtk_add_history( widget );
+	widget = GTK_BIN( widget )->child;
 	gtk_entry_set_text( GTK_ENTRY( widget ), g_get_real_name() );
+
+	widget = glade_xml_get_widget( xml, "page_title" );
+	screem_gtk_add_history( widget );
+
+	widget = glade_xml_get_widget( xml, "page_description" );
+	screem_gtk_add_history( widget );
+
+	widget = glade_xml_get_widget( xml, "page_keywords" );
+	screem_gtk_add_history( widget );
 }
 
 void screem_page_druid_template_prepare( GnomeDruidPage *druid_page,
@@ -357,17 +390,16 @@ void screem_page_druid_template_prepare( GnomeDruidPage *druid_page,
 			"text", 1, NULL );
 
 	widget = glade_xml_get_widget( xml, "custom_template" );
-	/*sitepath = screem_site_get_pathname( site );*/
-	sitepath = screem_get_local_site_path( site );
+	sitepath = screem_site_get_pathname( site );
 	if( sitepath ) {
-		gnome_file_entry_set_default_path( GNOME_FILE_ENTRY( widget ), sitepath );
+		gtk_file_chooser_set_current_folder_uri( GTK_FILE_CHOOSER( widget ), sitepath );
 	}
 		
+	gtk_widget_set_sensitive( widget, FALSE );
 	template = screem_site_get_template_path( site );
 	if( template ) {
-		widget = glade_xml_get_widget( xml, "custom_template" );
-
-		gnome_file_entry_set_filename( GNOME_FILE_ENTRY( widget ), template );
+		gtk_file_chooser_set_uri( GTK_FILE_CHOOSER( widget ),
+				template );
 		widget = glade_xml_get_widget( xml, "use_custom" );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), TRUE );
 	}
@@ -403,12 +435,10 @@ void screem_page_druid_appearance_prepare( GnomeDruidPage *druid_page,
 
 	site = SCREEM_SITE( g_object_get_data( G_OBJECT( druid ),
 				"site" ) );
-	/*sitepath = screem_site_get_pathname( site );*/
-	sitepath = screem_get_local_site_path( site );
+	sitepath = screem_site_get_pathname( site );
 	if( sitepath ) {
 		widget = glade_xml_get_widget( xml, "page_stylesheet" );
-		gnome_file_entry_set_default_path( GNOME_FILE_ENTRY( widget ),
-						   sitepath );
+		gtk_file_chooser_set_current_folder_uri( GTK_FILE_CHOOSER( widget ), sitepath );
 	}
 
 }
@@ -459,12 +489,13 @@ void screem_page_druid_preview_prepare( GnomeDruidPage *druid_page,
 	screem_view_display( SCREEM_VIEW( preview ) );
 }
 
-static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
-					   ScreemWindow *window )
+static void screem_page_druid_create_page( ScreemPage *page, 
+		GladeXML *xml, ScreemWindow *window )
 {
 	GtkWidget *widget;
 
 	gchar *pathname;
+	const gchar *ext;
 	gchar *uri;
 	
 	gchar *stylesheet;
@@ -483,13 +514,14 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 
 	GString *newtext;
 	gchar *text;
-	gint pos;
-	gint start;
-	gint end;
+	guint pos;
+	guint start;
+	guint end;
 	gchar *name;
 	gchar *tag;
 
 	gboolean xhtml;
+	GCompareFunc compare;
 
 	gboolean setauthor;
 	gboolean setgenerator;
@@ -501,6 +533,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 
 	ScreemApplication *app;
 	ScreemDTDDB *db;
+	ScreemDTD *dtd;
 	ScreemSite *site;
 	const gchar *sitepath;
 	gchar *spath;
@@ -516,11 +549,18 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	site = screem_window_get_current( window );
 	
 	widget = glade_xml_get_widget( xml, "page_path" );
-	pathname = gnome_file_entry_get_full_path( GNOME_FILE_ENTRY( widget ), FALSE );
-
-
+	tmp = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( widget ) );
+	widget = glade_xml_get_widget( xml, "page_name" );
+	/* safe cast as we null or use it to make a copy */
+	pathname = (gchar*)gtk_entry_get_text( GTK_ENTRY( widget ) );
+	if( pathname && *pathname != '\0' ) {
+		pathname = g_build_filename( tmp, pathname, NULL );
+	} else {
+		pathname = NULL;
+	}
+	
 	widget = glade_xml_get_widget( xml, "page_stylesheet" );
-	stylesheet = gnome_file_entry_get_full_path( GNOME_FILE_ENTRY( widget ), FALSE );
+	stylesheet = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( widget ) ); 
 	if( stylesheet ) {
 		if( ! screem_site_get_fake_flag( site ) ) {
 			spath = relative_path( stylesheet,
@@ -546,21 +586,31 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	widget = GTK_BIN( widget )->child;
 	doctype = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
+	dtd = screem_dtd_db_get_dtd( db, doctype, NULL );
+	if( dtd ) {
+		g_object_get( G_OBJECT( dtd ), "compare", &compare, 
+				NULL );	
+	} else {
+		/* default to case sensitive if we failed to
+		 * load the dtd */
+		compare = (GCompareFunc)strcmp;
+	}
 	/* bit of a hack */
 	xhtml = ( strstr( doctype, " XHTML " ) != NULL );
 	if( pathname && ! xhtml ) {
-		xhtml = ! strcmp( ".xml", 
-				g_extension_pointer( pathname ) );
-		xhtml |= ! strcmp( ".xhtml",
-				g_extension_pointer( pathname ) );
+		ext = strrchr( pathname, '.' );
+		if( ext ) {
+			xhtml = ! strcmp( ".xml", ext );
+			xhtml |= ! strcmp( ".xhtml", ext );
+		}
 	}
 
 	widget = glade_xml_get_widget( xml, "page_title" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	title = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	widget = glade_xml_get_widget( xml, "page_author" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	author = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	widget = glade_xml_get_widget( xml, "page_charset" );
@@ -580,11 +630,11 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	}
 
 	widget = glade_xml_get_widget( xml, "page_description" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	description = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	widget = glade_xml_get_widget( xml, "page_keywords" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	keywords = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	template = NULL;
@@ -592,8 +642,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 		widget = glade_xml_get_widget( xml, "custom_template" );
 
-		template = gnome_file_entry_get_full_path( GNOME_FILE_ENTRY( widget ), TRUE );
-	 	
+		template = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( widget ) ); 
 	}
 	widget = glade_xml_get_widget( xml, "use_standard" );
 	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
@@ -662,7 +711,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 		
 		processed = FALSE;
 
-		if( ! g_strcasecmp( "/head", name ) ) {
+		if( ! compare( "/head", name ) ) {
 			/* so we can append unset meta elements, and
 			   a link to a stylesheet if specified */
 			if( ! setauthor ) {
@@ -751,7 +800,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 					g_string_append( newtext, "\" />\n\t" );
 				}
 			}
-		} else if( ! g_strcasecmp( "title", name ) && *title != '\0' ) {
+		} else if( ! compare( "title", name ) && *title != '\0' ) {
 			processed = TRUE;
 			g_string_append_len( newtext,
 					text + start, end - start );
@@ -760,7 +809,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			while( text[ pos ] != '<' && text[ pos ] != '\0' ) {
 				pos ++;
 			}
-		} else if( ! g_strcasecmp( "meta", name ) ) {
+		} else if( ! compare( "meta", name ) ) {
 			/* we are interested in:
 			   name="author"
 			   name="generator"
@@ -777,13 +826,13 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 
 				attr = list;
 				list = list->next;
-				if( ! g_strcasecmp( "name", attr->data ) &&
+				if( ! compare( "name", attr->data ) &&
 				    ( ! g_strcasecmp( "author", list->data ) ||
 				      ! g_strcasecmp( "generator", list->data ) ||
 				      ! g_strcasecmp( "description", list->data ) ||
 				      ! g_strcasecmp( "keywords", list->data ) ) ) {
 					nme = list;
-				} else if( ! g_strcasecmp( "http-equiv", attr->data ) &&
+				} else if( ! compare( "http-equiv", attr->data ) &&
 					   ( ! g_strcasecmp( "content-type",
 							     list->data ) ||
 					     ! g_strcasecmp( "Content-Style-Type",
@@ -792,7 +841,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 							     list->data ) ) ) {
 					nme = list;
 				}
-				if( ! g_strcasecmp( "content", attr->data ) ) {
+				if( ! compare( "content", attr->data ) ) {
 					content = list;
 				}
 			}
@@ -839,7 +888,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 					} else if( ! g_strcasecmp( "generator",
 								   list->data ) ) {
 						setgenerator = TRUE;
-					} else if( ! g_strcasecmp( "desription",
+					} else if( ! g_strcasecmp( "description",
 								list->data ) ) {
 						setdescription = TRUE;
 					} else if( ! g_strcasecmp( "keywords", 
@@ -871,7 +920,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			} else {
 				processed = FALSE;
 			}
-		} else if( ! g_strcasecmp( "body", name ) && override ) {
+		} else if( ! compare( "body", name ) && override ) {
 			GSList *bgcolor;
 			GSList *text;
 			GSList *link;
@@ -889,17 +938,17 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			/* find attributes in attrs */
 			bgcolor = text = link = vlink = alink = background = NULL;
 			for( list = attrs; list; list = list->next ) {
-				if( ! g_strcasecmp( "bgcolor", list->data ) ) {
+				if( ! compare( "bgcolor", list->data ) ) {
 					bgcolor = list->next;
-				} else if( ! g_strcasecmp( "text", list->data ) ) {
+				} else if( ! compare( "text", list->data ) ) {
 					text = list->next;
-				} else if( ! g_strcasecmp( "link", list->data ) ) {
+				} else if( ! compare( "link", list->data ) ) {
 					link = list->next;
-				} else if( ! g_strcasecmp( "vlink", list->data ) ) {
+				} else if( ! compare( "vlink", list->data ) ) {
 					vlink = list->next;
-				} else if( ! g_strcasecmp( "alink", list->data ) ) {
+				} else if( ! compare( "alink", list->data ) ) {
 					alink = list->next;
-				} else if( ! g_strcasecmp( "background", list->data ) ) {
+				} else if( ! compare( "background", list->data ) ) {
 					background = list->next;
 				}
 				list = list->next;
@@ -909,7 +958,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "bgcolorpicker" );
 				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
-				value = screem_gdk_color_to_string( &color );
+				value = screem_gdk_color_to_string( &color, TRUE );
 				if( bgcolor ) {
 					g_free( bgcolor->data );
 					bgcolor->data = value;
@@ -923,7 +972,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "textpicker" );
 				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
-				value = screem_gdk_color_to_string( &color );
+				value = screem_gdk_color_to_string( &color, TRUE );
 				if( text ) {
 					g_free( text->data );
 					text->data = value;
@@ -937,7 +986,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "linkpicker" );
 				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
-				value = screem_gdk_color_to_string( &color );
+				value = screem_gdk_color_to_string( &color, TRUE );
 				if( link ) {
 					g_free( link->data );
 					link->data = value;
@@ -951,7 +1000,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "vlinkpicker" );
 				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
-				value = screem_gdk_color_to_string( &color );
+				value = screem_gdk_color_to_string( &color, TRUE );
 				if( vlink ) {
 					g_free( vlink->data );
 					vlink->data = value;
@@ -965,7 +1014,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "alinkpicker" );
 				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
-				value = screem_gdk_color_to_string( &color );
+				value = screem_gdk_color_to_string( &color, TRUE );
 				if( alink ) {
 					g_free( alink->data );
 					alink->data = value;
