@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -33,13 +33,13 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-
-#include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
 #include <gconf/gconf-client.h>
 
 #include <glib/gconvert.h>
 #include <glib/gunicode.h>
+#include <glib/gi18n.h>
 
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtkliststore.h>
@@ -86,15 +86,6 @@ enum {
 
 static guint screem_page_signals[ LAST_SIGNAL ] = { 0 };
 
-
-static void screem_page_class_init( ScreemPageClass *klass );
-static void screem_page_init( ScreemPage *page );
-static void screem_page_set_prop( GObject *object, guint prop_id,
-				  const GValue *value, GParamSpec *spec );
-static void screem_page_get_prop( GObject *object, guint prop_id,
-				  GValue *value, GParamSpec *spec );
-static void screem_page_finalize( GObject *object );
-
 static gboolean bookmark_store_find( GtkTreeModel *model, GtkTreePath *path,
 				     GtkTreeIter *iter, gpointer data );
 
@@ -106,6 +97,14 @@ static void screem_page_monitor_cb( GnomeVFSMonitorHandle *handle,
 				    GnomeVFSMonitorEventType type,
 				    gpointer data );
 
+static gboolean screem_page_auto_save( ScreemPage *page );
+static void screem_page_auto_save_notify( GConfClient *client,
+					  guint cnxn_id,  GConfEntry *entry,
+					  gpointer data );
+static void screem_page_auto_save_interval_notify( GConfClient *client,
+					  guint cnxn_id,  GConfEntry *entry,
+					  gpointer data );
+
 struct ScreemPagePrivate {
 	ScreemPageModel *model;
 	
@@ -125,8 +124,16 @@ struct ScreemPagePrivate {
 
 	GnomeVFSMonitorHandle *monitor;
 	gboolean modified;
+	GTimeVal time;
 
 	gboolean compress;
+
+	gboolean auto_save;
+	guint auto_save_idle;
+	guint auto_save_interval;
+	guint auto_save_notify;
+	guint auto_save_interval_notify;
+
 };
 
 void screem_page_undo( ScreemPage *page )
@@ -249,6 +256,98 @@ const gchar *screem_page_get_pathname( ScreemPage *page )
 	return page->private->pathname;
 }
 
+gchar *screem_page_get_short_name( ScreemPage *page )
+{
+	const gchar *pathname;
+	gchar *basename;
+	gchar *temp;
+	
+	g_return_val_if_fail( SCREEM_IS_PAGE( page ), NULL );
+	
+	pathname = screem_page_get_pathname( page );
+
+	if( ! pathname ) {
+		pathname = _( "Untitled" );
+	}
+	basename = g_path_get_basename( pathname );
+	temp = gnome_vfs_unescape_string_for_display( basename );
+	g_free( basename );
+	
+	return temp;
+}
+
+gchar *screem_page_get_title_name( ScreemPage *page )
+{
+	const gchar *mime;
+	const gchar *showtype;
+	const gchar *pathname;
+	gchar *tmppath;
+	gchar *base;
+	gchar *dir;
+	gchar *temp;
+	gchar *title;
+	gchar *path;
+	
+	g_return_val_if_fail( SCREEM_IS_PAGE( page ), "Screem" );
+	
+	mime = screem_page_get_mime_type( page );
+	
+	showtype = NULL;
+	if( mime ) {
+		showtype = gnome_vfs_mime_get_description( mime );
+	} else {
+		showtype = "";
+	}
+	
+	pathname = screem_page_get_pathname( page );
+	
+	tmppath = NULL;
+	if( ! pathname ) {
+		pathname = _( "Untitled" );
+		base = g_strconcat( pathname, " ",
+				showtype, NULL );
+		dir = NULL;
+	} else {
+		if( g_str_has_prefix( pathname, "file://" ) ) {
+			pathname += strlen( "file://" );
+		}
+		path = NULL;
+		if( *pathname == '/' ) {
+			dir = g_build_path( G_DIR_SEPARATOR_S,
+				g_get_home_dir(), "", NULL );
+			if( g_str_has_prefix( pathname, dir ) ) {
+				path = g_build_path( G_DIR_SEPARATOR_S,
+					"~", pathname + strlen( dir ), NULL );
+				pathname = path; 
+			}
+			g_free( dir );
+		}
+		dir = g_path_get_dirname( pathname );
+		base = g_path_get_basename( pathname );
+		temp = gnome_vfs_unescape_string_for_display( base );
+		g_free( base );
+		base = temp;
+			
+		temp = gnome_vfs_unescape_string_for_display( dir );
+		g_free( dir );
+		dir = temp;
+
+		g_free( path );
+	}
+
+	if( ! dir ) {
+		title = g_strdup( base );
+	} else {
+		title = g_strdup_printf( "%s (%s)", base, dir );
+	}
+
+	g_free( tmppath );
+	g_free( base );
+	g_free( dir ); 
+
+	return title;
+}
+
 void screem_page_set_mime_type( ScreemPage *page, const gchar *mime_type )
 {
 	g_return_if_fail( SCREEM_IS_PAGE( page ) );
@@ -276,7 +375,8 @@ static void screem_page_set_mime_type_real( ScreemPage *page, const gchar *mime_
 	}
 	page->private->mime_type = g_strdup( mime_type );
 
-	screem_page_model_force_check( page->private->model );
+	screem_page_model_force_check( page->private->model,
+			FALSE );
 
 	g_free( type );
 }
@@ -314,7 +414,7 @@ void screem_page_set_data( ScreemPage *page, const gchar *data )
 	gtk_text_buffer_set_text( GTK_TEXT_BUFFER( page ), data, 
 				strlen( data ) );
 	
-	screem_page_model_force_check( page->private->model );
+/*	screem_page_model_force_check( page->private->model );*/
 		
 	page->private->loaded = TRUE;
 	gtk_source_buffer_end_not_undoable_action( GTK_SOURCE_BUFFER( page ) );
@@ -345,9 +445,28 @@ void screem_page_set_changed( ScreemPage *page, gboolean flag )
 
 static void screem_page_set_changed_real( ScreemPage *page, gboolean flag )
 {
+	ScreemPagePrivate *priv;
+	guint interval;
+	
 	g_return_if_fail( SCREEM_IS_PAGE( page ) );
 
-	page->private->changed = flag;
+	priv = page->private;
+
+	priv->changed = flag;
+
+	if( ! flag ) {
+		/* remove timer as we have nothing to save */
+		if( priv->auto_save_idle ) {
+			g_source_remove( priv->auto_save_idle );
+			priv->auto_save_idle = 0;
+		}
+		g_get_current_time( &priv->time );
+	} else if( priv->auto_save && ! priv->auto_save_idle ) {
+		/* add timer if needed */
+		interval = priv->auto_save_interval * 60 * 1000;
+		priv->auto_save_idle = g_timeout_add( interval,
+			(GSourceFunc)screem_page_auto_save, page );
+	}
 }
 
 gboolean screem_page_get_changed( ScreemPage *page )
@@ -364,9 +483,9 @@ gboolean screem_page_load( ScreemPage *page, GError **error )
 	gboolean compress;
 	gchar *charset;
 
-	GString *data;
+	gchar *data;
 
-    g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
+	g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
 
 	if( error ) {
 		*error = NULL;
@@ -383,10 +502,9 @@ gboolean screem_page_load( ScreemPage *page, GError **error )
 	
 	priv->compress = FALSE;
 	data = load_file( path, &compress, &charset, error );
-
+	
 	if( data ) {
-		screem_page_set_data( page, data->str );
-		g_string_free( data, TRUE );
+		screem_page_set_data( page, data );
 
 		screem_page_set_changed( page, FALSE );
 
@@ -398,6 +516,9 @@ gboolean screem_page_load( ScreemPage *page, GError **error )
 			g_object_set( G_OBJECT( priv->model ), "charset", charset, NULL );
 		}	
 		g_free( charset );
+		g_free( data );
+	} else {
+		g_print( "FAILED TO LOAD\n" );
 	}
 
 	page->private->modified = FALSE;
@@ -438,6 +559,8 @@ gboolean screem_page_save( ScreemPage *page, GError **error )
 	
 	g_signal_emit( G_OBJECT( page ),
 		       screem_page_signals[ SAVING ], 0 );
+
+	screem_page_model_force_check( priv->model, TRUE );
 	
 	data = screem_page_get_data( page );
 
@@ -479,7 +602,9 @@ gboolean screem_page_save( ScreemPage *page, GError **error )
 			    GNOME_VFS_PERM_GROUP_READ |
 			    GNOME_VFS_PERM_GROUP_WRITE |
 			    GNOME_VFS_PERM_OTHER_READ,
-			    priv->compress, error );
+			    priv->compress,
+			    TRUE,
+			    error );
 
 	g_free( data );
 
@@ -503,6 +628,34 @@ gboolean screem_page_save( ScreemPage *page, GError **error )
 	return retval;
 }
 
+void screem_page_unload( ScreemPage *page )
+{
+	GtkTextIter it;
+	GtkTextIter eit;
+	g_return_if_fail( SCREEM_IS_PAGE( page ) );
+	g_return_if_fail( screem_page_is_open( page ) == FALSE );
+		
+	if( screem_page_is_loaded( page ) ) {
+		page->private->loaded = FALSE;
+		gtk_text_buffer_get_bounds( GTK_TEXT_BUFFER( page ), &it, &eit );
+		gtk_text_buffer_delete( GTK_TEXT_BUFFER( page ), &it, &eit );
+	}
+}
+
+glong screem_page_get_seconds_since_last_save_or_load( ScreemPage *page )
+{
+	GTimeVal current_time;
+	ScreemPagePrivate *priv;
+	
+	g_return_val_if_fail( SCREEM_IS_PAGE( page ), -1); 
+	
+	priv = page->private;
+	g_get_current_time(&current_time); 
+	
+	return( current_time.tv_sec - priv->time.tv_sec );
+}
+
+
 gboolean screem_page_is_file_page( const gchar *path )
 {
 	gchar *mime_type;
@@ -539,6 +692,9 @@ gboolean screem_page_is_mime_type_page( const gchar *mime_type )
 				is_page |= ! strcmp( end, "+xml" );
 			}
 		}
+		if( ! is_page ) {
+			is_page = ( gnome_vfs_mime_type_get_equivalence( mime_type, "text/plain" ) == GNOME_VFS_MIME_PARENT );
+		}
 	}
 	
 	return is_page;
@@ -792,7 +948,7 @@ gboolean screem_page_is_markup( ScreemPage *page )
 gchar *screem_page_template_path( ScreemPage *page )
 {
 	gchar *text;
-	gint len;
+	guint len;
 	gchar *name;
 	gchar *ret;
 	gchar *end;
@@ -818,13 +974,17 @@ gchar *screem_page_template_path( ScreemPage *page )
 	return ret;
 }
 
-gboolean screem_page_is_template( ScreemPage *page, gchar **tag,
-				  gchar **path, GSList **blocks )
+gboolean screem_page_is_template( ScreemPage *page, 
+		const gchar *base,
+		gchar **tag, gchar **path, GSList **blocks )
 {
+	const gchar *pathname;
 	gchar *text;
-	gint len;
+	guint len;
 	gchar *name;
 	gboolean ret;
+	gchar *tmp;
+	gchar *dwcompat;
 
 	g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
 	g_return_val_if_fail( tag != NULL, FALSE );
@@ -833,19 +993,51 @@ gboolean screem_page_is_template( ScreemPage *page, gchar **tag,
 	if( ! screem_page_is_markup( page ) ) {
 		return FALSE;
 	}
-	
+		
 	text = screem_page_get_data( page );
 	name = find_text( text, "<!-- #BeginTemplate[ \t]+\"[^\"]*\"[ \t]*-->",
 			  NULL, &len );
 	ret = ( name != NULL );
+	
+	dwcompat = NULL;
+	if( ! ret ) {
+		/* unable to find template header, Dreamweaver
+		 * doesn't put this on the templates themselves,
+		 * check page pathname to see if it is /Templates */
+		GnomeVFSURI *uri;
+
+		pathname = screem_page_get_pathname( page );
+		pathname += strlen( base );
+		uri = gnome_vfs_uri_new( pathname );
+
+		if( uri ) {
+			pathname = gnome_vfs_uri_get_path( uri );
+		
+			if( ! strncmp( "/Templates/", pathname,
+					strlen( "/Templates/" ) ) ) {
+				dwcompat = g_strconcat( "<!-- #BeginTemplate \"",
+					pathname, "\" -->", NULL );
+				len = strlen( dwcompat );
+				name = dwcompat;
+				ret = TRUE;
+			}
+		}
+	}
+	
 	if( ret ) {
-		const gchar *pathname;
 		gchar *end;
 		*tag= g_strndup( name, len );
 		name += strlen( "<!-- #BeginTemplate \"" );
 		end = strchr( name, '"' );
-		*path = g_strndup( name, end - name );
-				
+	
+		tmp = g_strndup( name, end - name );	
+		if( ! strncmp( "/Templates/", name, strlen( "/Templates/" ) ) ) {
+			*path = g_strconcat( base, tmp, NULL );
+			g_free( tmp );
+		} else {
+			*path = tmp;
+		}
+		
 		pathname = screem_page_get_pathname( page );
 		if( ! gnome_vfs_uris_match( pathname, *path ) ) {
 			/* page is not the master template, it just 
@@ -856,12 +1048,36 @@ gboolean screem_page_is_template( ScreemPage *page, gchar **tag,
 		} else if( blocks ){
 			/* build list of blocks */
 			gchar *template_end;
-			name = strstr( name, "-->" );
-			g_assert( name );
-			name += strlen( "-->" );
 		
-			template_end = strstr( name, "<!-- #EndTemplate -->" );
-			
+			if( ! dwcompat ) {
+				name = strstr( name, "-->" );
+				g_assert( name );
+				name += strlen( "-->" );
+				template_end = strstr( name, "<!-- #EndTemplate -->" );
+			} else {
+				/* Dreamweaver compatability mode */
+				name = strstr( text, "<HTML" );
+				if( ! name ) {
+					name = strstr( text, "<html" );
+				}
+				if( name ) {
+					name = strchr( name, '>' );
+					if( name ) {
+						name += strlen( ">" );
+					}
+				}
+				/* no <html> to bypass */
+				if( ! name ) {
+					name = text;
+				}
+				template_end = strstr( name, 
+						"</HTML>" );
+				if( ! template_end ) {
+					template_end = strstr( name, 
+							"</html>" );
+				}
+			}
+		
 			do {
 				gchar *ed_name = NULL;
 
@@ -896,7 +1112,7 @@ gboolean screem_page_is_template( ScreemPage *page, gchar **tag,
 			*blocks = g_slist_reverse( *blocks );
 		}
 	}
-	
+	g_free( dwcompat );
 	g_free( text );
 	
 	return ret;
@@ -962,8 +1178,12 @@ ScreemDTD *screem_page_get_dtd( ScreemPage *page )
 	ret = NULL;
 	g_object_get( G_OBJECT( page->private->model ),
 			"dtd", &ret, NULL );
-
-	g_object_unref( ret );
+	/* model will be holding a references, the model
+	 * is kept around as long as the page, so
+	 * release the reference added with g_object_get */
+	if( ret ) {
+		g_object_unref( ret );
+	}
 	
 	return ret;
 }
@@ -980,6 +1200,7 @@ void screem_page_model_emit_built( ScreemPage *page )
 		       screem_page_signals[ BUILT ], 0 );
 }
 
+/* FIXME: be nice if we did some caching here */
 gboolean screem_page_is_xml( ScreemPage *page )
 {
 	ScreemPagePrivate *priv;
@@ -987,6 +1208,7 @@ gboolean screem_page_is_xml( ScreemPage *page )
 	ScreemDTD *dtd;
 	const gchar *mime;
 	const gchar *pathname;
+	const gchar *ext;
 	gchar *id;
 	gchar *data;
 	const gchar *tmp;
@@ -1020,10 +1242,11 @@ gboolean screem_page_is_xml( ScreemPage *page )
 			g_free( id );
 		}
 		if( pathname && ! ret ) {
-			ret = ! strcmp( ".xml", 
-					g_extension_pointer( pathname ) );
-			ret |= ! strcmp( ".xhtml",
-					g_extension_pointer( pathname ) );
+			ext = strrchr( pathname, '.' );
+			if( ext ) {
+				ret = ! strcmp( ".xml", ext );
+				ret |= ! strcmp( ".xhtml", ext );
+			}
 		}
 		if( ! ret ) {
 			data = screem_page_get_data( page );
@@ -1055,8 +1278,39 @@ void screem_page_set_lang( ScreemPage *page )
 	if( ! lang && screem_page_is_xml( page ) ) {
 		lang = gtk_source_languages_manager_get_language_from_mime_type( lm, "text/xml" );
 	}
+
+	/* iterate over languages again, this time checking for
+	 * parent types, still need the above special case for
+	 * text/xml though as we detect XML types in other methods
+	 * than just the mime type equivalence */
+	if( ! lang ) {
+		GSList *langs;
+		GSList *types;
+		GSList *tmp;
+		GnomeVFSMimeEquivalence equal;
+		
+		langs = gtk_source_languages_manager_get_available_languages( lm );
+		while( langs && ! lang ) {
+			types = gtk_source_language_get_mime_types( GTK_SOURCE_LANGUAGE( langs->data ) );
+			for( tmp = types; tmp; tmp = tmp->next ) {
+
+				equal = gnome_vfs_mime_type_get_equivalence( priv->mime_type, tmp->data );
+				
+				if( equal == GNOME_VFS_MIME_PARENT ) {
+					break;
+				}
+			}
+			g_slist_foreach( types, (GFunc)g_free, NULL );
+			g_slist_free( types );
+			if( tmp ) {
+				lang = langs->data;
+			}
+			langs = langs->next;
+		}
+	}
 	
 	if( lang ) {
+		screem_application_init_tag_styles( lang );
 		gtk_source_buffer_set_language( GTK_SOURCE_BUFFER( page ), lang );
 		table = gtk_text_buffer_get_tag_table( GTK_TEXT_BUFFER( page ) );
 		if( ! gtk_text_tag_table_lookup( table,
@@ -1065,87 +1319,159 @@ void screem_page_set_lang( ScreemPage *page )
 			tag = gtk_text_tag_new( SCREEM_INVALID_MARKUP_TAG );
 			gtk_text_tag_table_add( table, tag );
 			g_object_set( G_OBJECT( tag ),
-				"background", "#ffffffff0000",
-				"foreground", "#000000000000",
+/*				"background", "#ffffffff0000",
+				"foreground", "#000000000000",*/
+				"underline", PANGO_UNDERLINE_ERROR,
 				NULL );
 			g_object_unref( tag );
 		} 
 	}
 }
 
-/* G Object stuff */
-#define PARENT_TYPE GTK_TYPE_SOURCE_BUFFER
 
-static gpointer parent_class;
+static gboolean screem_page_auto_save( ScreemPage *page )
+{
+	ScreemPagePrivate *priv;
+	GError *error;
+	
+	priv = page->private;
+	
+	priv->auto_save_idle = 0;
+	
+	error = NULL;
+	if( ! screem_page_save( page, &error ) ) {
+	
+		if( error ) {
+			g_error_free( error );	
+		}
+	}
+	
+	return FALSE;
+}
+
+static void screem_page_auto_save_notify( GConfClient *client,
+					  guint cnxn_id,  GConfEntry *entry,
+					  gpointer data )
+{
+	ScreemPage *page;
+	ScreemPagePrivate *priv;
+
+	page = SCREEM_PAGE( data );
+	priv = page->private;
+
+	if( entry->value->type == GCONF_VALUE_BOOL ) {
+		gboolean state = gconf_value_get_bool( entry->value );
+
+		priv->auto_save = state;
+		
+		if( priv->auto_save_idle && ! state ) {
+			g_source_remove( priv->auto_save_idle );
+			priv->auto_save_idle = 0;
+		} else if( priv->changed && state && ! priv->auto_save_idle ) {
+			screem_page_set_changed_real( page, priv->changed );
+		}
+	}
+}
+
+static void screem_page_auto_save_interval_notify( GConfClient *client,
+					  guint cnxn_id,  GConfEntry *entry,
+					  gpointer data )
+{
+	ScreemPage *page;
+	ScreemPagePrivate *priv;
+
+	page = SCREEM_PAGE( data );
+	priv = page->private;
+
+	if( entry->value->type == GCONF_VALUE_INT ) {
+		gint interval = gconf_value_get_int( entry->value );
+
+		if( interval != priv->auto_save_interval ) {
+			priv->auto_save_interval = interval;		
+		
+			screem_page_set_changed_real( page, priv->changed );
+		}
+	}
+}
+
+
+/* G Object stuff */
+G_DEFINE_TYPE( ScreemPage, screem_page, GTK_TYPE_SOURCE_BUFFER )
+
+static void screem_page_set_prop( GObject *object, guint prop_id,
+				  const GValue *value, GParamSpec *spec );
+static void screem_page_get_prop( GObject *object, guint prop_id,
+				  GValue *value, GParamSpec *spec );
+static void screem_page_finalize( GObject *object );
+
 
 static void screem_page_class_init( ScreemPageClass *klass )
 {
 	GObjectClass *object_class;
-
+	GParamSpec *pspec;
+	
 	object_class = G_OBJECT_CLASS( klass );
-	parent_class = g_type_class_peek_parent( klass );
 
 	object_class->finalize = screem_page_finalize;
 	object_class->get_property = screem_page_get_prop;
 	object_class->set_property = screem_page_set_prop;
 
-	g_object_class_install_property(object_class,
-					PROP_APP,
-					g_param_spec_object("app",
-							    "Application",
-							    "The Application",
-							    G_TYPE_OBJECT,
-							    G_PARAM_READWRITE |
-							    G_PARAM_CONSTRUCT)
-					);
-	g_object_class_install_property(object_class,
-					PROP_PATHNAME,
-					g_param_spec_string("pathname",
-							    "Page pathname",
-							    "The pathname of the page",
-							    "",
-							    G_PARAM_READWRITE)
-					);
-	g_object_class_install_property(object_class,
-					PROP_CHANGED,
-					g_param_spec_boolean("changed",
-							     "changed",
-							     "If the page has changed",
-							     FALSE,
-							     G_PARAM_READWRITE)
-					);
-	g_object_class_install_property(object_class,
-					PROP_MIMETYPE,
-					g_param_spec_string("mime-type",
-							    "Page mime type",
-							    "The mime-type of the page",
-							    "",
-							    G_PARAM_READWRITE)
-					);
-	g_object_class_install_property(object_class,
-					PROP_OPEN,
-					g_param_spec_boolean("open",
-							     "open",
-							     "If the page is open",
-							     FALSE,
-							     G_PARAM_READWRITE)
-					);
-	g_object_class_install_property(object_class,
-					PROP_CHARSET,
-					g_param_spec_string("charset",
-							    "Page character set",
-							    "The character set of the page",
-							    "",
-							    G_PARAM_READABLE)
-					);
-	g_object_class_install_property(object_class,
-					PROP_DTD,
-					g_param_spec_object("dtd",
-							    "DTD",
-							    "The ScreemDTD for the page",
-							    G_TYPE_OBJECT,
-							    G_PARAM_READWRITE)
-					);
+
+	pspec = g_param_spec_object( "app",
+			"Application",
+			"The Application",
+			G_TYPE_OBJECT,
+			G_PARAM_READWRITE | G_PARAM_CONSTRUCT );
+	g_object_class_install_property( object_class, PROP_APP,
+			pspec );
+
+	pspec = g_param_spec_string( "pathname",
+			"Page pathname",
+			"The pathname of the page",
+			"",
+			G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, PROP_PATHNAME,
+			pspec );
+
+	pspec = g_param_spec_boolean( "changed",
+			"changed",
+			"If the page has changed",
+			FALSE,
+			G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, PROP_CHANGED,
+			pspec );
+
+	pspec = g_param_spec_string( "mime-type",
+			"Page mime type",
+			"The mime-type of the page",
+			"",
+			G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, PROP_MIMETYPE,
+			pspec );
+
+	pspec = g_param_spec_boolean( "open",
+			"open",
+			"If the page is open",
+			FALSE,
+			G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, PROP_OPEN,
+			pspec );
+
+	pspec = g_param_spec_string( "charset",
+			"Page character set",
+			"The character set of the page",
+			"",
+			G_PARAM_READABLE );
+	g_object_class_install_property( object_class, PROP_CHARSET,
+			pspec );
+	
+	pspec = g_param_spec_object( "dtd",
+			"DTD",
+			"The ScreemDTD for the page",
+			G_TYPE_OBJECT,
+			G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, PROP_DTD,
+			pspec );
 	
 	screem_page_signals[ SAVING ] =
 		g_signal_new( "saving",
@@ -1187,27 +1513,28 @@ static void screem_page_class_init( ScreemPageClass *klass )
 
 static void screem_page_init( ScreemPage *page )
 {
-	page->private = g_new0( ScreemPagePrivate, 1 );
+	ScreemPagePrivate *priv;
+	
+	priv = page->private = g_new0( ScreemPagePrivate, 1 );
 
-	page->private->client = gconf_client_get_default();
-	page->private->highlight = gconf_client_get_bool(page->private->client, "/apps/screem/editor/highlight",
-							 NULL );
-	page->private->highlight_notify = 
-		gconf_client_notify_add( page->private->client,
+	priv->client = gconf_client_get_default();
+	priv->highlight = gconf_client_get_bool( priv->client,
+		"/apps/screem/editor/highlight", NULL );
+	priv->highlight_notify = 
+		gconf_client_notify_add( priv->client,
 					 "/apps/screem/editor/highlight",
 					 screem_page_highlight_notify,
 					 page, NULL, NULL );
 	
-	page->private->bookmarks = g_hash_table_new( g_str_hash, g_str_equal );
+	priv->bookmarks = g_hash_table_new( g_str_hash, g_str_equal );
 
 	/* NAME, LINE, BOOKMARK STRUCT */
-	page->private->bookmarkstore = gtk_list_store_new( 3, G_TYPE_STRING,
-							   G_TYPE_INT,
-							   G_TYPE_POINTER,
-							   NULL );	
-	page->private->mime_type = gconf_client_get_string( page->private->client, "/apps/screem/default_mimetype", NULL );
-	if( ! page->private->mime_type ) {
-		page->private->mime_type = g_strdup( "text/html" );
+	priv->bookmarkstore = gtk_list_store_new( 3, G_TYPE_STRING,
+		G_TYPE_INT, G_TYPE_POINTER, NULL );	
+	priv->mime_type = gconf_client_get_string( priv->client, 
+		"/apps/screem/default_mimetype", NULL );
+	if( ! priv->mime_type ) {
+		priv->mime_type = g_strdup( "text/html" );
 	}
 	
 	g_signal_connect( G_OBJECT( page ),
@@ -1215,6 +1542,20 @@ static void screem_page_init( ScreemPage *page )
 
 	gtk_text_buffer_add_selection_clipboard( GTK_TEXT_BUFFER( page ),
 			gtk_clipboard_get( GDK_SELECTION_PRIMARY ) );
+
+	priv->auto_save = gconf_client_get_bool( priv->client,
+			"/apps/screem/general/auto_save", NULL );
+	priv->auto_save_interval = gconf_client_get_int( priv->client,
+			"/apps/screem/general/auto_save_interval", NULL );
+
+	priv->auto_save_notify = gconf_client_notify_add( priv->client,
+			"/apps/screem/general/auto_save",
+			screem_page_auto_save_notify, page, NULL, NULL );
+	priv->auto_save_interval_notify = gconf_client_notify_add( priv->client,
+			"/apps/screem/general/auto_save_interval",
+			screem_page_auto_save_interval_notify, page, NULL, NULL );
+
+	g_get_current_time( &priv->time );
 }
 
 static void screem_page_set_prop( GObject *object, guint prop_id,
@@ -1306,11 +1647,17 @@ static void screem_page_get_prop( GObject *object, guint prop_id,
 static void screem_page_finalize( GObject *object )
 {
 	ScreemPage *page;
+	ScreemPagePrivate *priv;
 
 	page = SCREEM_PAGE( object );
+	priv = page->private;
 
-	gconf_client_notify_remove( page->private->client, 
-				    page->private->highlight_notify );
+	gconf_client_notify_remove( priv->client, 
+				    priv->highlight_notify );
+	gconf_client_notify_remove( priv->client,
+					priv->auto_save_notify );
+	gconf_client_notify_remove( priv->client,
+					priv->auto_save_interval_notify );
 
 	g_hash_table_foreach( page->private->bookmarks,
 			      (GHFunc)screem_page_clear_bookmarks, page );
@@ -1330,30 +1677,6 @@ static void screem_page_finalize( GObject *object )
 
 	g_free( page->private );
 
-	G_OBJECT_CLASS( parent_class )->finalize( object );
+	G_OBJECT_CLASS( screem_page_parent_class )->finalize( object );
 }
 
-GType screem_page_get_type()
-{
-	static GType type = 0;
-	
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemPageClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_page_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemPage ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_page_init
-		};
-
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemPage",
-					       &info, 0 );
-	}
-
-	return type;
-}
