@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -47,12 +47,9 @@
 
 #include "screem-window.h"
 #include "screem-window-private.h"
-#include "screem-editor.h"
 #include "screem-page.h"
 #include "screem-dtd.h"
 
-#include "screem-file-browser.h"
-
 #include "fileops.h"
 #include "support.h"
 
@@ -65,7 +62,7 @@
 static GSList *trees = NULL;
 static GSList *treefiles = NULL;
 static GtkListStore *filesstore = NULL;
-static GMutex *mutex = NULL;
+static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 
 static xmlDocPtr config = NULL;
 static gchar *configpath = NULL;
@@ -109,7 +106,7 @@ static void screem_tag_tree_add_dtd( ScreemPage *page, GParamSpec *spec,
 
 static void screem_tagtree_sel_change( GtkTreeSelection *selection,
 					ScreemTagTree *tree );
-static void screem_tag_tree_set_model( ScreemTagTree *tree ); 
+static gboolean screem_tag_tree_set_model( ScreemTagTree *tree ); 
 static gpointer screem_tag_tree_create( gpointer data );
 static void screem_tag_tree_add_file( ScreemTagTree *tree,
 					ScreemTagFile *file,
@@ -163,11 +160,11 @@ ScreemTagTree *screem_tag_tree_new()
 
 gboolean screem_tag_tree_load_trees( void )
 {
+	GConfClient *client;
 	gchar *dot;
 
-	if( ! mutex ) {
-		gdk_threads_enter();
-		mutex = g_mutex_new();
+	g_static_mutex_lock( &mutex );
+	if( ! filesstore ) {
 		filesstore = gtk_list_store_new( MAX_TAG_FILE_COLS,
 						G_TYPE_STRING,
 						G_TYPE_STRING,
@@ -184,19 +181,27 @@ gboolean screem_tag_tree_load_trees( void )
 			config = xmlParseFile( configpath );
 		}
 		if( ! config ) {
-			config = xmlNewDoc( XML_DEFAULT_VERSION );
+			config = xmlNewDoc( (const xmlChar*)XML_DEFAULT_VERSION );
 			xmlDocSetRootElement( config,
 					xmlNewDocNode( config, NULL,
-						"refconfig", NULL ) );
+						(const xmlChar*)"refconfig", NULL ) );
 		}
-		gdk_threads_leave();
 	}
-#if 0
-	g_idle_add( screem_tag_tree_create, NULL );
-#else
-	g_thread_create( screem_tag_tree_create, NULL, FALSE, NULL );
-#endif
+	g_static_mutex_unlock( &mutex );
+
+#if 1
 	
+	client = gconf_client_get_default();
+	if( ! gconf_client_get_bool( client,
+				"/apps/screem/general/threaded_tree_load",
+				NULL ) ) {
+		g_idle_add( (GSourceFunc)screem_tag_tree_create, NULL );
+	} else {
+		g_thread_create( screem_tag_tree_create, NULL, 
+				FALSE, NULL );
+	}
+	g_object_unref( client );
+#endif	
 	return FALSE;
 }
 
@@ -227,6 +232,7 @@ gboolean screem_tag_tree_add_uri( const gchar *uri,
 	gchar *desc;
 	GtkTreeIter it;
 	xmlNodePtr node;
+	xmlChar *enable;
 
 	/* don't add already existing uris */
 	if( g_slist_find_custom( treefiles, uri,
@@ -238,7 +244,7 @@ gboolean screem_tag_tree_add_uri( const gchar *uri,
 	screem_tag_file_set_format( tfile, format );
 
 	screem_tag_file_set_uri( tfile, uri );
-	g_mutex_lock( mutex );
+	g_static_mutex_lock( &mutex );
 	active = screem_tag_tree_is_active( uri, &name, &desc );
 	if( name ) {
 		screem_tag_file_set_name( tfile, name );
@@ -248,43 +254,34 @@ gboolean screem_tag_tree_add_uri( const gchar *uri,
 	}
 
 	treefiles = g_slist_prepend( treefiles, tfile );
-	g_signal_connect( G_OBJECT( tfile ),
-			"notify::active",
-			G_CALLBACK( screem_tag_file_active_set ),
-			NULL );
 	gdk_threads_enter();
-	gtk_list_store_append( filesstore, &it );
-	gtk_list_store_set( filesstore, &it,
+	/* use large pos to ensure append */
+	gtk_list_store_insert_with_values( filesstore, &it,
+			G_MAXINT32,
 			TAG_FILE_NAME_COL, name,
 			TAG_FILE_URI_COL, uri,
 			TAG_FILE_DESC_COL, desc,
 			TAG_FILE_OBJ_COL, tfile,
 			-1 );	
-	gdk_threads_leave();
-	g_mutex_unlock( mutex );
+	g_static_mutex_unlock( &mutex );
 	g_free( name );
 	g_free( desc );
+	gdk_threads_leave();
 	
-	g_object_set( G_OBJECT( tfile ), "active", active, NULL );
-	
-	if( active ) {
-		g_mutex_lock( mutex );	
-		node = screem_tag_tree_get_file_node( uri );
-		if( ! node ) {
-			/* new node being added, add to config */
-			node = xmlDocGetRootElement( config );
-			node = xmlNewChild( node, NULL, "tagfile", NULL );
-			xmlSetProp( node, "src", uri );
-			if( name ) {
-				xmlSetProp( node, "name", name );
-			}
-			if( desc ) {
-				xmlSetProp( node, "description", desc );
-			}
-			xmlSetProp( node, "enable", "1" );
-			screem_tag_tree_save_config();
+	g_signal_connect( G_OBJECT( tfile ),
+			"notify::active",
+			G_CALLBACK( screem_tag_file_active_set ),
+			NULL );
+	node = screem_tag_tree_get_file_node( uri );
+	if( node ) {
+		enable = xmlGetProp( node, (const xmlChar*)"enable" );
+		if( enable && *enable == '1' ) {
+			screem_tag_file_load( tfile, 
+					SCREEM_TAG_UNKNOWN_FORMAT );
+		}
+		if( enable ) {
+			xmlFree( enable );
 		}
-		g_mutex_unlock( mutex );
 	}
 	
 	return TRUE;
@@ -295,11 +292,12 @@ void screem_tag_tree_remove_uri( const gchar *uri )
 	GSList *tmp;
 	const gchar *furi;
 	ScreemTagFile *tfile;
-	GtkTreeIter *it;
+	GtkTreeIter it;
 	gboolean active;
 	xmlNodePtr node;
+	gboolean found;
 	
-	g_mutex_lock( mutex );
+	g_static_mutex_lock( &mutex );
 
 	tmp = g_slist_find_custom( treefiles, uri,
 			(GCompareFunc)screem_tag_tree_find_tag_file );
@@ -307,12 +305,13 @@ void screem_tag_tree_remove_uri( const gchar *uri )
 	if( tmp ) {
 		tfile = SCREEM_TAGFILE( tmp->data );
 		furi = screem_tag_file_get_uri( tfile );
-		it = screem_support_find_in_list( filesstore,
-				TAG_FILE_URI_COL, furi );
-		g_assert( it );
 
-		gtk_list_store_remove( filesstore, it );
-		gtk_tree_iter_free( it );
+		found = screem_gtk_list_store_find_string( filesstore, 
+				&it, TAG_FILE_URI_COL, furi );
+
+		g_assert( found );
+
+		gtk_list_store_remove( filesstore, &it );
 
 		treefiles = g_slist_remove( treefiles, tfile );
 
@@ -332,7 +331,7 @@ void screem_tag_tree_remove_uri( const gchar *uri )
 		}
 		g_object_unref( tfile );
 	}
-	g_mutex_unlock( mutex );
+	g_static_mutex_unlock( &mutex );
 }
 
 GSList *screem_tag_tree_autocomplete( const gchar *mime_type,
@@ -350,21 +349,24 @@ GSList *screem_tag_tree_autocomplete( const gchar *mime_type,
 
 	ret = NULL;
 
-	g_mutex_lock( mutex );
+	g_static_mutex_lock( &mutex );
 	
 	for( tmp = treefiles; tmp; tmp = tmp->next ) {
 		tfile = SCREEM_TAGFILE( tmp->data );
 
 		type = screem_tag_file_for_type( tfile );
-		if( type && ! strcmp( mime_type, type ) ) {
+		if( type ) {
+			if( ( ! strcmp( mime_type, type ) ) || 
+				( gnome_vfs_mime_type_get_equivalence( mime_type, (gchar*)type ) == GNOME_VFS_MIME_PARENT ) ) {
 
-			tret = screem_tag_file_autocomplete( tfile,
-					prefix, fortip );
-			ret = g_slist_concat( ret, tret );
+				tret = screem_tag_file_autocomplete( tfile,
+						prefix, fortip );
+				ret = g_slist_concat( ret, tret );
+			}
 		}
 	}
 	
-	g_mutex_unlock( mutex );
+	g_static_mutex_unlock( &mutex );
 	
 	return ret;
 }
@@ -380,7 +382,8 @@ static const gchar *screem_tag_tree_dtd_cache( ScreemTagTree *tree,
 	GString *str;
 	gchar *data;
 	gchar *escaped;
-
+	gboolean cache;
+	
 	g_return_val_if_fail( SCREEM_IS_DTD( dtd ), NULL );
 	
 	ret = g_object_get_data( G_OBJECT( dtd ),
@@ -390,6 +393,9 @@ static const gchar *screem_tag_tree_dtd_cache( ScreemTagTree *tree,
 		gboolean html;
 		const gchar *closing;
 		
+		cache = ( g_object_get_data( G_OBJECT( dtd ),
+				"NO_CACHE" ) == NULL );
+		
 		elements = screem_dtd_get_elements( dtd );
 	
 		html = FALSE;
@@ -478,18 +484,27 @@ static const gchar *screem_tag_tree_dtd_cache( ScreemTagTree *tree,
 		}
 
 		g_string_append( str, "</ref></refs>" );
-		
-		tmp = g_slist_append( tree->private->dtdcache,
-					dtd );
-		tree->private->dtdcache = tmp;
+
 		ret = str->str;
 
-		g_object_set_data( G_OBJECT( dtd ), "tagtree_string",
+		
+		if( cache ) {
+			tmp = g_slist_append( tree->private->dtdcache,
+					dtd );
+			tree->private->dtdcache = tmp;
+			g_object_set_data( G_OBJECT( dtd ), 
+					"tagtree_string",
 				   (gpointer)ret );
+		} else {
+			g_object_set_data_full( G_OBJECT( dtd ), 
+					"tagtree_string",
+					(gpointer)ret, g_free );
+		}
 		
 		g_string_free( str, FALSE );
 		
-		if( ( ++ tree->private->dtdcachelen ) > 4 ) {
+		if( cache &&
+			( ++ tree->private->dtdcachelen ) > 4 ) {
 			tmp = tree->private->dtdcache;
 			tree->private->dtdcache = tmp->next;
 			tmp->next = NULL;
@@ -509,18 +524,32 @@ static const gchar *screem_tag_tree_dtd_cache( ScreemTagTree *tree,
 static void screem_tag_tree_add_dtd( ScreemPage *page, GParamSpec *spec,
 				     ScreemTagTree *tag_tree )
 {
+	ScreemTagTreePrivate *priv;
 	ScreemDTD *dtd;
 	const gchar *str;
-
+	
+	priv = tag_tree->private;
+	
 	if( spec ) {
-		tag_tree->private->dtd = screem_page_get_dtd( page );
+		dtd = screem_page_get_dtd( page );
+		if( dtd == priv->dtd ) {
+			return;
+		}
+		if( dtd ) {
+			g_object_ref( dtd );
+		}
+		if( priv->dtd ) {
+			g_object_unref( priv->dtd );
+		}
+		priv->dtd = dtd;
+	}
+	
+	dtd = priv->dtd;
+	if( dtd ) {
+		str = screem_tag_tree_dtd_cache( tag_tree, dtd );
+		screem_tag_file_set_from_string( priv->dtdbranch,
+						str, 0 );
 	}
-
-	dtd = tag_tree->private->dtd;
-	str = screem_tag_tree_dtd_cache( tag_tree, dtd );
-
-	screem_tag_file_set_from_string( tag_tree->private->dtdbranch,
-					str, 0 );
 }
 
 static gboolean screem_tag_tree_click( GtkTreeView *tree_view,
@@ -541,15 +570,15 @@ static gboolean screem_tag_tree_click( GtkTreeView *tree_view,
 	if( sel ) {
 		NodeInfo *info;
 		ScreemWindow *window;
-		ScreemEditor *editor;
+		ScreemView *editor;
 
 		window = SCREEM_WINDOW( tag_tree->private->window );
 
-		editor = SCREEM_EDITOR( window->details->editor );
+		editor = SCREEM_VIEW( window->details->editor );
 
 		gtk_tree_selection_get_selected( sel, &model, &iter );
-		gtk_tree_model_get( model, &iter, FILE_BROWSER_DATA_COL,
-					 &info, -1 );
+		gtk_tree_model_get( model, &iter, 
+				SCREEM_TAG_FILE_DATA, &info, -1 );
 
 		if( info && ( ! info->text ) && info->file ) {
 			GnomeVFSURI *vuri;
@@ -559,13 +588,11 @@ static gboolean screem_tag_tree_click( GtkTreeView *tree_view,
 				screem_window_show_error( window, _( "Invalid URI for requested resource" ), FALSE );
 			} else if( gnome_vfs_uri_is_local( vuri ) ||
 				  ! window->details->offline ) {
-				GString *data;
-				
-				data = load_file( info->file, NULL, NULL, NULL );
-				/* now set info->text to be the file contents,
-			   	this way we don't refetch files we already have */
-				info->text = data->str;
-				g_string_free( data, FALSE );
+				/*  set info->text to be the file 
+				 *  contents, this way we don't
+				 *  refetch files we already have */
+				info->text = load_file( info->file, 
+						NULL, NULL, NULL );
 			} else {
 				screem_window_show_message( window, _( "Remote resources are not available in offline mode" ), FALSE );	
 			}
@@ -574,8 +601,8 @@ static gboolean screem_tag_tree_click( GtkTreeView *tree_view,
 				gnome_vfs_uri_unref( vuri );
 			}
 		} else if( info && info->open ) {
-			screem_editor_insert_markup( editor, info->open,
-						     info->close );
+			screem_view_insert_markup( SCREEM_VIEW( editor ),
+					info->open, info->close );
 		}
 		/* we do info->text here so you can have something like
 		 * <insert open="&lt;title&gt;" close="&lt;title&gt;">
@@ -583,10 +610,11 @@ static gboolean screem_tag_tree_click( GtkTreeView *tree_view,
 		 * </insert>
 		 */
 		if( info && info->text ) {
-			screem_editor_insert( editor, -1, info->text );
+			screem_view_insert( SCREEM_VIEW( editor ),
+					-1, info->text );
 			if( g_str_has_suffix( info->text, ")" ) ) {
-				pos = screem_editor_get_pos( editor );
-				screem_editor_set_pos ( editor, pos - 1 );
+				pos = screem_view_get_pos( SCREEM_VIEW( editor ) );
+				screem_view_set_pos( SCREEM_VIEW( editor ), pos - 1 );
 			}
 		}
 	} else {
@@ -622,8 +650,8 @@ static void screem_tagtree_sel_change( GtkTreeSelection *selection,
 	rows = gtk_tree_selection_count_selected_rows( selection );
 	if( rows > 0 ) {
 		gtk_tree_selection_get_selected( selection, &model, &iter );
-		gtk_tree_model_get( model, &iter, FILE_BROWSER_DATA_COL,
-					 &info, -1 );
+		gtk_tree_model_get( model, &iter, 
+				SCREEM_TAG_FILE_DATA, &info, -1 );
 		if( info && info->description ) {
 			screem_window_show_message( window, info->description, FALSE );
 		}
@@ -631,23 +659,18 @@ static void screem_tagtree_sel_change( GtkTreeSelection *selection,
 	gtk_widget_set_sensitive( priv->toolbar, ( rows > 0 ) );
 }
 
-static void screem_tag_tree_set_model( ScreemTagTree *tree )
+static gboolean screem_tag_tree_set_model( ScreemTagTree *tree )
 {
 	ScreemTagTreePrivate *priv;
 	GtkTreeModel *model;
 	
-	g_return_if_fail( SCREEM_IS_TAG_TREE( tree ) );
+	g_return_val_if_fail( SCREEM_IS_TAG_TREE( tree ), FALSE );
 	
 	priv = tree->private;
 
-	priv->umodel = egg_tree_model_union_new( FILE_BROWSER_MAX_COL,
-					   G_TYPE_STRING,
-					   GDK_TYPE_PIXBUF,
-					   G_TYPE_POINTER,
-					   G_TYPE_STRING,
-					   G_TYPE_POINTER,
-					   G_TYPE_STRING,
-					   G_TYPE_POINTER );
+	priv->umodel = egg_tree_model_union_new( SCREEM_TAG_FILE_COLS,
+			GDK_TYPE_PIXBUF, G_TYPE_STRING, 
+			G_TYPE_POINTER );
 
 	priv->dtdbranch = screem_tag_file_new();
 
@@ -655,16 +678,19 @@ static void screem_tag_tree_set_model( ScreemTagTree *tree )
 	gtk_tree_sortable_set_sort_func( GTK_TREE_SORTABLE( model ),
 			0, screem_tag_tree_sort, tree, NULL );
 	gtk_tree_sortable_set_sort_column_id( GTK_TREE_SORTABLE( model ),
-			FILE_BROWSER_NAME_COL, GTK_SORT_ASCENDING );
+			SCREEM_TAG_FILE_NAME, GTK_SORT_ASCENDING );
 	priv->sorted = model;
 
-	gdk_threads_leave();
 	screem_tag_tree_add_file( tree, priv->dtdbranch, TRUE );
-	gdk_threads_enter();
 
+	gdk_threads_enter();
 	gtk_tree_view_set_model( GTK_TREE_VIEW( tree->private->view ), 
 			model );
+	gdk_threads_leave();
+
 	g_object_unref( model );
+
+	return FALSE;
 }
 
 static gint tagtree_filter( const GnomeVFSFileInfo *info )
@@ -721,16 +747,16 @@ static gpointer screem_tag_tree_create( gpointer data )
 	GSList *tmp;
 	
 	gint i;
-	const gchar *treepaths[] = {
+	gchar *treepaths[] = {
 		DATADIR"/screem/tagtrees",
 		NULL,
 		NULL
 	};
 	xmlNodePtr node;
 	xmlNodePtr dead;
-	gchar *file;
+	xmlChar *file;
 	GnomeVFSResult res;
-	
+
 	/* FIXME: windows should show a busy-interactive cursor
 	   while this function is running */
 
@@ -750,17 +776,17 @@ static gpointer screem_tag_tree_create( gpointer data )
 
 	/* now load any additional trees that may be present in the
 	 * tagtreeconfig.xml */
-	g_mutex_lock( mutex );
+	g_static_mutex_lock( &mutex );
 	files = NULL;
 	node = xmlDocGetRootElement( config );
 	node = node->children;
 	while( node ) {
-		file = xmlGetProp( node, "src" );
+		file = xmlGetProp( node, (const xmlChar*)"src" );
 
-		if( uri_exists( file, &res ) ) {
+		if( file && uri_exists( (const gchar*)file, &res ) ) {
 			files = g_slist_prepend( files, file );
 			node = node->next;
-		} else if( res == GNOME_VFS_ERROR_NOT_FOUND ){
+		} else if( file && res == GNOME_VFS_ERROR_NOT_FOUND ){
 			/* remove if not found, we will
 			 * allow other errors to keep the uri
 			 * in the tag config */
@@ -768,16 +794,17 @@ static gpointer screem_tag_tree_create( gpointer data )
 			node = node->next;
 			xmlUnlinkNode( dead );
 			xmlFreeNode( dead );
+			xmlFree( file );
 		}
 	}
 	screem_tag_tree_save_config();
-	g_mutex_unlock( mutex );
+	g_static_mutex_unlock( &mutex );
 	for( tmp = files; tmp; tmp = tmp->next ) {
 		screem_tag_tree_add_screem_format( tmp->data );
 		g_free( tmp->data );
 	}
 	g_slist_free( files );
-	
+
 	return NULL;
 }
 
@@ -818,8 +845,8 @@ static void screem_tag_tree_tag_help( GtkAction *action, ScreemTagTree *tree )
 	rows = gtk_tree_selection_count_selected_rows( selection );
 	if( rows > 0 ) {
 		gtk_tree_selection_get_selected( selection, &model, &it );
-		gtk_tree_model_get( model, &it, FILE_BROWSER_DATA_COL,
-					 &info, -1 );
+		gtk_tree_model_get( model, &it, 
+				SCREEM_TAG_FILE_DATA, &info, -1 );
 		if( info ) {
 			GtkWidget *dialog;
 			GtkWidget *box;
@@ -827,7 +854,7 @@ static void screem_tag_tree_tag_help( GtkAction *action, ScreemTagTree *tree )
 			GtkWidget *notebook;
 			gchar *title;
 			ScreemWindow *window;
-			ScreemEditor *editor;
+			ScreemView *editor;
 			GtkWidget *sw;
 			GtkTextBuffer *buffer;
 			gchar *txt;
@@ -838,7 +865,7 @@ static void screem_tag_tree_tag_help( GtkAction *action, ScreemTagTree *tree )
 			gchar *attrtxt;
 			
 			g_object_get( G_OBJECT( tree ), "window", &window, NULL );
-			editor = SCREEM_EDITOR( window->details->editor );
+			editor = SCREEM_VIEW( window->details->editor );
 			
 			box = gtk_vbox_new( FALSE, 0 );
 			gtk_box_set_spacing( GTK_BOX( box ), 6 );
@@ -861,12 +888,15 @@ static void screem_tag_tree_tag_help( GtkAction *action, ScreemTagTree *tree )
 			gtk_label_set_justify( GTK_LABEL( widget ), GTK_JUSTIFY_LEFT );
 			gtk_misc_set_alignment( GTK_MISC( widget ), 0.0, 0.0 );
 			gtk_box_pack_start( GTK_BOX( box ), widget, FALSE, TRUE, 0 );
+			gtk_label_set_line_wrap( GTK_LABEL( widget ), TRUE );
+			gtk_label_set_selectable( GTK_LABEL( widget ), TRUE );
 	
 
 			widget = gtk_label_new( info->description );
 			gtk_label_set_justify( GTK_LABEL( widget ), GTK_JUSTIFY_LEFT );
 			gtk_misc_set_alignment( GTK_MISC( widget ), 0.0, 0.0 );
 			gtk_label_set_line_wrap( GTK_LABEL( widget ), TRUE );
+			gtk_label_set_selectable( GTK_LABEL( widget ), TRUE );
 			gtk_box_pack_start( GTK_BOX( box ), widget, FALSE, TRUE, 0 );
 
 			txt = NULL;
@@ -878,13 +908,13 @@ static void screem_tag_tree_tag_help( GtkAction *action, ScreemTagTree *tree )
 					txt = g_strdup( _( "Invalid URI for requested resource" ) );
 				} else if( gnome_vfs_uri_is_local( vuri ) ||
 					  ! window->details->offline ) {
-					GString *data;
-				
-					data = load_file( info->file, NULL, NULL, NULL );
-					/* now set info->text to be the file contents,
-				   	this way we don't refetch files we already have */
-					info->text = data->str;
-					g_string_free( data, FALSE );
+					/* now set info->text to be the
+					 * file contents, this way we
+					 * don't refetch files we
+					 * already have */
+					info->text = load_file( info->file, 
+							NULL, NULL, 
+							NULL );
 				} else {
 					txt = g_strdup( _( "Remote resources are not available in offline mode" ) );	
 				}
@@ -943,6 +973,8 @@ static void screem_tag_tree_tag_help( GtkAction *action, ScreemTagTree *tree )
 				
 					gtk_label_set_justify( GTK_LABEL( widget ), GTK_JUSTIFY_LEFT );
 					gtk_misc_set_alignment( GTK_MISC( widget ), 0.0, 0.0 );
+					gtk_label_set_line_wrap( GTK_LABEL( widget ), TRUE );
+					gtk_label_set_selectable( GTK_LABEL( widget ), TRUE );
 
 					gtk_box_pack_start( GTK_BOX( attrbox ), widget, FALSE, FALSE, 0 );
 					gtk_box_reorder_child( GTK_BOX( attrbox ), widget, 0 );
@@ -1004,9 +1036,9 @@ static void screem_tag_tree_tag_help( GtkAction *action, ScreemTagTree *tree )
 			if( gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_APPLY ) {
 				/* insert txt into the editor */
 				if( info->open ) {
-					screem_editor_insert_markup( editor, info->open, info->close );
+					screem_view_insert_markup( SCREEM_VIEW( editor ), info->open, info->close );
 				} else {
-					screem_editor_insert( editor, -1, txt );
+					screem_view_insert( SCREEM_VIEW( editor ), -1, txt );
 				}
 			}
 			g_free( txt );
@@ -1028,7 +1060,9 @@ static gboolean screem_tag_tree_page_set( gpointer data )
 	tree = SCREEM_TAG_TREE( data );
 	page = tree->private->page;
 
-	gdk_threads_enter();
+	if( ! tree->private->dtdbranch ) {
+		return TRUE;
+	}
 	
 	if( page ) {
 		g_signal_connect(G_OBJECT( page ), "notify::dtd",
@@ -1036,8 +1070,16 @@ static gboolean screem_tag_tree_page_set( gpointer data )
 				 tree );
 		dtd = screem_page_get_dtd( page );
 		if( dtd != tree->private->dtd ) {
+			if( dtd ) {
+				g_object_ref( dtd );
+			}
+			if( tree->private->dtd ) {
+				g_object_unref( tree->private->dtd );
+			}
 			tree->private->dtd = dtd;
+			gdk_threads_enter();
 			screem_tag_tree_add_dtd( page, NULL, tree );
+			gdk_threads_leave();
 		}
 	} else {
 		tree->private->dtd = NULL;
@@ -1045,8 +1087,6 @@ static gboolean screem_tag_tree_page_set( gpointer data )
 
 	tree->private->timeout = 0;
 	
-	gdk_threads_leave();
-	
 	return FALSE;
 }
 
@@ -1093,39 +1133,59 @@ static void screem_tag_tree_remove_file( ScreemTagTree *tree,
 	}
 }
 
-static void screem_tag_file_active_set( ScreemTagFile *file,
-					GParamSpec *spec,
-					gpointer data )
+static gboolean active_set_cb( ScreemTagFile *file )
 {
 	gboolean active;
 	const gchar *uri;
-	GtkTreeIter *it;
+	GtkTreeIter it;
 	GSList *tmp;
 	xmlNodePtr node;
 	
 	const gchar *name;
 	const gchar *desc;
+
+	gboolean found;
 	
-	g_mutex_lock( mutex );
+	g_static_mutex_lock( &mutex );
 
 	uri = screem_tag_file_get_uri( file );
 	g_object_get( G_OBJECT( file ), 
 			"active", &active, 
 			NULL );
 	
-	it = screem_support_find_in_list( filesstore, 
-			TAG_FILE_URI_COL, uri );
-	g_assert( it );
+	found = screem_gtk_list_store_find_string( filesstore, 
+				&it, TAG_FILE_URI_COL, uri );
+	g_assert( found );
 
 	name = screem_tag_file_get_name( file );
 	desc = screem_tag_file_get_desc( file );
-
-	gtk_list_store_set( filesstore, it,
+		
+	node = screem_tag_tree_get_file_node( uri );
+	if( ! node ) {
+		/* new node being added, add to config */
+		node = xmlDocGetRootElement( config );
+		node = xmlNewChild( node, NULL, 
+				(const xmlChar*)"tagfile", NULL );
+	
+		xmlSetProp( node, (const xmlChar*)"src", 
+				(xmlChar*)uri );
+		if( name ) {
+			xmlSetProp( node, (const xmlChar*)"name", 
+					(xmlChar*)name );
+		}
+		if( desc ) {
+			xmlSetProp( node, 
+					(const xmlChar*)"description",
+					(xmlChar*)desc );
+		}
+	}
+	
+	gdk_threads_enter();
+	gtk_list_store_set( filesstore, &it,
 			TAG_FILE_NAME_COL, name,
 			TAG_FILE_DESC_COL, desc,
 			TAG_FILE_ACTIVE_COL, active, -1 );
-
-	gtk_tree_iter_free( it );
+	gdk_threads_leave();
 
 	if( active ) {
 		for( tmp = trees; tmp; tmp = tmp->next ) {
@@ -1139,13 +1199,22 @@ static void screem_tag_file_active_set( ScreemTagFile *file,
 		}
 	}
 
-	if( ( node = screem_tag_tree_get_file_node( uri ) ) ) {
-		xmlSetProp( node, "enable", active ? "1" : "0" );
-	}
-
+	xmlSetProp( node, (const xmlChar*)"enable", 
+			active ? (const xmlChar*)"1" : 
+			(const xmlChar*) "0" );
+	
 	screem_tag_tree_save_config();
 	
-	g_mutex_unlock( mutex );
+	g_static_mutex_unlock( &mutex );
+
+	return FALSE;
+}
+
+static void screem_tag_file_active_set( ScreemTagFile *file,
+					GParamSpec *spec,
+					gpointer data )
+{
+	g_idle_add( (GSourceFunc)active_set_cb, file );
 }
 
 /* NOTE: do not call without the mutex locked */
@@ -1161,14 +1230,15 @@ static void screem_tag_tree_save_config( void )
 		uri = screem_tag_file_get_uri( SCREEM_TAGFILE( tmp->data ) );
 		node = screem_tag_tree_get_file_node( uri );
 
+		/* possible we don't find a node for new tag
+		 * files that are still loading, waiting to be loaded */
 		if( node ) {
 			name = screem_tag_file_get_name( SCREEM_TAGFILE( tmp->data ) );
 			desc = screem_tag_file_get_desc( SCREEM_TAGFILE( tmp->data ) );
-			xmlSetProp( node, "name", name );
-			xmlSetProp( node, "description", desc );
-		} else {
-			g_warning( "ScreemTagFile %p not in config",
-					tmp->data );
+			xmlSetProp( node, (const xmlChar*)"name", 
+					(xmlChar*)name );
+			xmlSetProp( node, (const xmlChar*)"description",
+					(xmlChar*)desc );
 		}
 	}
 	
@@ -1189,9 +1259,9 @@ static xmlNodePtr screem_tag_tree_get_file_node( const gchar *url )
 	
 	ret = ret->children;
 	while( ret ) {
-		src = xmlGetProp( ret, "src" );
+		src = xmlGetProp( ret, (const xmlChar*)"src" );
 		if( src ) {
-			if( ! strcmp( src, url ) ) {
+			if( ! strcmp( (const gchar*)src, url ) ) {
 				xmlFree( src );
 				break;
 			}
@@ -1221,13 +1291,13 @@ static gboolean screem_tag_tree_is_active( const gchar *url,
 
 	node = screem_tag_tree_get_file_node( url );
 	if( node ) {
-		enable = xmlGetProp( node, "enable" );
+		enable = xmlGetProp( node, (const xmlChar*)"enable" );
 		if( enable ) {
 			ret = ( *enable == '1' );
 			xmlFree( enable );
 		}
-		*name = xmlGetProp( node, "name" );
-		*desc = xmlGetProp( node, "description" );
+		*name = (gchar*)xmlGetProp( node, (const xmlChar*)"name" );
+		*desc = (gchar*)xmlGetProp( node, (const xmlChar*)"description" );
 	} 	
 
 	return ret;
@@ -1238,7 +1308,6 @@ static void screem_tag_tree_search( GtkAction *action,
 {
 	ScreemTagTree *tree;
 	ScreemTagTreePrivate *priv;
-	gchar *glade_path;
 	GtkWidget *widget;
 	GtkTreeModel *model;
 	GladeXML *xml;
@@ -1264,9 +1333,8 @@ static void screem_tag_tree_search( GtkAction *action,
 		gtk_tree_path_free( path );
 	}
 	
-	glade_path = screem_get_glade_path();	
-	xml = glade_xml_new( glade_path, "find_tagtree", NULL );
-	g_free( glade_path );
+	xml = glade_xml_new( GLADE_PATH"/screem.glade", 
+			"find_tagtree", NULL );
 	
 	widget = glade_xml_get_widget( xml, "tagtrees" );
 	model = screem_top_level_model_new();
@@ -1285,13 +1353,9 @@ static void screem_tag_tree_search( GtkAction *action,
 	
 	gtk_tree_view_column_set_title( col, _( "Resources" ) );
 	gtk_tree_view_column_set_attributes( col, rend,
-					     "text", 
-					     FILE_BROWSER_NAME_COL,
-					     NULL );
+			"text", SCREEM_TAG_FILE_NAME, NULL );
 	gtk_tree_view_column_set_attributes( col, prend,
-					     "pixbuf",
-					     FILE_BROWSER_ICON_COL,
-					     NULL );
+			"pixbuf", SCREEM_TAG_FILE_ICON, NULL );
 
 	/* HIG says 1 column lists should have no header */
 	gtk_tree_view_set_headers_visible( GTK_TREE_VIEW( widget ), 
@@ -1310,18 +1374,23 @@ static void screem_tag_tree_search( GtkAction *action,
 		gtk_tree_path_free( path );
 	}
 
+	widget = glade_xml_get_widget( xml, "tag_tree_find_pattern" );
+	screem_gtk_add_history( widget );
+	
 	widget = glade_xml_get_widget( xml, "find_tagtree" );
 	gtk_widget_show_all( widget );
 	
 	if( gtk_dialog_run( GTK_DIALOG( widget ) ) == GTK_RESPONSE_OK ) {
-		widget = glade_xml_get_widget( xml, "pattern" );
+		widget = glade_xml_get_widget( xml, "tag_tree_find_pattern" );
+		widget = GTK_BIN( widget )->child;
 		priv->search_pattern = gtk_entry_get_text( GTK_ENTRY( widget ) );
 		widget = glade_xml_get_widget( xml, "regexp" );
 		priv->regexp = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
 		if( gtk_tree_selection_get_selected( selection, &model, &it ) ) {
 			path = gtk_tree_model_get_path( model, &it );
 			gtk_tree_model_get( model, &it,
-					FILE_BROWSER_DATA_COL, &info, -1 );
+					SCREEM_TAG_FILE_DATA, &info,
+					-1 );
 			if( info ) {
 				model = screem_tag_file_get_model( SCREEM_TAGFILE( info->tfile ) );
 				gtk_tree_model_foreach( model, 
@@ -1362,19 +1431,18 @@ static gboolean screem_tag_tree_search_func( GtkTreeModel *model,
 	ScreemTagTree *tree;
 	ScreemTagTreePrivate *priv;
 	gchar *name;
-	gint len;
+	guint len;
 
 	tree = SCREEM_TAG_TREE( data );
 	priv = tree->private;
 
 	gtk_tree_model_get( model, iter,
-				FILE_BROWSER_NAME_COL, &name,
-				-1 );
+			SCREEM_TAG_FILE_NAME, &name, -1 );
 
 	priv->search_result = NULL;
 
 	if( ! screem_search_static( name, priv->search_pattern,
-				priv->regexp, 0, &len ) ) {
+				priv->regexp, 0, &len, NULL, NULL ) ) {
 		priv->search_result = gtk_tree_path_copy( path );
 	}
 
@@ -1394,11 +1462,9 @@ static gint screem_tag_tree_sort( GtkTreeModel *model,
 	
 	tree = SCREEM_TAG_TREE( data );
 	
-	gtk_tree_model_get( model, a,
-			FILE_BROWSER_NAME_COL, &aname,
+	gtk_tree_model_get( model, a, SCREEM_TAG_FILE_NAME, &aname,
 			-1 );
-	gtk_tree_model_get( model, b,
-			FILE_BROWSER_NAME_COL, &bname,
+	gtk_tree_model_get( model, b, SCREEM_TAG_FILE_NAME, &bname,
 			-1 );
 
 	ret = 0;
@@ -1417,14 +1483,14 @@ static gboolean screem_tag_tree_add_current( ScreemTagTree *tree )
 {
 	GSList *tmp;
 	
-	g_mutex_lock( mutex );
+	g_static_mutex_lock( &mutex );
 	trees = g_slist_prepend( trees, tree );
 
 	for( tmp = treefiles; tmp; tmp = tmp->next ) {
 		screem_tag_tree_add_file( tree, 
 				SCREEM_TAGFILE( tmp->data ), FALSE );
 	}
-	g_mutex_unlock( mutex );
+	g_static_mutex_unlock( &mutex );
 
 	return FALSE;
 }
@@ -1432,9 +1498,7 @@ static gboolean screem_tag_tree_add_current( ScreemTagTree *tree )
 
 
 /* G Object stuff */
-#define PARENT_TYPE GTK_TYPE_BIN
-
-static gpointer parent_class;
+G_DEFINE_TYPE( ScreemTagTree, screem_tag_tree, GTK_TYPE_BIN )
 
 static void screem_tag_tree_class_init( ScreemTagTreeClass *klass )
 {
@@ -1447,7 +1511,6 @@ static void screem_tag_tree_class_init( ScreemTagTreeClass *klass )
 	widget_class = (GtkWidgetClass *)klass;
 
 	object_class->finalize = screem_tag_tree_finalize;
-	parent_class = g_type_class_peek_parent( klass );
 
 	object_class->get_property = screem_tag_tree_get_prop;
 	object_class->set_property = screem_tag_tree_set_prop;
@@ -1481,7 +1544,7 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 	GtkTreeSelection *selection;
 	gchar *dot;
 
-	static GtkActionEntry action_entries[] = {
+	static const GtkActionEntry action_entries[] = {
 		{
 		   "SearchTagTree", GTK_STOCK_FIND,
 		   N_( "Find in Tag Tree..." ),
@@ -1494,15 +1557,15 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 	  	   G_CALLBACK( screem_tag_tree_tag_help ) 
 		}
 	};
-	static guint action_n_entries=G_N_ELEMENTS( action_entries );
+	static const guint action_n_entries=G_N_ELEMENTS( action_entries );
 	GtkActionGroup *group;
 	GtkUIManager *merge;
 	GError *error;
 	
 	ScreemTagTreePrivate *priv;
 
-	if( ! mutex ) {
-		mutex = g_mutex_new();
+	g_static_mutex_lock( &mutex );
+	if( ! filesstore ) {
 		filesstore = gtk_list_store_new( MAX_TAG_FILE_COLS,
 						G_TYPE_STRING,
 						G_TYPE_STRING,
@@ -1519,13 +1582,13 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 			config = xmlParseFile( configpath );
 		}
 		if( ! config ) {
-			config = xmlNewDoc( XML_DEFAULT_VERSION );
+			config = xmlNewDoc( (const xmlChar*)XML_DEFAULT_VERSION );
 			xmlDocSetRootElement( config,
 					xmlNewDocNode( config, NULL,
-						"refconfig", NULL ) );
+						(const xmlChar*)"refconfig", NULL ) );
 		}
 	}
-	
+	g_static_mutex_unlock( &mutex );
 	priv = tag_tree->private = g_new0( ScreemTagTreePrivate, 1 );
 
 	priv->box = gtk_vbox_new( FALSE, 0 );
@@ -1557,13 +1620,9 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 	
 	gtk_tree_view_column_set_title( col, _( "Resources" ) );
 	gtk_tree_view_column_set_attributes( col, rend,
-					     "text", 
-					     FILE_BROWSER_NAME_COL,
-					     NULL );
+			"text", SCREEM_TAG_FILE_NAME, NULL );
 	gtk_tree_view_column_set_attributes( col, prend,
-					     "pixbuf",
-					     FILE_BROWSER_ICON_COL,
-					     NULL );
+			"pixbuf", SCREEM_TAG_FILE_ICON, NULL );
 
 	/* HIG says 1 column lists should have no header */
 	gtk_tree_view_set_headers_visible( GTK_TREE_VIEW( priv->view ), FALSE ); 
@@ -1610,7 +1669,8 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 	
 	gtk_widget_show_all( GTK_WIDGET( tag_tree ) );
 
-	screem_tag_tree_set_model( tag_tree );
+	g_idle_add( (GSourceFunc)screem_tag_tree_set_model,
+			tag_tree );
 
 	g_idle_add( (GSourceFunc)screem_tag_tree_add_current,
 			tag_tree );
@@ -1632,7 +1692,7 @@ static void screem_tag_tree_finalize( GObject *object )
 	}
 
 	/* clear and unref union model */
-	g_mutex_lock( mutex );
+	g_static_mutex_lock( &mutex );
 	
 	for( tmp = treefiles; tmp; tmp = tmp->next ) {
 		g_object_get( G_OBJECT( tmp->data ),
@@ -1651,7 +1711,7 @@ static void screem_tag_tree_finalize( GObject *object )
 
 	g_free( priv );
 	
-	G_OBJECT_CLASS( parent_class )->finalize( object );
+	G_OBJECT_CLASS( screem_tag_tree_parent_class )->finalize( object );
 
 	trees = g_slist_remove( trees, tree );
 	last = ( trees == NULL );
@@ -1663,13 +1723,11 @@ static void screem_tag_tree_finalize( GObject *object )
 		}
 	}
 	
-	g_mutex_unlock( mutex );
+	g_static_mutex_unlock( &mutex );
 
 	if( last ) {
-		g_mutex_free( mutex );
 		g_free( configpath );
 		xmlFreeDoc( config );
-		mutex = NULL;
 	}
 }
 
@@ -1686,22 +1744,23 @@ static void screem_tag_tree_set_prop( GObject *object, guint property_id,
 		tree->private->window = g_value_get_pointer( value );
 		break;
 	case ARG_PAGE:
-		page = tree->private->page;
-		if( page ) {
-			g_signal_handlers_disconnect_matched( G_OBJECT( page ),
-							      G_SIGNAL_MATCH_DATA,
-							      0, 0, NULL, NULL, tree );
-		}
 		page = g_value_get_object( value );
+		if( page != tree->private->page ) {
+			if( tree->private->page ) {
+				g_signal_handlers_disconnect_matched( G_OBJECT( tree->private->page ),
+								      G_SIGNAL_MATCH_DATA,
+								      0, 0, NULL, NULL, tree );
+			}
 
-		tree->private->page = page;
-		if( tree->private->timeout ) {
-			g_source_remove( tree->private->timeout );
-			tree->private->timeout = 0;
+			tree->private->page = page;
+			if( tree->private->timeout ) {
+				g_source_remove( tree->private->timeout );
+				tree->private->timeout = 0;
+			}
+			tree->private->timeout = g_timeout_add( 2000, 
+					(GSourceFunc)screem_tag_tree_page_set, 
+					tree );
 		}
-		tree->private->timeout = g_timeout_add( 2000, 
-				(GSourceFunc)screem_tag_tree_page_set, 
-				tree );
 		break;
 	}
 }
@@ -1761,27 +1820,3 @@ static void screem_tag_tree_size_allocate( GtkWidget *widget,
 	}
 }
 
-GType screem_tag_tree_get_type()
-{
-	static GType type = 0;
-	
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemTagTreeClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_tag_tree_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemTagTree ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_tag_tree_init
-		};
-
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemTagTree",
-					       &info, 0 );
-	}
-
-	return type;
-}
