@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -28,7 +28,6 @@
 #include <unistd.h>
 
 #include <glib/gi18n.h>
-#include <libgnome/gnome-util.h>
 #include <libgnome/gnome-exec.h>
 
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -76,59 +75,6 @@ void change_visible( GtkWidget *widget, gpointer data )
 	}
 }
 
-gchar *convert_keysym_state_to_string( guint keysym, guint state )
-{
-        gchar *key;
-	GString *str;
-
-        if( keysym == GDK_VoidSymbol || keysym == GDK_BackSpace ||
-	    keysym == GDK_Delete || keysym == 0 ) {
-                return NULL;
-	}
-
-        gdk_error_trap_push();
-        key = gdk_keyval_name( keysym );
-        gdk_error_trap_pop();
-        if( ! key )
-		return NULL;
-
-	str = g_string_new( NULL );
-
-        if( state & GDK_CONTROL_MASK ) {
-		g_string_append( str, "Control-" );
-	}
-        if( state & GDK_LOCK_MASK ) {
-		g_string_append( str, "Lock-" );
-        }
-        if( state & GDK_SHIFT_MASK ) {
-		g_string_append( str, "Shift-" );
-        }
-        if( state & GDK_MOD1_MASK ) {
-		g_string_append( str, "Mod1-" );
-        }
-        if( state & GDK_MOD2_MASK ) {
-		g_string_append( str, "Mod2-" );
-        }
-        if( state & GDK_MOD3_MASK ) {
-		g_string_append( str, "Mod3-" );
-        }
-        if( state & GDK_MOD4_MASK ) {
-		g_string_append( str, "Mod4-" );
-        }
-        if( state & GDK_MOD5_MASK ) {
-		g_string_append( str, "Mod5-" );
-        }
-
-	if( str->len ) {
-		g_string_append( str, key );
-	}
-
-	key = str->str;
-	g_string_free( str, FALSE );
-
-	return key;
-}
-
 gchar *screem_support_ctags( const gchar *dirname,
 			     const gchar *word,
 			     gchar **pattern )
@@ -190,7 +136,8 @@ gchar *screem_support_escape_regexp( const gchar *pattern )
 	g_return_val_if_fail( pattern != NULL, NULL );
 	g_return_val_if_fail( *pattern == '/', NULL );
 
-	temp = g_string_new( NULL );
+	/* will need at least strlen( pattern ) */
+	temp = g_string_sized_new( strlen( pattern ) );
 	
 	pattern ++;
 	while( *pattern != '\0' &&
@@ -227,45 +174,166 @@ gchar *screem_support_escape_regexp( const gchar *pattern )
 	return ret;
 }
 
-
-GtkTreeIter *screem_support_find_in_list( GtkListStore *store, 
-		guint column, const gchar *str )
+gboolean screem_gtk_list_store_find( GtkListStore *store,
+		GtkTreeIter *iter, guint col, GValue *value,
+		GCompareFunc compare )
 {
-	GtkTreeIter it;
-	GtkTreeIter *ret;
-	gchar *val;
-	
-	ret = NULL;
+	GtkTreeModel *model;
+	GValue mvalue = { 0 };
+	gboolean ret;
 
-	g_assert( gtk_tree_model_get_column_type( GTK_TREE_MODEL( store ),
-						  column ) == G_TYPE_STRING );
+	g_return_val_if_fail( iter != NULL, FALSE );
+	g_return_val_if_fail( value != NULL, FALSE );
+	g_return_val_if_fail( compare != NULL, FALSE );
 	
-	if( gtk_tree_model_get_iter_first( GTK_TREE_MODEL( store ), &it ) ) {
-		do {
-			gtk_tree_model_get( GTK_TREE_MODEL( store ), &it, 
-					    column, &val, -1 );
+	model = GTK_TREE_MODEL( store );
 
-			if( val && ! strncmp( "regex:", val, strlen( "regex:" ) ) )  {
-				/* regexp pattern */
-				g_warning( "regex: pattern match not supported yet\n" );
-			} else if( val && ! strncmp( "glob:", val, strlen( "glob:" ) ) ) {
-				/* glob */
-				if( fnmatch( val + strlen( "glob:" ), str, 0 ) == 0 ) {
-					ret = gtk_tree_iter_copy( &it );
-				}
-			} else 	if( val && ! strcmp( val, str ) ) {
-				/* string pattern */
-				
-				ret = gtk_tree_iter_copy( &it );
-			}
-			g_free( val );
-		} while( ( ! ret ) && 
-			 gtk_tree_model_iter_next( GTK_TREE_MODEL( store ),
-				 		   &it ) );
+	g_return_val_if_fail( gtk_tree_model_get_column_type( model,
+			col ) == G_VALUE_TYPE( value ), FALSE );
+
+	ret = gtk_tree_model_get_iter_first( model, iter );
+	while( ret ) {
+		gtk_tree_model_get_value( model, iter, col, &mvalue );
+		if( ! compare( value, &mvalue ) ) {
+			g_value_unset( &mvalue );
+			break;
+		}
+		g_value_unset( &mvalue );
+
+		ret = gtk_tree_model_iter_next( model, iter );
+	}
+
+	return ret;
+}
+
+static gint screem_g_type_string_compare( GValue *a, GValue *b )
+{
+	const gchar *astr;
+	const gchar *bstr;
+	gint ret;
+	
+	g_assert( G_VALUE_TYPE( a ) == G_TYPE_STRING );
+	g_assert( G_VALUE_TYPE( b ) == G_TYPE_STRING );
+
+	astr = g_value_get_string( a );
+	bstr = g_value_get_string( b );
+
+	if( astr && bstr ) {
+		ret = strcmp( astr, bstr );
+	} else {
+		ret = ( astr - bstr );
 	}
 	
 	return ret;
 }
+gboolean screem_gtk_list_store_find_string( GtkListStore *store,
+		GtkTreeIter *iter, guint col, const gchar *value )
+
+{
+	GValue val = { 0 };
+	gboolean ret;
+
+	g_value_init( &val, G_TYPE_STRING );
+	g_value_set_string( &val, value );
+
+	ret = screem_gtk_list_store_find( store, iter, col,
+			&val, 
+			(GCompareFunc)screem_g_type_string_compare );
+	g_value_unset( &val );
+	
+	return ret;
+}
+
+static gint find_pattern( GValue *a, GValue *b )
+{
+	const gchar *astr;
+	const gchar *bstr;
+	gint ret;
+
+	g_assert( G_VALUE_TYPE( a ) == G_TYPE_STRING );
+	g_assert( G_VALUE_TYPE( b ) == G_TYPE_STRING );
+
+	astr = g_value_get_string( a );
+	bstr = g_value_get_string( b );
+
+	g_assert( astr );
+	g_assert( bstr );
+	
+	if( ! strncmp( "regex:", bstr, strlen( "regex:" ) ) )  {
+		/* regexp pattern */
+		g_warning( "regex: pattern match not supported yet\n" );
+		ret = -1;
+	} else if( ! strncmp( "glob:", bstr, strlen( "glob:" ) ) ) {
+		/* glob */
+		ret = fnmatch( bstr + strlen( "glob:" ), astr, 0 );
+	} else 	{
+		ret = strcmp( bstr, astr );
+	}
+
+	return ret;
+}
+gboolean screem_gtk_list_store_find_pattern( GtkListStore *store,
+		GtkTreeIter *iter, guint col, const gchar *value )
+
+{
+	GValue val = { 0 };
+	gboolean ret;
+	
+	g_value_init( &val, G_TYPE_STRING );
+	g_value_set_string( &val, value );
+
+	ret = screem_gtk_list_store_find( store, iter, col,
+			&val, (GCompareFunc)find_pattern );
+	g_value_unset( &val );
+	
+	return ret;
+}
+
+static gint screem_mime_type_compare_types( GValue *a, GValue *b )
+{
+	const gchar *astr;
+	const gchar *bstr;
+	gint ret;
+	GnomeVFSMimeEquivalence eq;
+
+	g_assert( G_VALUE_TYPE( a ) == G_TYPE_STRING );
+	g_assert( G_VALUE_TYPE( b ) == G_TYPE_STRING );
+
+	astr = g_value_get_string( a );
+	bstr = g_value_get_string( b );
+
+	g_assert( astr );
+	g_assert( bstr );
+
+	eq = gnome_vfs_mime_type_get_equivalence( astr, bstr );
+	
+	if( eq == GNOME_VFS_MIME_IDENTICAL ) {
+		ret = 0;
+	} else {
+		ret = strcmp( astr, bstr );
+	}
+
+	return ret;
+}
+
+gboolean screem_gtk_list_store_find_mime_type( GtkListStore *store,
+		GtkTreeIter *iter, guint col, const gchar *value )
+
+{
+	GValue val = { 0 };
+	gboolean ret;
+
+	g_value_init( &val, G_TYPE_STRING );
+	g_value_set_string( &val, value );
+
+	ret = screem_gtk_list_store_find( store, iter, col,
+			&val, 
+			(GCompareFunc)screem_mime_type_compare_types );
+	g_value_unset( &val );
+	
+	return ret;
+}
+
 
 gchar *screem_support_charset_convert( const gchar *data, guint len,
 		gchar **data_charset )
@@ -318,7 +386,7 @@ gchar *screem_support_charset_convert( const gchar *data, guint len,
 			}
 		}
 		
-	} else if( ! g_utf8_validate( data, strlen( data ), &end ) ) {
+	} else if( ! g_utf8_validate( data, len, &end ) ) {
 		gchar *temp = NULL;
 		const gchar *charset;
 		gboolean utf8;
@@ -358,7 +426,7 @@ gchar *screem_support_charset_convert( const gchar *data, guint len,
 
 		ret = temp;
 	} else  {
-		ret = g_strdup( data );
+		ret = g_strndup( data, len );
 		if( data_charset ) {
 			*data_charset = g_strdup( "UTF-8" );
 		}
@@ -375,15 +443,17 @@ gchar *screem_support_charset_convert_to( const gchar *data,
 	gchar *ret;
 
 	ret = NULL;
+
+	g_return_val_if_fail( from != NULL, NULL );
+	g_return_val_if_fail( charset != NULL, NULL );
 	
-	if( from && charset && g_strcasecmp( from, charset ) ) {
+	if( g_strcasecmp( from, charset ) ) {
 		/* need to convert from UTF-8 to charset */
 		gchar *temp;
-		guint len;
-		
+	
 		temp = g_convert( data, strlen( data ), charset, 
 				  from, 
-				  NULL, &len, NULL );
+				  NULL, NULL, NULL );
 
 		ret = temp;
 	}
@@ -393,7 +463,8 @@ gchar *screem_support_charset_convert_to( const gchar *data,
 
 /* HACK due to the huge annoyance of gnome-vfs mime data
  * determining that anything starting <?xml  is text/xml
- * no matter what any *.mime files say about the extension */
+ * no matter what any *.mime files say about the extension,
+ * likewise php files can be identified as html */
 gchar *screem_get_mime_type( const gchar *filename, gboolean fast )
 {
 	GnomeVFSFileInfo *info;
@@ -447,6 +518,10 @@ gchar *screem_get_mime_type_override( const gchar *filename,
 			ret = g_strdup( "text/vnd.wap.wmlc" );
 		} else if( ! strcmp( ".wmlsc", ext ) ) {
 			ret = g_strdup( "text/vnd.wap.wmlscriptc" );
+		} else if( ! strcmp( ".php", ext ) ) {
+			ret = g_strdup( "application/x-php" );
+		} else if( ! strcmp( ".phps", ext ) ) {
+			ret = g_strdup( "application/x-php-souce" );
 		} else {
 			ret = g_strdup( type );
 		}
@@ -457,20 +532,17 @@ gchar *screem_get_mime_type_override( const gchar *filename,
 	return ret;
 }
 
-const gchar *g_utf8_skip_space( const gchar *txt )
+const gchar *screem_utf8_skip_space( const gchar *txt )
 {
 	gunichar c;
 
 	g_return_val_if_fail( txt != NULL, NULL );
-	
-	while( *txt ) {
-		c = g_utf8_get_char( txt );
-		if( ! g_unichar_isspace( c ) ) {
-			break;
-		}
-		txt = g_utf8_next_char( txt );
-	}
 
+	do {
+		c = g_utf8_get_char( txt );
+	} while( c != '\0' && g_unichar_isspace( c ) &&
+		( txt = g_utf8_next_char( txt ) ) );
+	
 	return txt;
 }
 
@@ -479,20 +551,15 @@ const gchar *screem_utf8_skip_to_space( const gchar *txt )
 	gunichar c;
 
 	g_return_val_if_fail( txt != NULL, NULL );
-	
-	while( *txt ) {
-		c = g_utf8_get_char( txt );
-		if( g_unichar_isspace( c ) ) {
-			break;
-		}
-		txt = g_utf8_next_char( txt );
-	}
 
+	do {
+		c = g_utf8_get_char( txt );
+	} while( c != '\0' && ! g_unichar_isspace( c ) &&
+		( txt = g_utf8_next_char( txt ) ) );
+	
 	return txt;
 }
 
-
-
 /* ripped out of gedit2 */
 gchar* screem_escape_underlines( const gchar* text )
 {
@@ -505,7 +572,9 @@ gchar* screem_escape_underlines( const gchar* text )
 
     	length = strlen( text );
 
-	str = g_string_new( NULL );
+	/* result will be at least length in size, so alloc
+	 * that much space to start with */
+	str = g_string_sized_new( length );
 
   	p = text;
   	end = text + length;
@@ -586,7 +655,7 @@ void g_string_append_utf8_len( GString *str, const gchar *utf8,
 GtkPositionType screem_get_tab_position( void )
 {
 	GConfClient *client;
-	static GConfEnumStringPair positions[] = {
+	static const GConfEnumStringPair positions[] = {
 		{ GTK_POS_LEFT, "left" },
 		{ GTK_POS_RIGHT, "right" },
 		{ GTK_POS_TOP, "top" },
@@ -711,11 +780,19 @@ static gboolean filter_mime_inherits( const GtkFileFilterInfo *info,
 	gboolean ret;
 
 	ret = FALSE;
+
+	if( ! data ) {
+		g_warning( "Hmm, NULL path given to filter_mime_inherits()\n" );
+		return FALSE;
+	}
 	
 	if( info->contains & GTK_FILE_FILTER_MIME_TYPE ) {
-		ret = ! strcmp( (gchar*)data, info->mime_type );
-		if( ! ret ) {
-			ret = ( gnome_vfs_mime_type_get_equivalence( info->mime_type, (gchar*)data ) == GNOME_VFS_MIME_PARENT );
+		if( info->mime_type ) {
+			ret = ! strcmp( (gchar*)data, info->mime_type );			if( ! ret ) {
+				ret = ( gnome_vfs_mime_type_get_equivalence( info->mime_type, (gchar*)data ) == GNOME_VFS_MIME_PARENT );
+			}
+		} else {
+			g_warning( "NULL mime type in mime type filter\n" );
 		}
 	}
 
@@ -730,8 +807,12 @@ GtkFileFilter *screem_get_file_filter( const gchar *name )
 	GHashTable *feature;
 	GtkFileFilter *filter = NULL;
 
-	/* cleanup mode */
 	if( ! name ) {
+		return NULL;
+	}
+	
+	/* cleanup mode */
+	if( (void*)name == (void*)0x1 ) {
 		if( filters ) {
 			g_hash_table_destroy( filters );
 			filters = NULL;
@@ -811,7 +892,7 @@ GtkFileFilter *screem_get_file_filter( const gchar *name )
 /* override a GnomeFileEntry widgets browser button, horrible
  * hack but it gets us the ability to place / size the dialog +
  * use non local volumes */
-void screem_file_entry_browse( GnomeFileEntry *entry, gpointer data )
+void screem_file_entry_browse( GtkWidget *entry, gpointer data )
 {
 	gchar *title;
 	gchar *dir;
@@ -845,7 +926,7 @@ void screem_file_entry_browse( GnomeFileEntry *entry, gpointer data )
 		
 	}
 	
-	filename = screem_fileselect( title, NULL, action, dir, NULL ); 
+	filename = screem_fileselect( title, NULL, action, dir, NULL, FALSE ); 
 
 	if( filename ) {
 		gtk_entry_set_text( GTK_ENTRY( widget ), filename );
@@ -857,30 +938,6 @@ void screem_file_entry_browse( GnomeFileEntry *entry, gpointer data )
 	g_object_unref( G_OBJECT( widget ) );
 }
 
-gchar *screem_get_glade_path( void )
-{
-	GConfClient *client;
-	gchar *glade_path;
-	
-	client = gconf_client_get_default();
-	glade_path = gconf_client_get_string( client,
-			"/apps/screem/general/glade_path", NULL );
-	/* check for existance as well, if screem has been installed
-	 * into another prefix and the old version erased the gconf
-	 * key will point to the wrong place */
-	if( ( ! glade_path ) || ! uri_exists( glade_path, NULL ) ) {
-		glade_path = g_strconcat( GLADE_PATH, "/screem.glade", 
-				NULL );
-		gconf_client_set_string( client,
-				"/apps/screem/general/glade_path",
-				glade_path, NULL );
-	}
-
-	g_object_unref( client );
-
-	return glade_path;
-}
-
 static gboolean find_accel( GtkAccelKey *key, GClosure *closure,
 		GdkEventKey *event )
 {
@@ -919,54 +976,16 @@ gint screem_mime_type_compare( gconstpointer a, gconstpointer b )
 	return g_strcasecmp( showa, showb );
 }
 
-void screem_hig_alert( const gchar *stock_id,
+void screem_hig_alert( GtkMessageType type, GtkButtonsType buttons,
 		const gchar *primary, const gchar *secondary,
 		GtkWidget *parent )
 {
 	GtkWidget *widget;
-	GtkWidget *hbox;
-	GtkWidget *icon;
-	
-	GtkWidget *label;
-	gchar *txt;
-	gchar *labeltxt;
 
-	widget = gtk_dialog_new();
-	gtk_dialog_set_has_separator( GTK_DIALOG( widget ), FALSE );
-	gtk_window_set_role( GTK_WINDOW( widget ), 
-			"screem_startup_error" );
-	gtk_container_set_border_width( GTK_CONTAINER( widget ), 6 );
-	
-	gtk_box_set_spacing( GTK_BOX( GTK_DIALOG( widget )->vbox ), 
-			12 );
-	
-	hbox = gtk_hbox_new( FALSE, 0 );
-	gtk_container_set_border_width( GTK_CONTAINER( hbox ), 6 );
-	gtk_box_set_spacing( GTK_BOX( hbox ), 12 );
-		
-	gtk_container_add( GTK_CONTAINER( GTK_DIALOG( widget )->vbox ),
-			hbox );
-
-	gtk_dialog_add_button( GTK_DIALOG( widget ),
-			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
-	
-	txt = g_strconcat( "<span weight=\"bold\" size=\"larger\">",
-			primary, "</span>", NULL );
-	label = gtk_label_new( "" );
-	labeltxt = g_strconcat( txt, "\n\n", secondary, NULL );
-	gtk_label_set_markup( GTK_LABEL( label ), labeltxt );
-	g_free( txt );
-	g_free( labeltxt );
-	gtk_label_set_selectable( GTK_LABEL( label ), TRUE );
-
-	icon = gtk_image_new_from_stock( GTK_STOCK_DIALOG_ERROR,
-			GTK_ICON_SIZE_DIALOG );
-	gtk_misc_set_alignment( GTK_MISC( icon ), 0.0, 0.0 );
-	gtk_box_pack_start_defaults( GTK_BOX( hbox ), icon );
-	
-	gtk_label_set_line_wrap( GTK_LABEL( label ), TRUE );
-	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.0 );
-	gtk_box_pack_start_defaults( GTK_BOX( hbox ), label );
+	widget = gtk_message_dialog_new( GTK_WINDOW( parent ), 
+			GTK_DIALOG_MODAL,
+			type, buttons, primary );
+	gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( widget ), secondary );
 	
 	
 	gtk_widget_show_all( widget );
@@ -1010,15 +1029,11 @@ GtkListStore *screem_support_get_mime_types_store( void )
 	types = g_slist_sort( types, (GCompareFunc)screem_mime_type_compare );
 	for( langs = types; langs; langs = langs->next ) {
 		const gchar *showtype;
-		GtkTreeIter *fit;
 
 		showtype = gnome_vfs_mime_get_description( langs->data );
-		if( ! showtype ) {
-			showtype = langs->data;
-		}
-			
-		fit = screem_support_find_in_list( GTK_LIST_STORE( list ), 1, langs->data );
-		if( ! fit ) {
+		if( showtype &&
+		    ! screem_gtk_list_store_find_mime_type( GTK_LIST_STORE( list ), &it, 1, langs->data ) &&
+		    ! screem_gtk_list_store_find_string( GTK_LIST_STORE( list ), &it, 0, showtype ) ) {
 			gtk_list_store_append( GTK_LIST_STORE( list ),
 					&it );
 			gtk_list_store_set( GTK_LIST_STORE( list ), 
@@ -1026,29 +1041,35 @@ GtkListStore *screem_support_get_mime_types_store( void )
 					0, showtype,
 					1, langs->data,
 					-1 );
-		} else {
-			gtk_tree_iter_free( fit );
-		}
+		} 
 	}
 
 	return GTK_LIST_STORE( list );
 }
 
-gchar *screem_gdk_color_to_string( GdkColor *colour )
+gchar *screem_gdk_color_to_string( const GdkColor *colour, 
+		gboolean eightbit )
 {
 	guint16 r;
 	guint16 g;
 	guint16 b;
-		
+	gchar *ret;
+	
 	r = colour->red;
 	g = colour->green;
 	b = colour->blue;
 
-	r = r >> 8;
-	g = g >> 8;
-	b = b >> 8;
+	if( eightbit ) {
+		r = r >> 8;
+		g = g >> 8;
+		b = b >> 8;
 	
-	return g_strdup_printf( "#%.2x%.2x%.2x", r, g, b );
+		ret = g_strdup_printf( "#%.2x%.2x%.2x", r, g, b );
+	} else {
+		ret = g_strdup_printf( "#%04x%04x%04x", r, g, b );
+	}
+
+	return ret;
 }
 
 gchar* screem_escape_char( const gchar* text, gchar c )
@@ -1057,12 +1078,13 @@ gchar* screem_escape_char( const gchar* text, gchar c )
 	gint length;
 	const gchar *p;
  	const gchar *end;
-
+	
   	g_return_val_if_fail( text != NULL, NULL );
 
     	length = strlen( text );
 
-	str = g_string_new( NULL );
+	/* will need at least strlen( text ) */
+	str = g_string_sized_new( strlen( text ) );
 
   	p = text;
   	end = text + length;
@@ -1082,3 +1104,242 @@ gchar* screem_escape_char( const gchar* text, gchar c )
 	return g_string_free( str, FALSE );
 }
 
+void screem_gtk_add_history( GtkWidget *widget )
+{
+	const gchar *name;
+	GtkTreeModel *model;
+	const gchar *gconf_base = "/apps/gnome-settings/screem/";
+	gchar *path;
+	GConfClient *client;
+	GSList *values;
+	GSList *tmp;
+	GtkTreeIter it;
+
+	GtkEntryCompletion *completion;
+	
+	name = gtk_widget_get_name( widget );
+
+	model = NULL;
+	if( GTK_IS_COMBO_BOX_ENTRY( widget ) ) {
+		model = gtk_combo_box_get_model( GTK_COMBO_BOX( widget ) );
+
+		if( model ) {
+			return;
+		}
+	
+		/* create a new model */
+		model = GTK_TREE_MODEL( gtk_list_store_new( 1, G_TYPE_STRING ) );
+		gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), model );
+		
+		/* get history */
+		client = gconf_client_get_default();
+		path = g_strconcat( gconf_base, "history-", name, NULL );
+		values = gconf_client_get_list( client, path, 
+				GCONF_VALUE_STRING, NULL );
+		
+		/* add to model */
+		for( tmp = values; tmp; tmp = tmp->next ) {
+			gtk_list_store_append( GTK_LIST_STORE( model ), &it );
+			gtk_list_store_set( GTK_LIST_STORE( model ), &it,
+					0, tmp->data, -1  );
+			g_free( tmp->data );
+		}
+	
+		g_slist_free( values );	
+		g_free( path );
+		g_object_unref( client );
+
+		gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ), 0 );
+	
+		/* get GtkEntry so we can add a GtkEntryCompletion,
+		 * and listen for changes so we can add new
+		 * items to the model if needed */
+		widget = GTK_BIN( widget )->child;
+	}
+	if( GTK_IS_ENTRY( widget ) ) {
+		if( ! model ) {
+			model = GTK_TREE_MODEL( gtk_list_store_new( 1, G_TYPE_STRING ) );
+		}
+		completion = gtk_entry_completion_new();
+		gtk_entry_completion_set_model( completion, model );
+		gtk_entry_completion_set_text_column( completion, 0 );
+		gtk_entry_set_completion( GTK_ENTRY( widget ), completion );
+		g_object_unref( completion );
+		
+		g_object_unref( model );
+	}
+}
+
+/*
+ * string1 = title, string2 = "filter,filter,filter"
+ * int1 = GtkFileChooserAction value
+ * int2 = 1 if we want a preview
+ * 
+ */
+GtkWidget *screem_create_gtk_file_chooser_button( gchar *name, 
+		const gchar *title, const gchar *filters_str, 
+		gint action, gint preview )
+{
+	GtkWidget *dialog;
+	GtkWidget *button;
+	GtkFileChooserAction chooser_action;
+	gchar **filter_names;
+	gchar **tmp;
+	GSList *filters;
+	GtkFileFilter *filter;
+	
+	switch( action ) {
+		case 3:
+			chooser_action = GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER;
+			break;
+		case 2:
+			chooser_action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+			break;
+		case 1:
+			chooser_action = GTK_FILE_CHOOSER_ACTION_SAVE;
+			break;
+		case 0:
+		default:
+			chooser_action = GTK_FILE_CHOOSER_ACTION_OPEN;
+			break;
+	}
+	filter_names = NULL;
+	filters = NULL;
+	if( filters_str ) {
+		filter_names = g_strsplit( filters_str, ",", 0 );
+	}
+	for( tmp = filter_names; tmp && *tmp; tmp ++ ) {
+		g_strstrip( *tmp );
+		filter = screem_get_file_filter( gettext( *tmp ) );
+		filters = g_slist_prepend( filters, (gpointer)filter );
+	}
+	g_strfreev( filter_names );
+	
+	dialog = screem_file_dialog_create( title,
+			NULL, chooser_action, FALSE, NULL, filters, preview );
+	g_slist_free( filters );
+
+	g_signal_connect( G_OBJECT( dialog ), "show",
+			G_CALLBACK( screem_gtk_file_chooser_button_show ), 
+			NULL );
+	g_signal_connect( G_OBJECT( dialog ), "response",
+			G_CALLBACK( screem_gtk_file_chooser_button_response ), 
+			NULL );
+	
+	button = gtk_file_chooser_button_new_with_dialog( dialog );
+	gtk_file_chooser_set_action( GTK_FILE_CHOOSER( button ),
+			chooser_action );
+	gtk_widget_set_name( button, name );
+
+	gtk_widget_show_all( button );
+	
+	return button;
+}
+
+
+/* modified from gnome-desktop-item.c */
+gchar *screem_lookup_icon( GtkIconTheme *icon_theme, const gchar *icon, 
+		gint desired_size, gint flags )
+{
+	GtkIconInfo *info;
+	gchar *full = NULL;
+	
+	if( icon == NULL || *icon == '\0' ) {
+		
+		return NULL;
+		
+	} else if( g_path_is_absolute( icon ) ) {
+		
+		if( g_file_test( icon, G_FILE_TEST_EXISTS ) ) {
+			
+			return g_strdup( icon );
+			
+		} else {
+			return NULL;
+		}
+	} else {
+		gchar *icon_no_extension;
+		gchar *p;
+
+		if( icon_theme == NULL ) {
+			icon_theme = gtk_icon_theme_get_default();
+		} else {
+			g_object_ref( icon_theme );
+		}
+
+		icon_no_extension = g_strdup( icon );
+		p = strrchr( icon_no_extension, '.' );
+		if( p &&
+		    ( strcmp (p, ".png" ) == 0 ||
+		      strcmp (p, ".xpm" ) == 0 ||
+		      strcmp (p, ".svg" ) == 0 ) ) {
+		    *p = 0;
+		}
+
+		info = gtk_icon_theme_lookup_icon( icon_theme,
+				icon_no_extension,
+				desired_size, flags );
+		if( info ) {
+			full = (gchar*)gtk_icon_info_get_filename( info );
+			if( full ) {
+				full = g_strdup( full );
+			}
+			gtk_icon_info_free( info );
+		}
+		
+		g_object_unref( icon_theme );
+		
+		g_free( icon_no_extension );
+	}
+
+	if( ! full ) { 
+		/* Fall back on old Gnome code */
+		full = gnome_program_locate_file( NULL,
+				GNOME_FILE_DOMAIN_PIXMAP, icon,
+				TRUE /* only_if_exists */,
+				NULL /* ret_locations */);
+		
+		if( ! full ) {
+			full = gnome_program_locate_file (NULL,
+					GNOME_FILE_DOMAIN_APP_PIXMAP,
+					icon,
+					TRUE /* only_if_exists */,
+					NULL /* ret_locations */);
+		}
+	}
+
+	return full;    
+}
+
+gchar *screem_strip_extra_space( const gchar *str )
+{
+	gchar *ret;
+	GString *tmp;
+	gunichar c;
+		
+	g_return_val_if_fail( str != NULL, NULL );
+		
+	/* this will be too long if we do strip space, but
+	 * avoids reallocs */
+	tmp = g_string_sized_new( strlen( str ) );
+	c = g_utf8_get_char( str );
+	while( c != '\0' ) {
+		if( c == '\t' || c == '\r' || c == '\n' ) {
+			c = ' ';
+		}
+		g_string_append_unichar( tmp, c );
+		if( g_unichar_isspace( c ) ) {
+			str = screem_utf8_skip_space( str );
+		} else {
+			str = g_utf8_next_char( str );
+		}
+		c = g_utf8_get_char( str );
+	}
+	
+	ret = tmp->str;
+	g_string_free( tmp, FALSE );
+	
+	return ret;
+}
+
+
